@@ -7,7 +7,7 @@
  *  @brief Main defines for the USB devices.
  *
  * Based on sources acquired from Plustek Inc.<br>
- * Copyright (C) 2001-2004 Gerhard Jaeger <gerhard@gjaeger.de>
+ * Copyright (C) 2001-2005 Gerhard Jaeger <gerhard@gjaeger.de>
  *
  * History:
  * - 0.40 - starting version of the USB support
@@ -30,6 +30,16 @@
  *        - added additional defines for cis and epson-ccd sensor
  * - 0.47 - cleanup work
  *        - added gamma to struct HWDefault
+ * - 0.48 - added DEVCAPSFLAG_LargeTPA
+ *        - added _WAF_BIN_FROM_COLOR and _WAF_GRAY_FROM_COLOR
+ *        - added dHighSpeed to struct HwDefault
+ * - 0.49 - added a_bRegs, fModFirstHome and fLastScanIsAdf
+ *          to struct DeviceDef
+ *        - added CRYSTAL_FREQ
+ *        - added IPCDef
+ * - 0.50 - cleanup
+ *        - removed obsolete _WAF_BLACKFINE
+ *        - added MODEL_CANON_LIDE25
  * .
  * <hr>
  * This file is part of the SANE package.
@@ -74,102 +84,105 @@
 #ifndef __PLUSTEK_USB_H__
 #define __PLUSTEK_USB_H__
 
-/** CCD ID (PCB ID): total 3 bits */
-#define	kNEC3799	0
-#define kSONY518	1
-#define	kSONY548	2
-#define	kNEC8861	3
-#define	kNEC3778	4
-#define	kNECSLIM	5
-#define	kCIS650 	6
-#define	kCIS670 	7
-#define	kCIS1220	8
-#define	kCIS1240	9
-#define	kEPSON  	10
+/** CCD ID (PCB ID): total 3 bits (on Plustek devices) */
+#define kNEC3799    0
+#define kSONY518    1
+#define kSONY548    2
+#define kNEC8861    3
+#define kNEC3778    4
+#define kNECSLIM    5
+#define kCIS650     6
+#define kCIS670     7
+#define kCIS1220    8
+#define kCIS1240    9
+#define kEPSON     10
+
+/** 48MHz Quartz */
+#define CRYSTAL_FREQ 48000000UL
 
 /*********************************** plustek_types.h!!! ************************/
 
 /* makes trouble with gcc3
-#define _SWAP(x,y)	(x)^=(y)^=(x)^=(y)
+#define _SWAP(x,y)  (x)^=(y)^=(x)^=(y)
 */
-#define _SWAP(x,y)	{(x)^=(y); (x)^=((y)^=(x));}
+#define _SWAP(x,y)   { (x)^=(y); (x)^=((y)^=(x));}
 
-#define _LOWORD(x)	((u_short)(x & 0xffff))
-#define _HIWORD(x)	((u_short)(x >> 16))
+#define _LOWORD(x)  ((u_short)(x & 0xffff))
+#define _HIWORD(x)  ((u_short)(x >> 16))
 #define _LOBYTE(x)  ((u_char)((x) & 0xFF))
 #define _HIBYTE(x)  ((u_char)((x) >> 8))
 
-#define _HILO2WORD(x)	((u_short)x.bHi * 256U + x.bLo)
-#define _PHILO2WORD(x)	((u_short)x->bHi * 256U + x->bLo)
+#define _HILO2WORD(x)   ((u_short)x.bHi * 256U + x.bLo)
+#define _PHILO2WORD(x)  ((u_short)x->bHi * 256U + x->bLo)
 
 /* useful for RGB-values */
 typedef struct {
 	u_char Red;
 	u_char Green;
 	u_char Blue;
-} RGBByteDef, *pRGBByteDef;
+} RGBByteDef;
 
 typedef struct {
 	u_short Red;
 	u_short Green;
 	u_short Blue;
-} RGBUShortDef, *pRGBUShortDef;
+} RGBUShortDef;
 
 typedef struct {
 	u_long Red;
 	u_long Green;
 	u_long Blue;
-} RGBULongDef, *pRGBULongDef;
+} RGBULongDef;
 
 typedef struct {
 	u_char a_bColor[3];
-} ColorByteDef, *pColorByteDef;
+} ColorByteDef;
 
 typedef struct {
 	u_char bHi;
 	u_char bLo;
-} HiLoDef, *pHiLoDef;
+} HiLoDef;
 
 typedef union {
 	HiLoDef HiLo[3];
-	u_short	Colors[3];
-} ColorWordDef, *pColorWordDef;
+	u_short Colors[3];
+} ColorWordDef;
 
 typedef union {
-	HiLoDef	HiLo;
-	u_short	Mono;
-} MonoWordDef, *pMonoWordDef;
+	HiLoDef HiLo;
+	u_short Mono;
+} MonoWordDef;
 
 typedef union {
 
-	u_char	     *pb;
+	u_char       *pb;
 	u_short      *pw;
-	pMonoWordDef  pmw;
-	pColorByteDef pcb;
-	pColorWordDef pcw;
-	pRGBByteDef   pb_rgb;
-	pRGBUShortDef pw_rgb;
-	pHiLoDef	  philo;
+	MonoWordDef  *pmw;
+	ColorByteDef *pcb;
+	ColorWordDef *pcw;
+	RGBByteDef   *pb_rgb;
+	RGBUShortDef *pw_rgb;
+	HiLoDef      *philo;
 
-} AnyPtr, *pAnyPtr;
+} AnyPtr;
 
 typedef struct {
 	unsigned short x;
 	unsigned short y;
-} XY, *pXY;
+} XY;
 
-#define _VAR_NOT_USED(x)	((x)=(x))
+#define _VAR_NOT_USED(x) ((x)=(x))
 
 /*****************************************************************************/
 
-#define IDEAL_GainNormal				0xf000UL		/* 240 */
-#define IDEAL_GainPositive				0xfe00UL		/* 254 */
-#define IDEAL_Offset					0x1000UL		/* 20  */
+#define IDEAL_GainNormal                0xf000UL        /* 240 */
+#define IDEAL_GainPositive              0xfe00UL        /* 254 */
+#define IDEAL_Offset                    0x1000UL        /* 20  */
 
-#define GAIN_Target						65535UL
+#define GAIN_Target                     65535UL
 
-#define DRAM_UsedByAsic8BitMode			216				/* in KB */
-#define DRAM_UsedByAsic16BitMode		196 /*192*/		/* in KB */
+#define DRAM_UsedByAsic8BitMode         216             /* in KB */
+#define DRAM_UsedByAsic16BitMode        196 /*192*/     /* in KB */
 
 /** Chip-types */
 typedef enum _CHIPSET
@@ -211,11 +224,12 @@ enum _SENSORCOLOR
 /** DCapsDef.wFlags */
 enum _DEVCAPSFLAG
 {
-	DEVCAPSFLAG_Normal		= 0x0001,
-	DEVCAPSFLAG_Positive	= 0x0002,
-	DEVCAPSFLAG_Negative	= 0x0004,
-	DEVCAPSFLAG_TPA			= 0x0006,
-	DEVCAPSFLAG_Adf			= 0x0008
+	DEVCAPSFLAG_Normal   = 0x0001,
+	DEVCAPSFLAG_Positive = 0x0002,
+	DEVCAPSFLAG_Negative = 0x0004,
+	DEVCAPSFLAG_TPA      = 0x0006,
+	DEVCAPSFLAG_Adf      = 0x0008,
+	DEVCAPSFLAG_LargeTPA = 0x0010
 };
 
 /** to allow some workarounds */
@@ -228,7 +242,9 @@ enum _WORKAROUNDS
 	_WAF_BYPASS_CALIBRATION = 0x00000008, /* no calibration,use linear gamma */
 	_WAF_INV_NEGATIVE_MAP   = 0x00000010, /* the backend does the neg. stuff */
 	_WAF_SKIP_FINE          = 0x00000020, /* skip the fine calbration        */
-	_WAF_SKIP_WHITEFINE     = 0x00000040  /* skip the fine white calbration  */
+	_WAF_SKIP_WHITEFINE     = 0x00000040, /* skip the fine white calbration  */
+	_WAF_BIN_FROM_COLOR     = 0x00000080, /* generate binary & gray images   */
+	_WAF_GRAY_FROM_COLOR    = 0x00000100  /* from color scans                */
 };
 
 /** for lamps connected to the misc I/O pins*/
@@ -244,13 +260,13 @@ enum _LAMPS
 };
 
 /** for encoding a misc I/O register as TPA */
-#define _TPA(register)          ((u_long)(register << 16))
+#define _TPA(register) ((u_long)(register << 16))
 
 /** Mask to check for available TPA */
-#define _HAS_TPA(flag)		(flag & 0xFFFF0000)
+#define _HAS_TPA(flag) (flag & 0xFFFF0000)
 
 /** Get the TPA misc I/O register */
-#define _GET_TPALAMP(flag)	(flag >> 16)
+#define _GET_TPALAMP(flag) (flag >> 16)
 
 /** motor types */
 typedef enum
@@ -265,6 +281,7 @@ typedef enum
 	MODEL_CANON600 ,      /**< for CanoScan 600dpi models   */
 	MODEL_CANON1200,      /**< for Canon 1200dpi models     */
 	MODEL_CANONCCD1200,   /**< for Canon CCD 1200dpi models */
+	MODEL_CANON_LIDE25,   /**< for CanoScan LiDE25          */
 	MODEL_UMAX,           /**< for UMAX 3400/3450           */
 	MODEL_UMAX1200,       /**< for UMAX 5400                */
 	MODEL_LAST
@@ -297,57 +314,62 @@ enum MODULEMOVE
 /** SCANDEF.dwFlags */
 enum SCANFLAG
 {
-	SCANFLAG_bgr			= 0x00000010,
-	SCANFLAG_BottomUp		= 0x00000020,
-	SCANFLAG_Invert			= 0x00000040,
-	SCANFLAG_DWORDBoundary	= 0x00000080,
-	SCANFLAG_RightAlign		= 0x00000100,
-	SCANFLAG_StillModule	= 0x00000200, 
-	SCANFLAG_StartScan		= 0x40000000,
-	SCANFLAG_Scanning		= 0x20000080,
-	SCANFLAG_Pseudo48		= 0x08000000,
-	SCANFLAG_SampleY		= 0x04000000
+	SCANDEF_Transparency    = 0x00000100,  /* Scanning from transparency*/
+	SCANDEF_Negative        = 0x00000200,  /* Scanning from negative    */
+	SCANDEF_QualityScan     = 0x00000400,  /* Scanning in quality mode  */
+	SCANDEF_ContinuousScan  = 0x00001000,
+	SCANDEF_Adf             = 0x00002000,  /* Scan from ADF tray        */
+
+	SCANFLAG_bgr            = 0x00004000,
+	SCANFLAG_BottomUp       = 0x00008000,
+	SCANFLAG_DWORDBoundary  = 0x00020000,
+	SCANFLAG_RightAlign     = 0x00040000,
+	SCANFLAG_StillModule    = 0x00080000, 
+	SCANFLAG_StartScan      = 0x40000000,
+	SCANFLAG_Scanning       = 0x20020000,
+	SCANFLAG_Pseudo48       = 0x08000000,
+	SCANFLAG_SampleY        = 0x04000000
 };
 
 typedef	struct Origins
 {
 	long lLeft;  /* How many pix to move the scanning org left, in optic res */
-	long lUp;	 /* How many pix to move the scanning or up, in optic res    */
-} OrgDef, *pOrgDef;
+	long lUp;    /* How many pix to move the scanning or up, in optic res    */
+} OrgDef;
 
 typedef struct SrcAttr
 {
-	XY     DataOrigin;		/**< The origin x is from visible pixel not CCD  */
-                            /*   pixel 0, in 300 DPI base.                   */
-							/*   The origin y is from visible top            */
-                            /*  (glass area), in 300 DPI                     */
-	short  ShadingOriginY;	/**< The origin y is from top of scanner body    */
+	XY     DataOrigin;      /**< The origin x is from visible pixel not CCD  */
+	                        /*   pixel 0, in 300 DPI base.                   */
+	                        /*   The origin y is from visible top            */
+	                        /*  (glass area), in 300 DPI                     */
+	short  ShadingOriginY;  /**< The origin y is from top of scanner body    */
 	short  DarkShadOrgY;    /**< if the device has a dark calibration strip  */
-	XY     Size;			/**< Scanning width/height, in 300 DPI base.     */
-	XY     MinDpi;			/**< Minimum dpi supported for scanning          */
-	u_char bMinDataType;	/**< Minimum data type supports                  */
+	XY     Size;            /**< Scanning width/height, in 300 DPI base.     */
+	XY     MinDpi;          /**< Minimum dpi supported for scanning          */
+	u_char bMinDataType;    /**< Minimum data type supports                  */
 
-} SrcAttrDef, *pSrcAttrDef;
+} SrcAttrDef;
 
 typedef struct DevCaps
 {
-	SrcAttrDef	Normal;			/**< Reflection                              */
-	SrcAttrDef	Positive;		/**< Positive film                           */
-	SrcAttrDef	Negative;		/**< Negative film                           */
-	SrcAttrDef	Adf;			/**< Adf device                              */
-	XY   		OpticDpi;		/**< Maximum DPI                             */
-	u_short		wFlags;			/**< Flag to indicate what kinds of elements */
-                                /*   are available                           */
-	u_char		bSensorOrder;	/**< CCD color sequences, see _SENSORORDER   */
-	u_char		bSensorDistance;/**< CCD Color distance                      */
-	u_char		bButtons;		/**< Number of buttons                       */
-	u_char		bCCD;			/**< CCD ID                                  */
-	u_char		bPCB;			/**< PCB ID                                  */
-	u_long		workaroundFlag;	/**< Flag to allow special work arounds, see */
+	SrcAttrDef Normal;          /**< Reflection                              */
+	SrcAttrDef Positive;        /**< Positive film                           */
+	SrcAttrDef Negative;        /**< Negative film                           */
+	SrcAttrDef Adf;             /**< Adf device                              */
+	XY         OpticDpi;        /**< Maximum DPI                             */
+	u_short    wFlags;          /**< Flag to indicate what kinds of elements */
+	                            /*   are available                           */
+	u_char     bSensorOrder;    /**< CCD color sequences, see _SENSORORDER   */
+	u_char     bSensorDistance; /**< CCD Color distance                      */
+	u_char     bButtons;        /**< Number of buttons                       */
+	u_char     bCCD;            /**< CCD ID                                  */
+	u_char     bPCB;            /**< PCB ID                                  */
+	u_long     workaroundFlag;  /**< Flag to allow special work arounds, see */
 	                            /*   _WORKAROUNDS                            */
-	u_long      lamp;           /**< for lamp: loword: normal, hiword: tpa   */
+	u_long     lamp;            /**< for lamp: loword: normal, hiword: tpa   */
 
-} DCapsDef, *pDCapsDef;
+} DCapsDef;
 
 /**
  * for keeping intial illumination settings
@@ -363,7 +385,7 @@ typedef struct
 	u_short blue_lamp_on;
 	u_short blue_lamp_off;
 
-} IllumiDef, *pIllumiDef;
+} IllumiDef;
 
 
 /** basic register settings
@@ -372,6 +394,7 @@ typedef struct HWDefault
 {
 	double              dMaxMotorSpeed;    /* Inches/second, max. scan speed */
 	double              dMaxMoveSpeed;     /* Inches/second, max. move speed */
+	double              dHighSpeed;        /* for speeding up the sensor     */
 	double              dIntegrationTimeLowLamp;
 	double              dIntegrationTimeHighLamp;
 	u_short             wMotorDpi;                      /* Full step DPI */
@@ -381,7 +404,7 @@ typedef struct HWDefault
 	u_short             wGreenPWMDutyCycleLow;
 	u_short             wGreenPWMDutyCycleHigh;
 	/* Registers */
-	u_char              bSensorConfiguration;	/* 0x0b */
+	u_char              bSensorConfiguration;           /* 0x0b */
 	/* Sensor control settings */
 	u_char              bReg_0x0c;
 	u_char              bReg_0x0d;
@@ -399,23 +422,23 @@ typedef struct HWDefault
 	/* initial illumination settings */
 	IllumiDef           illu_mono;
 	IllumiDef           illu_color;
-	
+
 	/* 0x1a & 0x1b, remember the u_char order is not Intel
 	 * format, you have to pay your attention when you
 	 * write this value to register.
 	 */
 	u_short             StepperPhaseCorrection;	
-	
+
 	/* Sensor Pixel Configuration
 	 * Actually, the wActivePixelsStart will be set to 0 for shading purpose.
-     * We have to keep these values to adjust the origins when user does the
-     * scan. These settings are based on optic resolution.
-     */
+	 * We have to keep these values to adjust the origins when user does the
+	 * scan. These settings are based on optic resolution.
+	 */
 	u_char              bOpticBlackStart;       /* 0x1c        */
 	u_char              bOpticBlackEnd;         /* 0x1d        */
 	u_short             wActivePixelsStart;     /* 0x1e & 0x1f */
 	u_short             wLineEnd;               /* 0x20 & 0x21 */
-	
+
 	/* illumination settings (runtime) */
 	u_short             red_lamp_on;            /* 0x2c & 0x2d */
 	u_short             red_lamp_off;           /* 0x2e & 0x2f */
@@ -423,7 +446,7 @@ typedef struct HWDefault
 	u_short             green_lamp_off;         /* 0x32 & 0x33 */
 	u_short             blue_lamp_on;           /* 0x34 & 0x35 */
 	u_short             blue_lamp_off;          /* 0x36 & 0x37 */
-	
+
 	/* Misc */
 	u_char              bReg_0x45;
 	u_short             wStepsAfterPaperSensor2;/* 0x4c & 0x4d */
@@ -440,11 +463,11 @@ typedef struct HWDefault
 	u_char              bReg_0x5c;
 	u_char              bReg_0x5d;
 	u_char              bReg_0x5e;
-	
+
 	eChipDef            chip;           /* chiptype               */
 	eModelDef           motorModel;     /* to identify used motor */
 	double              gamma;          /* gamma setting          */
-} HWDef, *pHWDef;
+} HWDef;
 
 /** device description during runtime
  */
@@ -455,29 +478,31 @@ typedef struct DeviceDef
 	int         product;       /**< product ID                               */
 	DCapsDef    Caps;          /**< pointer to the attribute of current dev  */
 	HWDef       HwSetting;     /**< Pointer to the characteristics of device */
-	pSrcAttrDef pSource;       /**< Scanning src, it's equal to Caps.Normal  */
-							   /**< on the source that the user specified.   */
-	OrgDef	    Normal;        /**< Reflection - Pix to adjust scanning orgs */
-	OrgDef	    Positive;      /**< Pos film - Pix to adjust scanning orgs   */
-	OrgDef	    Negative;      /**< Neg film - Pix to adjust scanning orgs   */
-	OrgDef	    Adf;           /**< Adf - Pixels to adjust scanning origins  */
-	u_long	    dwWarmup;      /**< Ticks to wait for lamp stable, in ms.    */
-	u_long	    dwTicksLampOn; /**< The ticks when lamp turns on             */
-	u_long	    dwLampOnPeriod;/**< How many seconds to keep lamp on         */
-	SANE_Bool	bLampOffOnEnd; /**< switch lamp off on end or keep cur. state*/
-	int		    currentLamp;   /**< The lamp ID of the currently used lamp   */
+	SrcAttrDef *pSource;       /**< Scanning src, it's equal to Caps.Normal  */
+	                           /**< on the source that the user specified.   */
+	OrgDef      Normal;        /**< Reflection - Pix to adjust scanning orgs */
+	OrgDef      Positive;      /**< Pos film - Pix to adjust scanning orgs   */
+	OrgDef      Negative;      /**< Neg film - Pix to adjust scanning orgs   */
+	OrgDef      Adf;           /**< Adf - Pixels to adjust scanning origins  */
+	u_long      dwTicksLampOn; /**< The ticks when lamp turns on             */
+	u_long      dwLampOnPeriod;/**< How many seconds to keep lamp on         */
+	SANE_Bool   bLampOffOnEnd; /**< switch lamp off on end or keep cur. state*/
+	int         currentLamp;   /**< The lamp ID of the currently used lamp   */
+	SANE_Bool   fModFirstHome; /**<                                          */
+	SANE_Bool   fLastScanIsAdf;/**<                                          */
+	u_char      a_bRegs[0x80]; /**< our global register file                 */
 
-} DeviceDef, *pDeviceDef;
+} DeviceDef;
 
 
 typedef struct Settings
 {
-    char      *pIDString;
-	pDCapsDef  pDevCaps;
-	pHWDef     pHwDef;
+	char      *pIDString;
+	DCapsDef  *pDevCaps;
+	HWDef     *pHwDef;
 	char      *pModelString;
 
-} SetDef, *pSetDef;
+} SetDef;
 
 /**
  */
@@ -490,55 +515,50 @@ typedef struct
 
 	/** Driver Info */
 	u_long dwValidPixels; /**< only valid pixels, not incl. pad pix(B/W,Gray)*/
-	u_long dwPhyPixels;	  /**< inlcude pad pixels for ASIC (B/W, Gray)       */
+	u_long dwPhyPixels;   /**< inlcude pad pixels for ASIC (B/W, Gray)       */
 	u_long dwPhyBytes;    /**< bytes to read from ASIC                       */
-	u_long dwPhyLines;	  /**< should include the extra lines accord to the  */
-                          /*   request dpi (CCD lines distance)              */
+	u_long dwPhyLines;    /**< should include the extra lines accord to the  */
+	                      /*   request dpi (CCD lines distance)              */
 	u_long dwTotalBytes;  /**< Total bytes per scan                          */
 
-} WinInfo, *pWinInfo;
+} WinInfo;
 
 /**
  */
 typedef struct
 {
 	/* OUTPUT - Driver returned area. All are based on physical
-     * scanning conditions.
-     */
-	WinInfo	Size;			/* i/p:
-							 * dwPixels, dwBytes(without u_long boundary factor)
-                             * dwLines in user specified dpi
-							 * o/p:
-							 * dwPhyPixels, dwPhyBytes, dwPhyLines
-							 * so after called, caller have to change i
-                             */
-	XY      PhyDpi;			/* Driver DPI */
+	 * scanning conditions. */
+	WinInfo Size;                   /* i/p:
+	                                 * dwPixels, dwBytes(without u_long boundary factor)
+                                         * dwLines in user specified dpi
+	                                 * o/p:
+	                                 * dwPhyPixels, dwPhyBytes, dwPhyLines
+	                                 * so after called, caller have to change it */
+	XY      PhyDpi;                 /* Driver DPI */
 
 	/* INPUT - User info. All sizes and coordinates are specified in the
-     * unit based on 300 DPI
-     */
-	XY      UserDpi;		/**< User specified DPI                          */
-	XY   	Origin;			/**> Scanning origin in optic dpi                */
-	double	dMCLK;			/**< for positive & negative & Adf               */
-	short	brightness;		
-	short	contrast;		
-	short	siThreshold;	/**< only for B/W output                         */
-	u_char	bSource;		/**< Reflection/Positive/Negative/Adf(SOURCE_xxx)*/
-	u_char	bDataType;		/**< Bw, Gray or Color (see _SCANDATATYPE)       */
-	u_char	bBitDepth;		/**< 1/8/14                                      */
-	u_char	bChannels;		/**< Color or Gray                               */
-	u_char  bCalibration;	/**< 1 or 2: the origin.x is from CCD pixel 0 and
-							 *		   the origin.y is from Top of scanner.
-							 * 		   In this case, the WININFO.dwPhyLines
-                             *         will not included the extra lines for
-                             *         color distance factor.
-							 * 0: normal scan, the both directions have to
-                             *    add the distance
-                             */
-	int		swOffset[3];	/**< for calibration adjustment                  */
-	int	    swGain[3];      /**< for calibration adjustment                  */
+	 * unit based on 300 DPI */
+	XY      UserDpi;      /**< User specified DPI                          */
+	XY      Origin;	      /**< Scanning origin in optic dpi                */
+	double  dMCLK;        /**< for positive & negative & Adf               */
+	short   brightness;
+	short   contrast;
+	u_char  bSource;      /**< Reflection/Positive/Negative/Adf(SOURCE_xxx)*/
+	u_char  bDataType;    /**< Bw, Gray or Color (see _SCANDATATYPE)       */
+	u_char  bBitDepth;    /**< 1/8/14                                      */
+	u_char  bChannels;    /**< Color or Gray                               */
+	u_char  bCalibration; /**< 1 or 2: the origin.x is from CCD pixel 0 and
+	                       *           the origin.y is from Top of scanner.
+	                       *           In this case, the WININFO.dwPhyLines
+	                       *         will not included the extra lines for
+	                       *         color distance factor.
+	                       * 0: normal scan, the both directions have to
+	                       *    add the distance */
+	int     swOffset[3];  /**< for calibration adjustment                  */
+	int     swGain[3];    /**< for calibration adjustment                  */
 
-} ScanParam, *pScanParam;
+} ScanParam;
 
 struct Plustek_Device;
 
@@ -546,50 +566,50 @@ struct Plustek_Device;
  */
 typedef struct ScanDef
 {
-    SANE_Bool           fCalibrated;    /**< calibrated or not              */
-    SANE_Bool           skipCoarseCalib;/**< skip coarse calibration or not */
-	u_long				dwFlag;         /**< scan attributes                */
+	SANE_Bool fCalibrated;    /**< calibrated or not              */
+	SANE_Bool skipCoarseCalib;/**< skip coarse calibration or not */
+	u_long    dwFlag;         /**< scan attributes                */
 
-	ScanParam			sParam;         /**< all we need to scan            */
+	ScanParam sParam;         /**< all we need to scan            */
 
-	AnyPtr 				UserBuf;        /**< pointer to the user buffer     */
-	u_long				dwLinesUser;	/**< Number of lines of user buffer */
-	u_long				dwBytesLine;	/**< Bytes per line of user buffer. */
-	u_long				dwLinesToProcess;
+	AnyPtr    UserBuf;        /**< pointer to the user buffer     */
+	u_long    dwLinesUser;    /**< Number of lines of user buffer */
+	u_long    dwBytesLine;    /**< Bytes per line of user buffer. */
+	u_long    dwLinesToProcess;
 
 	/** Image processing routine according to the scan mode  */
 	void (*pfnProcess)(struct Plustek_Device*);
 
-	u_char*				pScanBuffer;	/**< our scan buffer */
+	u_char* pScanBuffer;      /**< our scan buffer */
 
-	u_long				dwLinesPerScanBufs;
-	u_long				dwNumberOfScanBufs;
-	u_long				dwLinesScanBuf;
+	u_long  dwLinesPerScanBufs;
+	u_long  dwNumberOfScanBufs;
+	u_long  dwLinesScanBuf;
 
-	u_char*				pbScanBufBegin;
-	u_char*				pbScanBufEnd;
-	u_char*             pbGetDataBuf;
-	u_long				dwBytesScanBuf;
-	u_long				dwLinesDiscard;
+	u_char* pbScanBufBegin;
+	u_char* pbScanBufEnd;
+	u_char* pbGetDataBuf;
+	u_long  dwBytesScanBuf;
+	u_long  dwLinesDiscard;
 
-	u_long				dwRedShift;
-	u_long				dwGreenShift;
-	u_long				dwBlueShift;
+	u_long  dwRedShift;
+	u_long  dwGreenShift;
+	u_long  dwBlueShift;
 
-	AnyPtr				Green;
-	AnyPtr				Red;
-	AnyPtr				Blue;
+	AnyPtr  Green;
+	AnyPtr  Red;
+	AnyPtr  Blue;
 	
-	long				lBufAdjust;		/**< bytes to adjust buffer pointer  */
-										/*   after a image line processed    */
-	u_short				wSumY;			/**<  for line sampling              */
+	long    lBufAdjust;       /**< bytes to adjust buffer pointer  */
+	                          /*   after a image line processed    */
+	u_short wSumY;            /**<  for line sampling              */
 	
-	u_char				bLineDistance;	/**< Color offset in specific dpi y  */
-	int					fGrayFromColor; /**< channel to use for gray mode    */
+	u_char  bLineDistance;    /**< Color offset in specific dpi y  */
+	int     fGrayFromColor;   /**< channel to use for gray mode    */
 
-	u_char				bLinesToSkip;	/**< how many lines to skip at start */
+	u_char  bLinesToSkip;     /**< how many lines to skip at start */
 
-} ScanDef, *pScanDef;
+} ScanDef;
 
 
 /** max number of different colck settings */
@@ -600,20 +620,20 @@ typedef struct ScanDef
 typedef struct
 {
 	u_char pwm;                 /**< PWM             */
-	u_char pwm_duty;			/**< PWM duty cycles */
-	u_char scan_lines_per_line;	/**< lines to scan to obtain 1 real line
+	u_char pwm_duty;            /**< PWM duty cycles */
+	u_char scan_lines_per_line; /**< lines to scan to obtain 1 real line
                                       will be used in 16bit color modes only */
-} MDef, *pMDef;
+} MDef;
 
 /** according to the CCD and motor, we provide various settings
  */
 typedef struct {
 
-	eModelDef motorModel;	/**< the motor ID */
+	eModelDef motorModel;    /**< the motor ID */
 
-	u_char pwm_fast;		/**< PWM during fast movement      */
-	u_char pwm_duty_fast;   /**< PWM duty during fast movement */
-	u_char mclk_fast;       /**< MCLK during fast movement     */
+	u_char  pwm_fast;       /**< PWM during fast movement      */
+	u_char  pwm_duty_fast;  /**< PWM duty during fast movement */
+	u_char  mclk_fast;      /**< MCLK during fast movement     */
 
     /**
      * here we define some ranges for better supporting
@@ -629,7 +649,16 @@ typedef struct {
 	double gray_mclk_8[_MAX_CLK];   /**< MCLK settings for gray scan         */
 	double gray_mclk_16[_MAX_CLK];  /**< MCLK settings for gray (16bit) scan */
 
-} ClkMotorDef, *pClkMotorDef;
+} ClkMotorDef;
+
+/** for transferring some info between child and parent after calibration
+ */
+#define _MAX_SHAD       0x4000
+#define _SHADING_BUF    (_MAX_SHAD*3)    /**< max size of the shading buffer */
+
+typedef struct {
+	u_long  transferRate;
+} IPCDef;
 
 #endif /* guard __PLUSTEK_USB_H__ */
 
