@@ -7,7 +7,7 @@
  *  @brief Functions to control the scanner hardware.
  *
  * Based on sources acquired from Plustek Inc.<br>
- * Copyright (C) 2001-2004 Gerhard Jaeger <gerhard@gjaeger.de>
+ * Copyright (C) 2001-2005 Gerhard Jaeger <gerhard@gjaeger.de>
  *
  * History:
  * - 0.40 - starting version of the USB support
@@ -40,6 +40,12 @@
  *        - added usb_WaitPos()
  *        - added usb_FillLampRegs() - sets also PWMDutyCylce now
  *        - added UMAX3450 TPA autodetection
+ * - 0.49 - a_bRegs is now part of the device structure
+ *        - fixed problem in backtracking, when speedup is enabled
+ *        - added usb_UpdateButtonStatus()
+ * - 0.50 - added button support for Plustek/Genius devices
+ *        - changed behaviour of usb_IsScannerReady
+ *        - added special misc I/O setup for CIS devices (usb_ResetRegisters)
  * .
  * <hr>
  * This file is part of the SANE package.
@@ -93,12 +99,6 @@
 
 /* HEINER: check the tick counts 'cause 1 tick on NT is about 10ms */
 
-static u_long dwCrystalFrequency = 48000000UL;
-
-static SANE_Bool fModuleFirstHome;  /* HEINER: this has to be initialized */
-static SANE_Bool fLastScanIsAdf;
-static u_char    a_bRegs[0x80];     /**< our global register file */
-
 /** the NatSemi 983x is a big endian chip, and the line protocol data all
  *  arrives big-endian.  This determines if we need to swap to host-order
  */
@@ -128,7 +128,7 @@ static void usb_Swap( u_short *pw, u_long dwBytes )
  * according to the model, the function returns the address of
  * the corresponding entry of the Motor table
  */
-static pClkMotorDef usb_GetMotorSet( eModelDef model )
+static ClkMotorDef *usb_GetMotorSet( eModelDef model )
 {
 	int i;
 
@@ -146,15 +146,14 @@ static pClkMotorDef usb_GetMotorSet( eModelDef model )
  * @param fOn    - SANE_TRUE means motor on, SANE_FALSE means motor off
  * @return always SANE_TRUE
  */
-static SANE_Bool usb_MotorOn( int handle, SANE_Bool fOn )
+static SANE_Bool usb_MotorOn( Plustek_Device *dev, SANE_Bool fOn )
 {
 	if( fOn )
-		a_bRegs[0x45] |= 0x10;
+		dev->usbDev.a_bRegs[0x45] |= 0x10;
 	else
-		a_bRegs[0x45] &= ~0x10;
-
-	usbio_WriteReg( handle, 0x45, a_bRegs[0x45] );
+		dev->usbDev.a_bRegs[0x45] &= ~0x10;
 
+	usbio_WriteReg( dev->fd, 0x45, dev->usbDev.a_bRegs[0x45] );
 	return SANE_TRUE;
 }
 
@@ -172,7 +171,8 @@ static SANE_Bool usb_IsScannerReady( Plustek_Device *dev )
 	u_char         value;
 	double         len;	
 	long           timeout;
-    struct timeval t;
+	struct timeval t;
+	SANE_Status    res;
 
 	/* time in s = 1000*scanner length in inches/max step speed/in */
 	len = (dev->usbDev.Caps.Normal.Size.y/(double)_MEASURE_BASE) + 5;
@@ -187,26 +187,29 @@ static SANE_Bool usb_IsScannerReady( Plustek_Device *dev )
 	timeout = t.tv_sec + len;
 
 	do {	
-		_UIO( usbio_ReadReg( dev->fd, 7, &value));
+		res = usbio_ReadReg( dev->fd, 7, &value);
+		if( res != SANE_STATUS_GOOD ) {
+			sleep(1);
+		} else {
+			if( value == 0 ) {
+				_UIO( usbio_ResetLM983x( dev ));
+				return SANE_TRUE;
+			}
 
-		if( value == 0 ) {
-			_UIO( usbio_ResetLM983x( dev ));
-			return SANE_TRUE;
-		}
+			if((value == 0) || (value >= 0x20) || (value == 0x03)) {
 
-		if((value == 0) || (value >= 0x20) || (value == 0x03)) {
-
-			if( !usbio_WriteReg( dev->fd, 0x07, 0 )) {
-				DBG( _DBG_ERROR, "Scanner not ready!!!\n" );
-				return SANE_FALSE;
+				if( !usbio_WriteReg( dev->fd, 0x07, 0 )) {
+					DBG( _DBG_ERROR, "Scanner not ready!!!\n" );
+					return SANE_FALSE;
+				}
+				else {
+					return SANE_TRUE;	
+				}
 			}
-			else
-				return SANE_TRUE;	
 		}
-	
-		gettimeofday( &t, NULL);	
+		gettimeofday( &t, NULL);
 		
-	} while( t.tv_sec < timeout );		
+	} while( t.tv_sec < timeout );
 	
 	DBG( _DBG_ERROR, "Scanner not ready!!!\n" );
 	return SANE_FALSE;
@@ -241,12 +244,14 @@ static SANE_Bool usb_WaitPos( Plustek_Device *dev, u_long to, SANE_Bool stay )
 {
 	SANE_Bool      retval;
 	u_char         value, mclk_div, mch;
+	u_char         r[2];
 	u_short        ffs, step, min_ffs;
 	long           dwTicks;
 	double         maxf, fac, speed;
 	struct timeval start_time, t2;
 
-	HWDef         *hw = &dev->usbDev.HwSetting;
+	HWDef         *hw   = &dev->usbDev.HwSetting;
+	u_char        *regs = dev->usbDev.a_bRegs;
 
 	/* get current master clock divider */
 	usbio_ReadReg( dev->fd, 0x08, &value );
@@ -257,37 +262,35 @@ static SANE_Bool usb_WaitPos( Plustek_Device *dev, u_long to, SANE_Bool stay )
 	mch = ((value & 0x07) > 1) ? 1:3;
 
 	/* calculate the current speed */
-	ffs   = a_bRegs[0x48] * 256 + a_bRegs[0x49];
-	speed = ((double)dwCrystalFrequency) /(double)((u_long)mclk_div * 32UL * 
+	ffs   = regs[0x48] * 256 + regs[0x49];
+	speed = ((double)CRYSTAL_FREQ) /(double)((u_long)mclk_div * 32UL *
 	                                (u_long)mch * (u_long)ffs * hw->wMotorDpi);
 
-	DBG( _DBG_INFO2, ">>>> CURRENT MCLK_DIV= %u\n", mclk_div );
-	DBG( _DBG_INFO2, ">>>> MCH             = %u\n", mch ); 
-	DBG( _DBG_INFO2, ">>>> FFS             = %u\n", ffs ); 
-	DBG( _DBG_INFO2, ">>>> HIGH-SPEED      = %.3f (%.3f)\n", 
-	                  speed, hw->dHighSpeed); 
-
 	/* disabled ? */
 	if((hw->dHighSpeed == 0.0) || (dev->adj.disableSpeedup != 0)) {
-		DBG( _DBG_INFO2, " * Speedup disabled or not available!\n" );
 		min_ffs = 0xffff;
 		maxf    = 0.0;
 		if( !stay )
 			return SANE_TRUE;
 
 	} else {
-		min_ffs = (u_short)(dwCrystalFrequency /((u_long)mclk_div * 32UL * 
+		min_ffs = (u_short)(CRYSTAL_FREQ /((u_long)mclk_div * 32UL *
 		                   (u_long)mch * hw->dHighSpeed * hw->wMotorDpi));
 		maxf = (ffs - min_ffs)/4;
 		if( maxf > 100.0 )
 			maxf = 100.0;
 		if( maxf < 5.0 )
 			maxf = 5.0;
-	} 
-	DBG( _DBG_INFO2, ">>>> MIN_FFS         = %u (%.3f)\n", min_ffs, maxf); 
+		DBG( _DBG_INFO2, ">>>> CURRENT MCLK_DIV = %u\n", mclk_div );
+		DBG( _DBG_INFO2, ">>>> MCH              = %u\n", mch ); 
+		DBG( _DBG_INFO2, ">>>> FFS              = %u\n", ffs ); 
+		DBG( _DBG_INFO2, ">>>> HIGH-SPEED       = %.3f (%.3f)\n", 
+		                  speed, hw->dHighSpeed); 
+		DBG( _DBG_INFO2, ">>>> MIN_FFS          = %u (%.3f)\n", min_ffs, maxf);
+	}
 
 	gettimeofday( &start_time, NULL );
-    dwTicks = start_time.tv_sec + to;
+	dwTicks = start_time.tv_sec + to;
 	step    = 1;
 	retval  = SANE_FALSE;
 
@@ -323,9 +326,9 @@ static SANE_Bool usb_WaitPos( Plustek_Device *dev, u_long to, SANE_Bool stay )
 				if((int)fac > 25 )
 					usleep( 150 * 1000 );
 
-				a_bRegs[0x48] = (u_char)(ffs >> 8);
-				a_bRegs[0x49] = (u_char)(ffs & 0xFF);
-				sanei_lm983x_write(dev->fd, 0x48, &a_bRegs[0x48], 2, SANE_TRUE);
+				r[0] = (u_char)(ffs >> 8);
+				r[1] = (u_char)(ffs & 0xFF);
+				sanei_lm983x_write(dev->fd, 0x48, r, 2, SANE_TRUE);
 				if(ffs == min_ffs )
 					ffs = 0;
 			} else {
@@ -356,8 +359,9 @@ static SANE_Bool usb_ModuleMove( Plustek_Device *dev,
 	u_char       bReg2, reg7, mclk_div;
 	u_short      wFastFeedStepSize;
 	double       dMaxMoveSpeed;
-	pClkMotorDef clk;
-	pHWDef       hw = &dev->usbDev.HwSetting;
+	ClkMotorDef *clk;
+	HWDef       *hw   = &dev->usbDev.HwSetting;
+	u_char      *regs = dev->usbDev.a_bRegs;
 
 	if( bAction != MOVE_ToPaperSensor   &&
 		bAction != MOVE_EjectAllPapers  &&
@@ -395,7 +399,7 @@ static SANE_Bool usb_ModuleMove( Plustek_Device *dev,
 			return SANE_FALSE;
 
 		usbio_WriteReg( dev->fd, 0x07, 0);
-		usbio_WriteReg( dev->fd, 0x58, a_bRegs[0x58]);
+		usbio_WriteReg( dev->fd, 0x58, regs[0x58]);
 
 		usbio_ReadReg( dev->fd, 0x02, &bReg2 );
 		hw->dMaxMoveSpeed = d;
@@ -417,18 +421,18 @@ static SANE_Bool usb_ModuleMove( Plustek_Device *dev,
 
 	mclk_div = clk->mclk_fast;
 
-	wFastFeedStepSize = (u_short)(dwCrystalFrequency /
+	wFastFeedStepSize = (u_short)(CRYSTAL_FREQ /
 	                    ((u_long)mclk_div * 8UL * 1 *
 	                    dMaxMoveSpeed * 4 * hw->wMotorDpi));
 
-	a_bRegs[0x48] = (u_char)(wFastFeedStepSize >> 8);
-	a_bRegs[0x49] = (u_char)(wFastFeedStepSize & 0xFF);
+	regs[0x48] = (u_char)(wFastFeedStepSize >> 8);
+	regs[0x49] = (u_char)(wFastFeedStepSize & 0xFF);
 
 	dwStep = dwStep * hw->wMotorDpi / 300UL;
-	a_bRegs[0x4a] = _HIBYTE(_LOWORD(dwStep));
-	a_bRegs[0x4b] = _LOBYTE(_LOWORD(dwStep));
+	regs[0x4a] = _HIBYTE(_LOWORD(dwStep));
+	regs[0x4b] = _LOBYTE(_LOWORD(dwStep));
 
-	a_bRegs[0x45] |= 0x10;
+	regs[0x45] |= 0x10;
 
 	DBG( _DBG_INFO2,"MotorDPI=%u, MaxMoveSpeed=%.3f, "
 					"FFStepSize=%u, Steps=%lu\n", hw->wMotorDpi,
@@ -436,8 +440,8 @@ static SANE_Bool usb_ModuleMove( Plustek_Device *dev,
 	DBG( _DBG_INFO2, "MOTOR: "
 					"PWM=0x%02x, PWM_DUTY=0x%02x 0x45=0x%02x "
                     "0x48=0x%02x, 0x49=0x%02x \n",
-					a_bRegs[0x56], a_bRegs[0x57], a_bRegs[0x45],
-					a_bRegs[0x48], a_bRegs[0x49] );
+					regs[0x56], regs[0x57], regs[0x45],
+					regs[0x48], regs[0x49] );
 
 	DBG( _DBG_INFO2,"MCLK_FFW = %u --> 0x%02x\n", mclk_div, (mclk_div-1)*2 );
 
@@ -462,14 +466,14 @@ static SANE_Bool usb_ModuleMove( Plustek_Device *dev,
 	if( !usbio_WriteReg(dev->fd, 0x26, 0x0C))
 		return SANE_FALSE;
 
-	_UIO(sanei_lm983x_write(dev->fd, 0x48, &a_bRegs[0x48], 2, SANE_TRUE));
-	_UIO(sanei_lm983x_write(dev->fd, 0x4A, &a_bRegs[0x4A], 2, SANE_TRUE));
+	_UIO(sanei_lm983x_write(dev->fd, 0x48, &regs[0x48], 2, SANE_TRUE));
+	_UIO(sanei_lm983x_write(dev->fd, 0x4A, &regs[0x4A], 2, SANE_TRUE));
 
 	/* disable home */
-	if( !usbio_WriteReg(dev->fd, 0x58, a_bRegs[0x58] & ~7))
+	if( !usbio_WriteReg(dev->fd, 0x58, regs[0x58] & ~7))
 		return SANE_FALSE;
 
-	if( !usbio_WriteReg(dev->fd, 0x45, a_bRegs[0x45] ))
+	if( !usbio_WriteReg(dev->fd, 0x45, regs[0x45] ))
 		return SANE_FALSE;
 
 	if( bAction == MOVE_Forward || bAction == MOVE_ToShading )
@@ -501,7 +505,7 @@ static SANE_Bool usb_ModuleMove( Plustek_Device *dev,
 
 				if( usb_SensorPaper( dev->fd )) {
 					usbio_WriteReg( dev->fd, 0x07, 0 );
-					usbio_WriteReg( dev->fd, 0x58, a_bRegs[0x58] );
+					usbio_WriteReg( dev->fd, 0x58, regs[0x58] );
 					usbio_ReadReg ( dev->fd, 0x02, &bReg2 );
 					return SANE_TRUE;
 				}
@@ -516,7 +520,7 @@ static SANE_Bool usb_ModuleMove( Plustek_Device *dev,
 
 				if( usb_SensorPaper( dev->fd )) {
 					usbio_WriteReg( dev->fd, 0x07, 0 );
-					usbio_WriteReg( dev->fd, 0x58, a_bRegs[0x58] );
+					usbio_WriteReg( dev->fd, 0x58, regs[0x58] );
 					usbio_ReadReg ( dev->fd, 0x02, &bReg2 );
 					return SANE_TRUE;
 				}
@@ -530,7 +534,7 @@ static SANE_Bool usb_ModuleMove( Plustek_Device *dev,
 			retval = usb_WaitPos( dev, 200, SANE_TRUE );
 		}
 
-		usbio_WriteReg( dev->fd, 0x58, a_bRegs[0x58] );
+		usbio_WriteReg( dev->fd, 0x58, regs[0x58] );
 		usbio_ReadReg ( dev->fd, 0x02, &bReg2 );
 	}
 
@@ -545,22 +549,23 @@ static SANE_Bool usb_ModuleToHome( Plustek_Device *dev, SANE_Bool fWait )
 {
 	u_char    mclk_div;
 	u_char    value;
-	pDCapsDef scaps = &dev->usbDev.Caps;
-	pHWDef    hw    = &dev->usbDev.HwSetting;
+	DCapsDef *scaps = &dev->usbDev.Caps;
+	HWDef    *hw    = &dev->usbDev.HwSetting;
+	u_char   *regs  = dev->usbDev.a_bRegs;
 
 	/* Check if merlin is ready for setting command */
 	usbio_WriteReg( dev->fd, 0x58, hw->bReg_0x58 );
 	usbio_ReadReg ( dev->fd, 2, &value );
 
 	if( value & 1 ) {
-		fModuleFirstHome = SANE_FALSE;
+		dev->usbDev.fModFirstHome = SANE_FALSE;
 		return SANE_TRUE;
 	}
 
 	_UIO( usbio_ReadReg( dev->fd, 0x07, &value ));
 
-	if( fModuleFirstHome ) {
-		fModuleFirstHome = SANE_FALSE;
+	if( dev->usbDev.fModFirstHome ) {
+		dev->usbDev.fModFirstHome = SANE_FALSE;
 		if( hw->motorModel != MODEL_Tokyo600 )
 			usb_ModuleMove( dev, MOVE_Forward, hw->wMotorDpi / 2);
 	}
@@ -579,12 +584,12 @@ static SANE_Bool usb_ModuleToHome( Plustek_Device *dev, SANE_Bool fWait )
 
 		if(!_IS_PLUSTEKMOTOR(hw->motorModel)) {
 
-			pClkMotorDef clk;
+			ClkMotorDef *clk;
 
 			clk = usb_GetMotorSet( hw->motorModel );
 
-			a_bRegs[0x56] = clk->pwm_fast;
-			a_bRegs[0x57] = clk->pwm_duty_fast;
+			regs[0x56] = clk->pwm_fast;
+			regs[0x57] = clk->pwm_duty_fast;
 			mclk_div      = clk->mclk_fast;
 
 		} else {
@@ -597,8 +602,8 @@ static SANE_Bool usb_ModuleToHome( Plustek_Device *dev, SANE_Bool fWait )
 				case MODEL_KaoHsiung:
 				case MODEL_HuaLien:
 				default:
-					a_bRegs[0x56] = 1;
-					a_bRegs[0x57] = 63;
+					regs[0x56] = 1;
+					regs[0x57] = 63;
 					break;
 				}
 			} else { /* if(Device.Caps.OpticDpi.x == 600) */
@@ -606,23 +611,23 @@ static SANE_Bool usb_ModuleToHome( Plustek_Device *dev, SANE_Bool fWait )
 				switch( hw->motorModel ) {
 
 				case MODEL_Tokyo600:
-					a_bRegs[0x56] = 4;
-					a_bRegs[0x57] = 4;	/* 2; */
+					regs[0x56] = 4;
+					regs[0x57] = 4;	/* 2; */
 					break;
 				case MODEL_HuaLien:
 					if( dev->caps.dwFlag & SFLAG_ADF ) {
-						a_bRegs[0x56] = 64;	/* 32; */
-						a_bRegs[0x57] = 4;	/* 16; */
+						regs[0x56] = 64;	/* 32; */
+						regs[0x57] = 4;	/* 16; */
 					} else {
-						a_bRegs[0x56] = 32;
-						a_bRegs[0x57] = 16;
+						regs[0x56] = 32;
+						regs[0x57] = 16;
 					}
 					break;
 				
 				case MODEL_KaoHsiung:
 				default:
-					a_bRegs[0x56] = 64;
-					a_bRegs[0x57] = 20;
+					regs[0x56] = 64;
+					regs[0x57] = 20;
 					break;
 				}
 			}
@@ -631,22 +636,22 @@ static SANE_Bool usb_ModuleToHome( Plustek_Device *dev, SANE_Bool fWait )
 		/* Compute fast feed step size, use equation 3 and equation 8
 		 * assumptions: MCLK = 6, Lineratemode (CM=1)
 		 */
-		wFastFeedStepSize = (u_short)(dwCrystalFrequency / (mclk_div * 8 * 1 *
+		wFastFeedStepSize = (u_short)(CRYSTAL_FREQ / (mclk_div * 8 * 1 *
 		                              hw->dMaxMotorSpeed * 4 * hw->wMotorDpi));
-		a_bRegs[0x48] = (u_char)(wFastFeedStepSize >> 8);
-		a_bRegs[0x49] = (u_char)(wFastFeedStepSize & 0xFF);
-		a_bRegs[0x4a] = 0;
-		a_bRegs[0x4b] = 0;
+		regs[0x48] = (u_char)(wFastFeedStepSize >> 8);
+		regs[0x49] = (u_char)(wFastFeedStepSize & 0xFF);
+		regs[0x4a] = 0;
+		regs[0x4b] = 0;
 
-		a_bRegs[0x45] |= 0x10;
+		regs[0x45] |= 0x10;
 
 		DBG( _DBG_INFO2,"MotorDPI=%u, MaxMotorSpeed=%.3f, FFStepSize=%u\n",
 						hw->wMotorDpi, hw->dMaxMotorSpeed, wFastFeedStepSize );
 		DBG( _DBG_INFO, "MOTOR: "
 						"PWM=0x%02x, PWM_DUTY=0x%02x 0x45=0x%02x "
                         "0x48=0x%02x, 0x49=0x%02x\n",
-						a_bRegs[0x56], a_bRegs[0x57],
-						a_bRegs[0x45], a_bRegs[0x48], a_bRegs[0x49] );
+						regs[0x56], regs[0x57],
+						regs[0x45], regs[0x48], regs[0x49] );
 
 		/* The setting for chassis moving is:
 		 * MCLK divider = 6, 8 bits/pixel, HDPI divider = 12,
@@ -672,10 +677,10 @@ static SANE_Bool usb_ModuleToHome( Plustek_Device *dev, SANE_Bool fWait )
 		if( !usbio_WriteReg(dev->fd, 0x26, 0x8C))
 			return SANE_FALSE;
 
-		_UIO(sanei_lm983x_write(dev->fd, 0x48, &a_bRegs[0x48], 4, SANE_TRUE));
-		_UIO(sanei_lm983x_write(dev->fd, 0x56, &a_bRegs[0x56], 3, SANE_TRUE));
+		_UIO(sanei_lm983x_write(dev->fd, 0x48, &regs[0x48], 4, SANE_TRUE));
+		_UIO(sanei_lm983x_write(dev->fd, 0x56, &regs[0x56], 3, SANE_TRUE));
 
-		if( !usbio_WriteReg(dev->fd, 0x45, a_bRegs[0x45]))
+		if( !usbio_WriteReg(dev->fd, 0x45, regs[0x45]))
 			return SANE_FALSE;
 
 		usbio_WriteReg(dev->fd, 0x0a, 0);
@@ -700,11 +705,12 @@ static SANE_Bool usb_ModuleToHome( Plustek_Device *dev, SANE_Bool fWait )
 				if (!value)
 					return TRUE;
 			}
-			wFastFeedStepSize = (WORD)(dwCrystalFrequency / (6UL * 8UL * 1 * Device.HwSetting.dMaxMotorSpeed * 4 *
+			wFastFeedStepSize = (WORD)(CRYSTAL_FREQ / 
+			    (6UL * 8UL * 1 * Device.HwSetting.dMaxMotorSpeed * 4 *
 				Device.HwSetting.wMotorDpi) * 60 / 78);	
-			a_bRegs[0x48] = (u_char)(wFastFeedStepSize >> 8);
-			a_bRegs[0x49] = (u_char)(wFastFeedStepSize & 0xFF);
-			WriteRegisters(0x48, &a_bRegs[0x48], 2);
+			regs[0x48] = (u_char)(wFastFeedStepSize >> 8);
+			regs[0x49] = (u_char)(wFastFeedStepSize & 0xFF);
+			WriteRegisters(0x48, &regs[0x48], 2);
 		}
 #endif
 	}
@@ -715,8 +721,9 @@ static SANE_Bool usb_ModuleToHome( Plustek_Device *dev, SANE_Bool fWait )
  */
 static SANE_Bool usb_MotorSelect( Plustek_Device *dev, SANE_Bool fADF )
 {
-	pDCapsDef sCaps = &dev->usbDev.Caps;
-	pHWDef    hw    = &dev->usbDev.HwSetting;
+	DCapsDef *sCaps = &dev->usbDev.Caps;
+	HWDef    *hw    = &dev->usbDev.HwSetting;
+	u_char   *regs  = dev->usbDev.a_bRegs;
 
 	if(!_IS_PLUSTEKMOTOR(hw->motorModel)) {
 		return SANE_TRUE;
@@ -731,7 +738,7 @@ static SANE_Bool usb_MotorSelect( Plustek_Device *dev, SANE_Bool fADF )
 			hw->dMaxMoveSpeed  = 1.5;
 			sCaps->OpticDpi.y  = 600;
 		}
-		a_bRegs[0x5b] |= 0x80;
+		regs[0x5b] |= 0x80;
 		
 	} else {
 	
@@ -742,14 +749,14 @@ static SANE_Bool usb_MotorSelect( Plustek_Device *dev, SANE_Bool fADF )
 			hw->dMaxMoveSpeed  = 0.9;
 			sCaps->OpticDpi.y  = 1200;
 		}
-		a_bRegs[0x5b] &= ~0x80;
+		regs[0x5b] &= ~0x80;
 	}
 
 	/* To stop the motor moving */
 	usbio_WriteReg( dev->fd, 0x07, 0 );
 	usleep(10 * 1000);
 
-	usbio_WriteReg( dev->fd, 0x5b, a_bRegs[0x5b] );
+	usbio_WriteReg( dev->fd, 0x5b, regs[0x5b] );
 	return SANE_TRUE;
 }
 
@@ -757,32 +764,33 @@ static SANE_Bool usb_MotorSelect( Plustek_Device *dev, SANE_Bool fADF )
  */
 static SANE_Bool usb_AdjustLamps( Plustek_Device *dev )
 {
-	pHWDef hw = &dev->usbDev.HwSetting;
+	HWDef  *hw   = &dev->usbDev.HwSetting;
+	u_char *regs = dev->usbDev.a_bRegs;
 
-	a_bRegs[0x2c] = _HIBYTE(hw->red_lamp_on);
-	a_bRegs[0x2d] = _LOBYTE(hw->red_lamp_on);
-	a_bRegs[0x2e] = _HIBYTE(hw->red_lamp_off);
-	a_bRegs[0x2f] = _LOBYTE(hw->red_lamp_off);
+	regs[0x2c] = _HIBYTE(hw->red_lamp_on);
+	regs[0x2d] = _LOBYTE(hw->red_lamp_on);
+	regs[0x2e] = _HIBYTE(hw->red_lamp_off);
+	regs[0x2f] = _LOBYTE(hw->red_lamp_off);
 
-	a_bRegs[0x30] = _HIBYTE(hw->green_lamp_on);
-	a_bRegs[0x31] = _LOBYTE(hw->green_lamp_on);
-	a_bRegs[0x32] = _HIBYTE(hw->green_lamp_off);
-	a_bRegs[0x33] = _LOBYTE(hw->green_lamp_off);
+	regs[0x30] = _HIBYTE(hw->green_lamp_on);
+	regs[0x31] = _LOBYTE(hw->green_lamp_on);
+	regs[0x32] = _HIBYTE(hw->green_lamp_off);
+	regs[0x33] = _LOBYTE(hw->green_lamp_off);
 
-	a_bRegs[0x34] = _HIBYTE(hw->blue_lamp_on);
-	a_bRegs[0x35] = _LOBYTE(hw->blue_lamp_on);
-	a_bRegs[0x36] = _HIBYTE(hw->blue_lamp_off);
-	a_bRegs[0x37] = _LOBYTE(hw->blue_lamp_off);
+	regs[0x34] = _HIBYTE(hw->blue_lamp_on);
+	regs[0x35] = _LOBYTE(hw->blue_lamp_on);
+	regs[0x36] = _HIBYTE(hw->blue_lamp_off);
+	regs[0x37] = _LOBYTE(hw->blue_lamp_off);
 
-	return sanei_lm983x_write( dev->fd, 0x2c,
-	                           &a_bRegs[0x2c], 0x37-0x2c+1, SANE_TRUE );
+	return sanei_lm983x_write( dev->fd, 0x2c, 
+	                           &regs[0x2c], 0x37-0x2c+1, SANE_TRUE );
 }
 
 /**
  */
 static void usb_AdjustCISLampSettings( Plustek_Device *dev, SANE_Bool on )
 {
-	pHWDef hw = &dev->usbDev.HwSetting;
+	HWDef *hw = &dev->usbDev.HwSetting;
 
 	if( !(hw->bReg_0x26 & _ONE_CH_COLOR))
 		return;
@@ -845,7 +853,7 @@ static void usb_AdjustCISLampSettings( Plustek_Device *dev, SANE_Bool on )
 		}
 	}
 
-	a_bRegs[0x29] = hw->bReg_0x29;
+	dev->usbDev.a_bRegs[0x29] = hw->bReg_0x29;
 	usb_AdjustLamps( dev );
 }
 
@@ -862,29 +870,29 @@ static void usb_GetLampRegAndMask( u_long flag, SANE_Byte *reg, SANE_Byte *msk )
 		*reg = 0x5b;
 		*msk = 0x80;
 
-   	} else if( _MIO5 == ( _MIO5 & flag )) {
+	} else if( _MIO5 == ( _MIO5 & flag )) {
 		*reg = 0x5b;
 		*msk = 0x08;
 
-   	} else if( _MIO4 == ( _MIO4 & flag )) {
+	} else if( _MIO4 == ( _MIO4 & flag )) {
 		*reg = 0x5a;
 		*msk = 0x80;
 
-   	} else if( _MIO3 == ( _MIO3 & flag )) {
+	} else if( _MIO3 == ( _MIO3 & flag )) {
 		*reg = 0x5a;
 		*msk = 0x08;
-   	
-    } else if( _MIO2 == ( _MIO2 & flag )) {
+
+	} else if( _MIO2 == ( _MIO2 & flag )) {
 		*reg = 0x59;
 		*msk = 0x80;
 
-   	} else if( _MIO1 == ( _MIO1 & flag )) {
+	} else if( _MIO1 == ( _MIO1 & flag )) {
 		*reg = 0x59;
 		*msk = 0x08;
 
-   	} else {
-       *reg = 0;
-       *msk = 0;
+	} else {
+		*reg = 0;
+		*msk = 0;
 	}
 }
 
@@ -899,9 +907,10 @@ static void usb_GetLampRegAndMask( u_long flag, SANE_Byte *reg, SANE_Byte *msk )
  */
 static int usb_GetLampStatus( Plustek_Device *dev )
 {
-	int    iLampStatus = 0;
-	pHWDef hw          = &dev->usbDev.HwSetting;
-	pDCapsDef sc       = &dev->usbDev.Caps;
+	int       iLampStatus = 0;
+	u_char   *regs = dev->usbDev.a_bRegs;
+	HWDef    *hw   = &dev->usbDev.HwSetting;
+	DCapsDef *sc   = &dev->usbDev.Caps;
 	SANE_Byte reg, msk, val;
 
 
@@ -910,28 +919,14 @@ static int usb_GetLampStatus( Plustek_Device *dev )
 		return -1;
 	}
 
-#if 0
-	/* on the CanoScan D660U switch always... */
-	if((dev->usbDev.vendor == 0x04A9) && (dev->usbDev.product==0x2208)) {
-		DBG( _DBG_INFO, "CanoScan D660U -> Lamp is off!!!\n" );
-		return 0;
-	}
-#endif
-
 	/* do we use the misc I/O pins for switching the lamp ? */
 	if( _WAF_MISC_IO_LAMPS & sc->workaroundFlag ) {
 
 		usb_GetLampRegAndMask( sc->lamp, &reg, &msk );
 
 		if( 0 == reg ) {
-#if 0
-			/* probably not correct, esp. when changing from color to gray...*/
-			usbio_ReadReg( dev->fd, 0x29, &a_bRegs[0x29] );
-			if( a_bRegs[0x29] & 3 )
-#else
 			usbio_ReadReg( dev->fd, 0x29, &reg );
 			if( reg & 3 )
-#endif
 				iLampStatus |= DEV_LampReflection;
 		} else {
 
@@ -952,30 +947,29 @@ static int usb_GetLampStatus( Plustek_Device *dev )
 				if( val & msk )
 					iLampStatus |= DEV_LampTPA;
 			}
-	
+
 			if((dev->usbDev.vendor == 0x04A9) && (dev->usbDev.product==0x2208)) {
-/*				DBG( _DBG_INFO, "CanoScan D660U -> Lamp is off!!! (STATUS=%i)\n", iLampStatus );*/
-				sanei_lm983x_read( dev->fd, 0x29, &a_bRegs[0x29], 3, SANE_TRUE );
-				DBG( _DBG_INFO, "[29]=0x%02x, [2A]=0x%02x, [2B]=0x%02x\n", a_bRegs[0x29], a_bRegs[0x2a], a_bRegs[0x2b] );
-				/*return 0;*/
+				sanei_lm983x_read( dev->fd, 0x29, &regs[0x29], 3, SANE_TRUE );
+				DBG( _DBG_INFO, "[29]=0x%02x, [2A]=0x%02x, [2B]=0x%02x\n", 
+				                regs[0x29], regs[0x2a], regs[0x2b] );
 			}
 		}
 	} else {
-		sanei_lm983x_read(dev->fd, 0x29,&a_bRegs[0x29],0x37-0x29+1,SANE_TRUE);
+		sanei_lm983x_read(dev->fd, 0x29,&regs[0x29],0x37-0x29+1,SANE_TRUE);
 
-		if((a_bRegs[0x29] & 3) == 1) {
+		if((regs[0x29] & 3) == 1) {
 
 /* HEINER: BETTER define register to check ! */
 
 			if(!_IS_PLUSTEKMOTOR(hw->motorModel)) {
 				iLampStatus |= DEV_LampReflection;
 
-            } else {
+			} else {
 
-				if((a_bRegs[0x2e] * 256 + a_bRegs[0x2f]) > hw->wLineEnd )
+				if((regs[0x2e] * 256 + regs[0x2f]) > hw->wLineEnd )
 					iLampStatus |= DEV_LampReflection;
 
-				if((a_bRegs[0x36] * 256 + a_bRegs[0x37]) > hw->wLineEnd )
+				if((regs[0x36] * 256 + regs[0x37]) > hw->wLineEnd )
 					iLampStatus |= DEV_LampTPA;
 			}
 		}
@@ -992,7 +986,8 @@ static SANE_Bool usb_switchLampX( Plustek_Device *dev,
                                   SANE_Bool on, SANE_Bool tpa )
 {
 	SANE_Byte reg, msk;
-	pDCapsDef sc = &dev->usbDev.Caps;
+	DCapsDef *sc   = &dev->usbDev.Caps;
+	u_char   *regs = dev->usbDev.a_bRegs;
 
 	if( tpa )
 		usb_GetLampRegAndMask( _GET_TPALAMP(sc->lamp), &reg, &msk );
@@ -1005,13 +1000,13 @@ static SANE_Bool usb_switchLampX( Plustek_Device *dev,
 	DBG( _DBG_INFO, "usb_switchLampX(ON=%u,TPA=%u)\n", on, tpa );
 
 	if( on )
-		a_bRegs[reg] |= msk;
+		regs[reg] |= msk;
 	else
-		a_bRegs[reg] &= ~msk;
+		regs[reg] &= ~msk;
 
 	DBG( _DBG_INFO, "Switch Lamp: %u, regs[0x%02x] = 0x%02x\n",
-	                on, reg, a_bRegs[reg] );
-	usbio_WriteReg( dev->fd, reg, a_bRegs[reg] );
+	                on, reg, regs[reg] );
+	usbio_WriteReg( dev->fd, reg, regs[reg] );
 	return SANE_TRUE;
 }
 
@@ -1040,14 +1035,14 @@ static void usb_LedOn( Plustek_Device *dev, SANE_Bool fOn )
 	if( dev->usbDev.HwSetting.motorModel != MODEL_HuaLien )
 		return;
 
-	value = a_bRegs[0x0d];
+	value = dev->usbDev.a_bRegs[0x0d];
 
 	if( fOn )
 		value |= 0x10;
 	else
 		value &= ~0x10;
 
-	a_bRegs[0x0d] = value;
+	dev->usbDev.a_bRegs[0x0d] = value;
 	usbio_WriteReg( dev->fd, 0x0d, value );
 }
 
@@ -1056,25 +1051,26 @@ static void usb_LedOn( Plustek_Device *dev, SANE_Bool fOn )
  */
 static void usb_FillLampRegs( Plustek_Device *dev )
 {
-	pHWDef hw = &dev->usbDev.HwSetting;
+	HWDef  *hw   = &dev->usbDev.HwSetting;
+	u_char *regs = dev->usbDev.a_bRegs;
 
-	a_bRegs[0x2a] = _HIBYTE( hw->wGreenPWMDutyCycleLow );
-	a_bRegs[0x2b] = _LOBYTE( hw->wGreenPWMDutyCycleLow );
+	regs[0x2a] = _HIBYTE( hw->wGreenPWMDutyCycleLow );
+	regs[0x2b] = _LOBYTE( hw->wGreenPWMDutyCycleLow );
 
-	a_bRegs[0x2c] = _HIBYTE( hw->red_lamp_on );
-	a_bRegs[0x2d] = _LOBYTE( hw->red_lamp_on );
-	a_bRegs[0x2e] = _HIBYTE( hw->red_lamp_off);
-	a_bRegs[0x2f] = _LOBYTE( hw->red_lamp_off);
+	regs[0x2c] = _HIBYTE( hw->red_lamp_on );
+	regs[0x2d] = _LOBYTE( hw->red_lamp_on );
+	regs[0x2e] = _HIBYTE( hw->red_lamp_off);
+	regs[0x2f] = _LOBYTE( hw->red_lamp_off);
 
-	a_bRegs[0x30] = _HIBYTE( hw->green_lamp_on );
-	a_bRegs[0x31] = _LOBYTE( hw->green_lamp_on );
-	a_bRegs[0x32] = _HIBYTE( hw->green_lamp_off);
-	a_bRegs[0x33] = _LOBYTE( hw->green_lamp_off);
+	regs[0x30] = _HIBYTE( hw->green_lamp_on );
+	regs[0x31] = _LOBYTE( hw->green_lamp_on );
+	regs[0x32] = _HIBYTE( hw->green_lamp_off);
+	regs[0x33] = _LOBYTE( hw->green_lamp_off);
 
-	a_bRegs[0x34] = _HIBYTE( hw->blue_lamp_on );
-	a_bRegs[0x35] = _LOBYTE( hw->blue_lamp_on );
-	a_bRegs[0x36] = _HIBYTE( hw->blue_lamp_off);
-	a_bRegs[0x37] = _LOBYTE( hw->blue_lamp_off);
+	regs[0x34] = _HIBYTE( hw->blue_lamp_on );
+	regs[0x35] = _LOBYTE( hw->blue_lamp_on );
+	regs[0x36] = _HIBYTE( hw->blue_lamp_off);
+	regs[0x37] = _LOBYTE( hw->blue_lamp_off);
 }
 
 /** usb_LampOn
@@ -1082,9 +1078,10 @@ static void usb_FillLampRegs( Plustek_Device *dev )
 static SANE_Bool usb_LampOn( Plustek_Device *dev,
                              SANE_Bool fOn, SANE_Bool fResetTimer )
 {
-	pDCapsDef      sc          = &dev->usbDev.Caps;
-	pScanDef       scanning    = &dev->scanning;
-	pHWDef         hw          = &dev->usbDev.HwSetting;
+	DCapsDef      *sc          = &dev->usbDev.Caps;
+	ScanDef       *scanning    = &dev->scanning;
+	HWDef         *hw          = &dev->usbDev.HwSetting;
+	u_char        *regs        = dev->usbDev.a_bRegs;
 	int            iLampStatus = usb_GetLampStatus( dev );
 	int            lampId      = -1;
 	struct timeval t;
@@ -1123,20 +1120,20 @@ static SANE_Bool usb_LampOn( Plustek_Device *dev,
 					usb_switchLampX( dev, SANE_FALSE, SANE_TRUE );
 			}
 
-			memset( &a_bRegs[0x29], 0, (0x37-0x29+1));
+			memset( &regs[0x29], 0, (0x37-0x29+1));
 			
-			a_bRegs[0x29] = hw->bReg_0x29;
+			regs[0x29] = hw->bReg_0x29;
 
 			if( !usb_switchLamp(dev, SANE_TRUE )) {
 
 				if( lampId == DEV_LampReflection ) {
-					a_bRegs[0x2e] = 16383 / 256;
-					a_bRegs[0x2f] = 16383 % 256;
+					regs[0x2e] = 16383 / 256;
+					regs[0x2f] = 16383 % 256;
 				}
 
 				if( lampId == DEV_LampTPA ) {
-					a_bRegs[0x36] = 16383 / 256;
-					a_bRegs[0x37] = 16383 % 256;
+					regs[0x36] = 16383 / 256;
+					regs[0x37] = 16383 % 256;
 				}
 			}
 
@@ -1144,7 +1141,7 @@ static SANE_Bool usb_LampOn( Plustek_Device *dev,
 				usb_FillLampRegs( dev );
 
 			sanei_lm983x_write( dev->fd, 0x29,
-			                    &a_bRegs[0x29], 0x37-0x29+1, SANE_TRUE );
+			                    &regs[0x29], 0x37-0x29+1, SANE_TRUE );
 			if( lampId != dev->usbDev.currentLamp ) {
 			
 				dev->usbDev.currentLamp = lampId;
@@ -1166,17 +1163,17 @@ static SANE_Bool usb_LampOn( Plustek_Device *dev,
 
 			DBG( _DBG_INFO, "Switching Lamp off\n" );
 		
-			memset( &a_bRegs[0x29], 0, 0x37-0x29+1 );
+			memset( &regs[0x29], 0, 0x37-0x29+1 );
 			if( !usb_switchLamp(dev, SANE_FALSE )) {
 			
 				if( iStatusChange & DEV_LampReflection ) {
-					a_bRegs[0x2e] = 16383 / 256;
-					a_bRegs[0x2f] = 16383 % 256;
+					regs[0x2e] = 16383 / 256;
+					regs[0x2f] = 16383 % 256;
 				}
-    	
+
 				if( iStatusChange & DEV_LampTPA ) {
-					a_bRegs[0x36] = 16383 / 256;
-					a_bRegs[0x37] = 16383 % 256;
+					regs[0x36] = 16383 / 256;
+					regs[0x37] = 16383 % 256;
 				}
 			}
 
@@ -1184,7 +1181,7 @@ static SANE_Bool usb_LampOn( Plustek_Device *dev,
 				usb_FillLampRegs( dev );
 
 			sanei_lm983x_write( dev->fd, 0x29,
-			                    &a_bRegs[0x29], 0x37-0x29+1, SANE_TRUE );
+			                    &regs[0x29], 0x37-0x29+1, SANE_TRUE );
 		}
 	}
 	if( usb_GetLampStatus(dev))
@@ -1217,40 +1214,41 @@ static void usb_ResetRegisters( Plustek_Device *dev )
 {
 	int linend;
 
-	pHWDef hw = &dev->usbDev.HwSetting;
+	HWDef  *hw   = &dev->usbDev.HwSetting;
+	u_char *regs = dev->usbDev.a_bRegs;
 
-	DBG( _DBG_INFO, "RESETTING REGISTERS(%i)\n", dev->initialized );
-	memset( a_bRegs, 0, sizeof(a_bRegs));
+	DBG( _DBG_INFO, "RESETTING REGISTERS(%i) - 0x%02x\n", dev->initialized,sizeof(dev->usbDev.a_bRegs));
+	memset( regs, 0, sizeof(dev->usbDev.a_bRegs));
 
-	memcpy( a_bRegs+0x0b, &hw->bSensorConfiguration, 4 );
-	memcpy( a_bRegs+0x0f, &hw->bReg_0x0f_Color, 10 );
-	a_bRegs[0x1a] = _HIBYTE( hw->StepperPhaseCorrection );
-	a_bRegs[0x1b] = _LOBYTE( hw->StepperPhaseCorrection );
+	memcpy( regs+0x0b, &hw->bSensorConfiguration, 4 );
+	memcpy( regs+0x0f, &hw->bReg_0x0f_Color, 10 );
+	regs[0x1a] = _HIBYTE( hw->StepperPhaseCorrection );
+	regs[0x1b] = _LOBYTE( hw->StepperPhaseCorrection );
 #if 0
-	a_bRegs[0x1e] = _HIBYTE( hw->wActivePixelsStart );
-	a_bRegs[0x1f] = _LOBYTE( hw->wActivePixelsStart );
+	regs[0x1e] = _HIBYTE( hw->wActivePixelsStart );
+	regs[0x1f] = _LOBYTE( hw->wActivePixelsStart );
 #endif
-	a_bRegs[0x20] = _HIBYTE( hw->wLineEnd );
-	a_bRegs[0x21] = _LOBYTE( hw->wLineEnd );
+	regs[0x20] = _HIBYTE( hw->wLineEnd );
+	regs[0x21] = _LOBYTE( hw->wLineEnd );
 
-	a_bRegs[0x22] = _HIBYTE( hw->bOpticBlackStart );
-	a_bRegs[0x22] = _LOBYTE( hw->bOpticBlackStart );
+	regs[0x22] = _HIBYTE( hw->bOpticBlackStart );
+	regs[0x22] = _LOBYTE( hw->bOpticBlackStart );
 
 	linend = hw->bOpticBlackStart + hw->wLineEnd;
 	if( linend < (hw->wLineEnd-20))
 		linend = hw->wLineEnd-20;
 
-	a_bRegs[0x24] = _HIBYTE( linend );
-	a_bRegs[0x25] = _LOBYTE( linend );
+	regs[0x24] = _HIBYTE( linend );
+	regs[0x25] = _LOBYTE( linend );
 
-	a_bRegs[0x2a] = _HIBYTE( hw->wGreenPWMDutyCycleHigh );
-	a_bRegs[0x2b] = _LOBYTE( hw->wGreenPWMDutyCycleHigh );
+	regs[0x2a] = _HIBYTE( hw->wGreenPWMDutyCycleHigh );
+	regs[0x2b] = _LOBYTE( hw->wGreenPWMDutyCycleHigh );
 
-	a_bRegs[0x45] = hw->bReg_0x45;
-	a_bRegs[0x4c] = _HIBYTE( hw->wStepsAfterPaperSensor2 );
-	a_bRegs[0x4d] = _LOBYTE( hw->wStepsAfterPaperSensor2 );
-	a_bRegs[0x50] = hw->bStepsToReverse;
-	a_bRegs[0x51] = hw->bReg_0x51;
+	regs[0x45] = hw->bReg_0x45;
+	regs[0x4c] = _HIBYTE( hw->wStepsAfterPaperSensor2 );
+	regs[0x4d] = _LOBYTE( hw->wStepsAfterPaperSensor2 );
+	regs[0x50] = hw->bStepsToReverse;
+	regs[0x51] = hw->bReg_0x51;
 
 	/* if already initialized, we ignore the MISC I/O settings as
 	 * they are used to determine the current lamp settings...
@@ -1258,19 +1256,30 @@ static void usb_ResetRegisters( Plustek_Device *dev )
 	if( dev->initialized >= 0 ) {
 
 		DBG( _DBG_INFO2, "USING MISC I/O settings\n" );
-		memcpy( a_bRegs+0x54, &hw->bReg_0x54, 0x58 - 0x54 + 1 );
-		a_bRegs[0x5c] = hw->bReg_0x5c;
-		a_bRegs[0x5d] = hw->bReg_0x5d;
-		a_bRegs[0x5e] = hw->bReg_0x5e;
-		sanei_lm983x_read( dev->fd, 0x59, &a_bRegs[0x59], 3, SANE_TRUE );
+		memcpy( regs+0x54, &hw->bReg_0x54, 0x58 - 0x54 + 1 );
+		regs[0x5c] = hw->bReg_0x5c;
+		regs[0x5d] = hw->bReg_0x5d;
+		regs[0x5e] = hw->bReg_0x5e;
+		sanei_lm983x_read( dev->fd, 0x59, &regs[0x59], 3, SANE_TRUE );
 	} else {
 
 		DBG( _DBG_INFO2, "SETTING THE MISC I/Os\n" );
-		memcpy( a_bRegs+0x54, &hw->bReg_0x54, 0x5e - 0x54 + 1 );
-		sanei_lm983x_write( dev->fd, 0x59, &a_bRegs[0x59], 3, SANE_TRUE );
+		memcpy( regs+0x54, &hw->bReg_0x54, 0x5e - 0x54 + 1 );
+
+		if( usb_IsCISDevice( dev )) {
+
+			/* this sequence seems to be needed at least for the
+			 * CanoScan devices to work properly after power-up
+			 */
+			sanei_lm983x_write_byte( dev->fd, 0x5b, regs[0x5b] );
+			sanei_lm983x_write_byte( dev->fd, 0x59, regs[0x59] );
+			sanei_lm983x_write_byte( dev->fd, 0x5a, regs[0x5a] );
+		} else {
+			sanei_lm983x_write( dev->fd, 0x59, &regs[0x59], 3, SANE_TRUE );
+		}
 	}
 	DBG( _DBG_INFO, "MISC I/O after RESET: 0x%02x, 0x%02x, 0x%02x\n",
-	                        a_bRegs[0x59], a_bRegs[0x5a], a_bRegs[0x5b] );
+	                        regs[0x59], regs[0x5a], regs[0x5b] );
 }
 
 /** usb_ModuleStatus
@@ -1278,7 +1287,7 @@ static void usb_ResetRegisters( Plustek_Device *dev )
 static SANE_Bool usb_ModuleStatus( Plustek_Device *dev )
 {
 	u_char value;
-	pHWDef hw = &dev->usbDev.HwSetting;
+	HWDef *hw = &dev->usbDev.HwSetting;
 
 /* HEINER: Maybe needed to avoid recalibration!!! */
 #if 0
@@ -1304,7 +1313,7 @@ static SANE_Bool usb_ModuleStatus( Plustek_Device *dev )
 			usbio_ReadReg( dev->fd, 2, &value );
 		}
 
-		usb_MotorOn( dev->fd, SANE_FALSE );
+		usb_MotorOn( dev, SANE_FALSE );
 		return SANE_TRUE;
 	}
 
@@ -1455,7 +1464,7 @@ static SANE_Bool usb_Wait4Warmup( Plustek_Device *dev )
 	u_long         dw;
 	struct timeval t;
 
-	pHWDef hw = &dev->usbDev.HwSetting;
+	HWDef *hw = &dev->usbDev.HwSetting;
 
 	if( hw->bReg_0x26 & _ONE_CH_COLOR ) {
 		DBG(_DBG_INFO,"Warmup: skipped for CIS devices\n" );
@@ -1511,7 +1520,7 @@ static SANE_Bool usb_HasTPA( Plustek_Device *dev )
 			return SANE_TRUE;
 		} else
 			DBG( _DBG_INFO, "EPSON-TPA NOT detected\n" );
-	
+
 		if( dev->adj.enableTpa ) {
 			DBG( _DBG_INFO, "EPSON-TPA usage forced\n" );
 			return SANE_TRUE;
@@ -1548,4 +1557,108 @@ static SANE_Bool usb_HasTPA( Plustek_Device *dev )
 	return SANE_FALSE;
 }
 
+/** function for reading the button states
+ */
+static SANE_Bool usb_UpdateButtonStatus( Plustek_Scanner *s )
+{
+	u_char          mio[3];
+	SANE_Byte       val, mask;
+	int             i, j, bc;
+	int             handle = -1;
+	SANE_Status     status;
+	Plustek_Device *dev  = s->hw;
+	DCapsDef       *caps = &dev->usbDev.Caps;
+
+	if (caps->bButtons == 0)
+		return SANE_FALSE;
+
+	status = sanei_access_lock( dev->sane.name, 3 );
+	if( SANE_STATUS_GOOD != status ) 
+		return SANE_FALSE;
+
+	if( -1 == dev->fd ) {
+
+		status = sanei_usb_open(dev->sane.name, &handle);
+		if( SANE_STATUS_GOOD != status ) {
+			sanei_access_unlock( dev->sane.name );
+			return SANE_FALSE;
+		}
+		dev->fd = handle;
+	}
+
+	/* we have to check all 6 misc I/O ports for input configuration*/
+	mio[0] = dev->usbDev.HwSetting.bReg_0x59;
+	mio[1] = dev->usbDev.HwSetting.bReg_0x5a;
+	mio[2] = dev->usbDev.HwSetting.bReg_0x5b;
+	
+	usbio_ReadReg( dev->fd, 0x07, &val );
+	if( val == 0 ) {
+
+		/* first read clears the status... */
+		usbio_ReadReg( dev->fd, 0x02, &val );
+
+		/* Plustek and KYE/Genius use altnernative button handling */
+		if((dev->usbDev.vendor == 0x07B3) || (dev->usbDev.vendor == 0x0458)) {
+
+			/* no button pressed so far */
+			for( i = 0; i < caps->bButtons; i++ )
+				s->val[OPT_BUTTON_0 + i].w = 0;
+
+			if (caps->bButtons == 2 || caps->bButtons == 5) {
+				val >>= 2;
+
+				switch( val ) {
+					case 1: s->val[OPT_BUTTON_1].w = 1; break;
+					case 2: s->val[OPT_BUTTON_0].w = 1; break;
+					case 3: s->val[OPT_BUTTON_2].w = 1; break;
+					case 4: s->val[OPT_BUTTON_3].w = 1; break;
+					case 6: s->val[OPT_BUTTON_4].w = 1; break;
+				}
+			} else if (caps->bButtons == 4 ) {
+				val >>= 5;
+				switch( val ) {
+					case 1: s->val[OPT_BUTTON_0].w = 1; break;
+					case 2: s->val[OPT_BUTTON_1].w = 1; break;
+					case 4: s->val[OPT_BUTTON_2].w = 1; break;
+					case 6: s->val[OPT_BUTTON_3].w = 1; break;
+				}
+			}
+
+		} else {
+
+			/* the generic section... */
+			val >>= 2;
+			bc    = 0; 
+
+			for( i = 0; i < 3; i++ ) {
+	
+				DBG( _DBG_INFO2, "Checking MISC IO[%u]=0x%02x\n", i, mio[i] );
+				mask = 0x01;
+
+				for( j = 0; j < 2; j++ ) {
+
+					if((mio[i] & mask) == 0) {
+						DBG( _DBG_INFO2, "* port %u configured as input,"
+						     " status: %s (%u)\n", (i*2)+j+1, 
+							 ((val & 1)?"PRESSED":"RELEASED"), (OPT_BUTTON_0 + bc));
+						s->val[OPT_BUTTON_0 + bc].w = val & 1; 
+						bc++;
+					}
+					val  >>= 1;
+					mask <<= 4;
+				}
+			}
+		}
+	} else {
+		DBG( _DBG_INFO2, "Scanner NOT idle: 0x%02x\n", val );
+	}
+
+	if( -1 != handle ) {
+		dev->fd = -1;
+		sanei_usb_close( handle );
+	}
+
+	sanei_access_unlock( dev->sane.name );
+	return SANE_TRUE;
+}
 /* END PLUSTEK-USBHW.C ......................................................*/
