@@ -7,7 +7,7 @@
  *  @brief Creating and manipulating lookup tables.
  *
  * Based on sources acquired from Plustek Inc.<br>
- * Copyright (C) 2001-2004 Gerhard Jaeger <gerhard@gjaeger.de>
+ * Copyright (C) 2001-2005 Gerhard Jaeger <gerhard@gjaeger.de>
  *
  * History:
  * - 0.40 - starting version of the USB support
@@ -18,6 +18,9 @@
  * - 0.45 - no changes
  * - 0.46 - no changes
  * - 0.47 - cleanup work
+ * - 0.48 - added support for binary from color scans
+ * - 0.49 - changed usb_MapDownload
+ * - 0.50 - cleanup
  * .
  * <hr>
  * This file is part of the SANE package.
@@ -60,15 +63,15 @@
  * <hr>
  */
 
-#define _MAP_SIZE		4096U
+#define _MAP_SIZE 4096
 
 static SANE_Byte a_bMap[_MAP_SIZE * 3];
 
 /** adjust acording to brightness and contrast
  */
-static void usb_MapAdjust( pPlustek_Device dev )
+static void usb_MapAdjust( Plustek_Device *dev )
 {
-	u_long i, tabLen;
+	int    i, tabLen;
 	double b, c, tmp;
 	
 	tabLen = _MAP_SIZE;
@@ -89,7 +92,7 @@ static void usb_MapAdjust( pPlustek_Device dev )
 
 	if( dev->scanning.sParam.brightness == 0 && dev->scanning.sParam.contrast )
 		return;
-	                
+
 	for( i = 0; i < tabLen; i++ ) {
 
 		tmp = ((double)(a_bMap[i] + b)) * c;
@@ -111,13 +114,13 @@ static void usb_MapAdjust( pPlustek_Device dev )
 
 /**
  */
-static SANE_Bool usb_MapDownload( pPlustek_Device dev, u_char bDataType )
+static SANE_Bool usb_MapDownload( Plustek_Device *dev )
 {
-	pScanDef  scanning = &dev->scanning;
-	pDCapsDef sc       = &dev->usbDev.Caps;
+	ScanDef  *scanning = &dev->scanning;
+	DCapsDef *sc       = &dev->usbDev.Caps;
 
-	int       color, maxColor;
-	int       i, iThreshold;
+	int       color;
+	int       i, threshold;
 	SANE_Byte value;
 	SANE_Bool fInverse = 0;
 	
@@ -125,59 +128,45 @@ static SANE_Bool usb_MapDownload( pPlustek_Device dev, u_char bDataType )
 
 	/* the maps are have been already set */
 	
-	/* do the brightness and contrast adjustment ... */			
-	if( scanning->sParam.bDataType != SCANDATATYPE_BW )	
+	/* do the brightness and contrast adjustment ... */
+	if( scanning->sParam.bDataType != SCANDATATYPE_BW )
 		usb_MapAdjust( dev );
 
 	if( !usbio_WriteReg( dev->fd, 7, 0))
 		return SANE_FALSE;
 
 	/* we download all the time all three color maps, as we run
-     * into trouble elsewhere on CanoScan models using gray mode
-     */
-#if 0
-	if( bDataType == SCANDATATYPE_Color ) {
-		color    = 0;
-		maxColor = 3;
-	} else {
-		color    = 1;
-		maxColor = 2;
-	}
-#else
-	_VAR_NOT_USED( bDataType );
-
-	color    = 0;
-	maxColor = 3;
-#endif
-
-	for( ; color < maxColor; color++) {
+	 * into trouble elsewhere on CanoScan models using gray mode
+	 */
+	for( color = 0; color < 3; color++) {
 	
 		/* select color */
 		value = (color << 2)+2;
-		
+
 		/* set gamma color selector */
 		usbio_WriteReg( dev->fd, 0x03, value );
 		usbio_WriteReg( dev->fd, 0x04, 0 );
 		usbio_WriteReg( dev->fd, 0x05, 0 );
-		
-		/* write the gamma table entry to merlin */
-		if( scanning->sParam.bDataType == SCANDATATYPE_BW )	{
-		
-			iThreshold = (int)((double)scanning->sParam.siThreshold *
-											(_MAP_SIZE/200.0)) + (_MAP_SIZE/2);
-			iThreshold = _MAP_SIZE - iThreshold;
-			if(iThreshold < 0)
-				iThreshold = 0;
-			if(iThreshold > (int)_MAP_SIZE)
-				iThreshold = _MAP_SIZE;
-	
-			DBG(_DBG_INFO, "* Threshold is at %u siThresh=%i\n",
-								iThreshold, scanning->sParam.siThreshold );
 
-			for(i = 0; i < iThreshold; i++)
+		/* write the gamma table entry to merlin */
+		if((scanning->sParam.bDataType == SCANDATATYPE_BW) ||
+		   (scanning->fGrayFromColor > 7 )) {
+
+			threshold = (int)((double)scanning->sParam.brightness *
+			                                (_MAP_SIZE/200.0)) + (_MAP_SIZE/2);
+			threshold = _MAP_SIZE - threshold;
+			if(threshold < 0)
+				threshold = 0;
+			if(threshold > (int)_MAP_SIZE)
+				threshold = _MAP_SIZE;
+	
+			DBG(_DBG_INFO, "* Threshold is at %u brightness=%i\n",
+			               threshold, scanning->sParam.brightness );
+
+			for(i = 0; i < threshold; i++)
 				a_bMap[color*_MAP_SIZE + i] = 0;
 				
-			for(i = iThreshold; i < (int)_MAP_SIZE; i++)
+			for(i = threshold; i < _MAP_SIZE; i++)
 				a_bMap[color*_MAP_SIZE + i] = 255;
 
 			fInverse = 1;
@@ -191,11 +180,6 @@ static SANE_Bool usb_MapDownload( pPlustek_Device dev, u_char bDataType )
 			(sc->workaroundFlag &_WAF_INV_NEGATIVE_MAP)) {
 			fInverse ^= 1;
 		}
-		
-		if((scanning->dwFlag & SCANFLAG_Invert) &&
-									!(scanning->dwFlag & SCANFLAG_Pseudo48)) {
-			fInverse ^= 1;
-		}	
 
 		if( fInverse ) {
 		
@@ -204,21 +188,20 @@ static SANE_Bool usb_MapDownload( pPlustek_Device dev, u_char bDataType )
 			
 			DBG( _DBG_INFO, "* Inverting Map\n" );
 			
-			for( i = 0; i < (int)_MAP_SIZE; i++, pMap++ )
+			for( i = 0; i < _MAP_SIZE; i++, pMap++ )
 				map[i] = ~*pMap;
 			
 			sanei_lm983x_write( dev->fd,  0x06, map, _MAP_SIZE, SANE_FALSE );
 			
 		} else {
 			DBG( _DBG_INFO, "* downloading map %u...\n", color );
-			sanei_lm983x_write( dev->fd,  0x06, a_bMap+color*_MAP_SIZE,
-								 _MAP_SIZE, SANE_FALSE );
-		}								
+			sanei_lm983x_write( dev->fd, 0x06, a_bMap+color*_MAP_SIZE,
+			                    _MAP_SIZE, SANE_FALSE );
+		}
 	
 	} /* for each color */
 
 	DBG( _DBG_INFO, "usb_MapDownload() done.\n" );
-	
 	return SANE_TRUE;
 }
 
