@@ -11,7 +11,7 @@
  * Original code taken from sane-0.71<br>
  * Copyright (C) 1997 Hypercore Software Design, Ltd.<br>
  * Also based on the work done by Rick Bronson<br>
- * Copyright (C) 2000-2004 Gerhard Jaeger <gerhard@gjaeger.de><br>
+ * Copyright (C) 2000-2005 Gerhard Jaeger <gerhard@gjaeger.de><br>
  *
  * History:
  * - 0.30 - initial version
@@ -71,6 +71,14 @@
  *        - closing now writer pipe, when reader_process is done
  * - 0.48 - added additional options
  *          split scanmode and bit-depth
+ * - 0.49 - improved multi-device capability
+ *        - tweaked some device settings
+ *        - added button support
+ *        - moved AFE stuff to enhanced options
+ * - 0.50 - cleanup
+ *        - activated IPC stuff
+ *        - added _DBG_DCALDATA for fine calibration data logging
+ *        - added OPT_SPEEDUP handling
  *.
  * <hr>
  * This file is part of the SANE package.
@@ -146,12 +154,16 @@
 #include "../include/sane/sanei.h"
 #include "../include/sane/saneopts.h"
 
-#define BACKEND_VERSION "0.48-10"
+#define BACKEND_VERSION "0.50-7"
+
 #define BACKEND_NAME    plustek
+#include "../include/sane/sanei_access.h"
 #include "../include/sane/sanei_backend.h"
 #include "../include/sane/sanei_config.h"
 #include "../include/sane/sanei_thread.h"
 
+#define USE_IPC
+
 #include "plustek-usb.h"
 #include "plustek.h"
 
@@ -164,8 +176,9 @@
 #define _DBG_PROC       7
 #define _DBG_SANE_INIT 10
 #define _DBG_INFO2     15
-#define _DBG_DREGS     20        
-#define _DBG_DPIC      25        
+#define _DBG_DREGS     20
+#define _DBG_DCALDATA  22
+#define _DBG_DPIC      25
 #define _DBG_READ      30
 
 /*****************************************************************************/
@@ -177,7 +190,8 @@
 #define _PLUSTEK_USB
 
 /* declare it here, as it's used in plustek-usbscan.c too :-( */
-static SANE_Bool cancelRead;
+static SANE_Bool  cancelRead;
+static DevList   *usbDevs;
 
 /* the USB-stuff... I know this is in general no good idea, but it works */
 #ifdef _PLUSTEK_USB
@@ -243,7 +257,7 @@ static SANE_Auth_Callback auth = NULL;
  * @param cnf - pointer to the configuration structure whose content should be
  *              displayed
  */
-static void show_cnf( pCnfDef cnf )
+static void show_cnf( CnfDef *cnf )
 {
 	DBG( _DBG_SANE_INIT,"Device configuration:\n" );
 	DBG( _DBG_SANE_INIT,"device name  : >%s<\n",cnf->devName                 );
@@ -302,6 +316,7 @@ drvclose( Plustek_Device *dev )
 		/* don't check the return values, simply do it */
 		usbDev_stopScan( dev );
 		usbDev_close   ( dev );
+		sanei_access_unlock( dev->sane.name );
 	}
 	dev->fd = -1;
 
@@ -402,6 +417,10 @@ static int reader_process( void *args )
 	struct SIGACTION act;
 	sigset_t         ignore_set;
 	Plustek_Scanner *scanner = (Plustek_Scanner *)args;
+#ifdef USE_IPC
+	IPCDef           ipc;
+	Plustek_Device  *dev = scanner->hw;
+#endif
 
 	if( sanei_thread_is_forked()) {
 		DBG( _DBG_PROC, "reader_process started (forked)\n" );
@@ -444,10 +463,22 @@ static int reader_process( void *args )
 		return SANE_STATUS_IO_ERROR;
 	}
 
-	/* here we read all data from the driver... */
+	/* prepare for scanning: speed-test, warmup, calibration */
 	buf    = scanner->buf;
 	status = usbDev_Prepare( scanner->hw, buf );
 
+#ifdef USE_IPC
+	/* prepare IPC structure */
+	memset( &ipc, 0, sizeof(ipc));
+	ipc.transferRate = DEFAULT_RATE;
+
+	if( dev->transferRate > 0 && dev->transferRate != DEFAULT_RATE )
+		ipc.transferRate = dev->transferRate;
+
+	/* write ipc back to parent in any case... */
+	write( scanner->w_pipe, &ipc, sizeof(ipc));
+#endif
+	/* on success, we read all data from the driver... */
 	if( 0 == status ) {
 
 		for( line = 0; line < scanner->params.lines; line++ ) {
@@ -468,7 +499,7 @@ static int reader_process( void *args )
 	scanner->w_pipe = -1;
 
 	if((int)status < 0 ) {
-		DBG( _DBG_ERROR, "read failed, status = %i, errno %i\n",
+		DBG( _DBG_ERROR,"reader_process: read failed, status = %i, errno %i\n",
                                                           (int)status, lerrn );
 		if( _E_ABORT == (int)status )
 			return SANE_STATUS_CANCELLED;
@@ -524,6 +555,7 @@ static SANE_Status do_cancel( Plustek_Scanner *scanner, SANE_Bool closepipe  )
 			sanei_thread_sendsig( scanner->reader_pid, SIGKILL );
 #endif
 		}
+
 		scanner->reader_pid = 0;
 		DBG( _DBG_PROC,"reader_process killed\n");
 #ifndef HAVE_SETITIMER
@@ -577,8 +609,8 @@ static SANE_Status initGammaSettings( Plustek_Scanner *s )
 		for( j = 0; j < s->gamma_length; j++ ) {
 
 			val = (s->gamma_range.max *
-			            pow((double) j / ((double)s->gamma_length - 1.0),
-			            1.0 / gamma ));
+			       pow((double)j / (double)(s->gamma_length-1.0),
+			       1.0 / gamma ));
 
 			if( val > s->gamma_range.max )
 				val = s->gamma_range.max;
@@ -633,7 +665,6 @@ static SANE_Status init_options( Plustek_Scanner *s )
 	s->val[OPT_NUM_OPTS].w 	   = NUM_OPTIONS;
 
 	/* "Scan Mode" group: */
-	s->opt[OPT_MODE_GROUP].name  = "scanmode-group";
 	s->opt[OPT_MODE_GROUP].title = SANE_I18N("Scan Mode");
 	s->opt[OPT_MODE_GROUP].desc  = "";
 	s->opt[OPT_MODE_GROUP].type  = SANE_TYPE_GROUP;
@@ -718,7 +749,6 @@ static SANE_Status init_options( Plustek_Scanner *s )
 	s->val[OPT_PREVIEW].w     = 0;
 
 	/* "Geometry" group: */
-	s->opt[OPT_GEOMETRY_GROUP].name  = "geometry-group";
 	s->opt[OPT_GEOMETRY_GROUP].title = SANE_I18N("Geometry");
 	s->opt[OPT_GEOMETRY_GROUP].desc  = "";
 	s->opt[OPT_GEOMETRY_GROUP].type  = SANE_TYPE_GROUP;
@@ -828,7 +858,6 @@ static SANE_Status init_options( Plustek_Scanner *s )
 		s->opt[OPT_EXT_MODE].cap |= SANE_CAP_INACTIVE;
 
 	/* "Device settings" group: */
-	s->opt[OPT_DEVICE_GROUP].name  = "device-settings";
 	s->opt[OPT_DEVICE_GROUP].title = SANE_I18N("Device-Settings");
 	s->opt[OPT_DEVICE_GROUP].desc  = "";
 	s->opt[OPT_DEVICE_GROUP].type  = SANE_TYPE_GROUP;
@@ -847,6 +876,15 @@ static SANE_Status init_options( Plustek_Scanner *s )
 	s->opt[OPT_CACHECAL].type  = SANE_TYPE_BOOL;
 	s->val[OPT_CACHECAL].w     = adj->cacheCalData;
 
+	s->opt[OPT_SPEEDUP].name  = "speedup-switch";
+	s->opt[OPT_SPEEDUP].title = SANE_I18N("Speedup sensor");;
+	s->opt[OPT_SPEEDUP].desc  = SANE_I18N("Enables or disables speeding up sensor movement.");
+	s->opt[OPT_SPEEDUP].type  = SANE_TYPE_BOOL;
+	s->val[OPT_SPEEDUP].w     = !(adj->disableSpeedup);
+
+	if( s->hw->usbDev.HwSetting.dHighSpeed == 0.0 )
+		s->opt[OPT_SPEEDUP].cap |= SANE_CAP_INACTIVE;
+
 	s->opt[OPT_LAMPOFF_ONEND].name  = SANE_NAME_LAMP_OFF_AT_EXIT;
 	s->opt[OPT_LAMPOFF_ONEND].title = SANE_TITLE_LAMP_OFF_AT_EXIT;
 	s->opt[OPT_LAMPOFF_ONEND].desc  = SANE_DESC_LAMP_OFF_AT_EXIT;
@@ -870,11 +908,10 @@ static SANE_Status init_options( Plustek_Scanner *s )
 	s->val[OPT_LAMPOFF_TIMER].w     = adj->lampOff;
 
 	/* "Analog Frontend" group*/
-	s->opt[OPT_AFE_GROUP].name  = "afe-group";
 	s->opt[OPT_AFE_GROUP].title = SANE_I18N("Analog frontend");
 	s->opt[OPT_AFE_GROUP].desc  = "";
 	s->opt[OPT_AFE_GROUP].type  = SANE_TYPE_GROUP;
-	s->opt[OPT_AFE_GROUP].cap   = 0;
+	s->opt[OPT_AFE_GROUP].cap   = SANE_CAP_ADVANCED;
 	
 	s->opt[OPT_OVR_REDGAIN].name  = "red-gain";
 	s->opt[OPT_OVR_REDGAIN].title = SANE_I18N("Red gain");
@@ -955,6 +992,31 @@ static SANE_Status init_options( Plustek_Scanner *s )
 		s->opt[OPT_OVR_BLUE_LOFF].cap  |= SANE_CAP_INACTIVE;
 	}
 
+	/* "Button" group*/
+	s->opt[OPT_BUTTON_GROUP].title = SANE_I18N("Buttons");
+	s->opt[OPT_BUTTON_GROUP].desc  = "";
+	s->opt[OPT_BUTTON_GROUP].type  = SANE_TYPE_GROUP;
+	s->opt[OPT_BUTTON_GROUP].cap   = SANE_CAP_ADVANCED;
+
+	/* scanner buttons */
+	for( i = OPT_BUTTON_0; i <= OPT_BUTTON_LAST; i++ ) {
+		s->opt[i].name  = "button";
+		s->opt[i].title = SANE_I18N("Scanner button");
+		s->opt[i].desc  = SANE_I18N("This option reflects the status "
+		                            "of the scanner buttons.");
+		s->opt[i].type = SANE_TYPE_BOOL;
+		s->opt[i].cap  = SANE_CAP_SOFT_DETECT | SANE_CAP_ADVANCED;
+		if (i - OPT_BUTTON_0 >= dev->usbDev.Caps.bButtons )
+			s->opt[i].cap |= SANE_CAP_INACTIVE;
+
+		s->opt[i].unit = SANE_UNIT_NONE;
+		s->opt[i].size = sizeof (SANE_Word);
+		s->opt[i].constraint_type = SANE_CONSTRAINT_RANGE;
+		s->opt[i].constraint.range = 0;
+		s->val[i].w = SANE_FALSE;
+	}
+
+	usb_UpdateButtonStatus( s );
 	return SANE_STATUS_GOOD;
 }
 
@@ -1115,7 +1177,7 @@ static SANE_Bool decodeDevName( char *src, char *dest )
 /** attach a device to the backend
  */
 static SANE_Status attach( const char *dev_name,
-                           pCnfDef cnf, Plustek_Device **devp )
+                           CnfDef *cnf, Plustek_Device **devp )
 {
 	int             cntr;
 	int             result;
@@ -1150,6 +1212,7 @@ static SANE_Status attach( const char *dev_name,
 	dev->sane.vendor  = "Plustek";
 	dev->initialized  = -1;                  /* will be used as index too */
 	dev->calFile      = NULL;
+	dev->transferRate = DEFAULT_RATE;
 
 	memcpy( &dev->adj, &cnf->adj, sizeof(AdjDef));
 
@@ -1173,7 +1236,7 @@ static SANE_Status attach( const char *dev_name,
 #endif
 
 	/* go ahead and open the scanner device */
-	handle = usbDev_open( dev );
+	handle = usbDev_open( dev, usbDevs, SANE_FALSE );
 	if( handle < 0 ) {
 		DBG( _DBG_ERROR,"open failed: %d\n", handle );
 		return SANE_STATUS_IO_ERROR;
@@ -1213,7 +1276,7 @@ static SANE_Status attach( const char *dev_name,
 	                 sizeof (SANE_Int));  
 
 	if (NULL == dev->res_list) {
-		DBG( _DBG_ERROR, "alloc fail, resolution problem\n" );
+		DBG( _DBG_ERROR, "calloc failed: %s\n", strerror(errno));
 		usbDev_close(dev);
 		return SANE_STATUS_INVAL;
 	}
@@ -1226,10 +1289,10 @@ static SANE_Status attach( const char *dev_name,
 	}
 
 	/* set the limits */
-	dev->dpi_range.min   = _DEF_DPI;
-	dev->dpi_range.max   = dev->usbDev.Caps.OpticDpi.x * 2;
-	dev->x_range.max     = SANE_FIX(dev->max_x);
-	dev->y_range.max     = SANE_FIX(dev->max_y);
+	dev->dpi_range.min = _DEF_DPI;
+	dev->dpi_range.max = dev->usbDev.Caps.OpticDpi.x * 2;
+	dev->x_range.max   = SANE_FIX(dev->max_x);
+	dev->y_range.max   = SANE_FIX(dev->max_y);
 
 	dev->fd = handle;
 	drvclose( dev );
@@ -1249,7 +1312,7 @@ static SANE_Status attach( const char *dev_name,
 /** function to preset a configuration structure
  * @param cnf - pointer to the structure that should be initialized
  */
-static void init_config_struct( pCnfDef cnf )
+static void init_config_struct( CnfDef *cnf )
 {
 	memset( cnf, 0, sizeof(CnfDef));
 
@@ -1293,6 +1356,7 @@ sane_init( SANE_Int *version_code, SANE_Auth_Callback authorize )
 	sanei_lm983x_init();
 #endif	
 	sanei_thread_init();
+	sanei_access_init(STRINGIFY(BACKEND_NAME));
 
 #if defined PACKAGE && defined VERSION
 	DBG( _DBG_INFO, "Plustek backend V"BACKEND_VERSION", part of "
@@ -1306,10 +1370,14 @@ sane_init( SANE_Int *version_code, SANE_Auth_Callback authorize )
 	first_dev    = NULL;
 	first_handle = NULL;
 	num_devices  = 0;
+	usbDevs      = NULL;
 
 	/* initialize the configuration structure */                
 	init_config_struct( &config );
 
+	/* try and get a list of all connected AND supported devices */
+	usbGetList( &usbDevs );
+
 	if( version_code != NULL )
 		*version_code = SANE_VERSION_CODE(V_MAJOR, V_MINOR, 0);
 
@@ -1321,7 +1389,7 @@ sane_init( SANE_Int *version_code, SANE_Auth_Callback authorize )
 	}
 
 	while( sanei_config_read( str, sizeof(str), fp)) {
-		
+
 		DBG( _DBG_SANE_INIT, ">%s<\n", str );
 		if( str[0] == '#')		/* ignore line comments */
 			continue;
@@ -1335,7 +1403,7 @@ sane_init( SANE_Int *version_code, SANE_Auth_Callback authorize )
 
 			int    ival;
 			double dval;
-		
+
 			ival = -1;
 			decodeVal( str, "warmup",    _INT, &config.adj.warmup,      &ival);
 			decodeVal( str, "lampOff",   _INT, &config.adj.lampOff,     &ival);
@@ -1352,7 +1420,7 @@ sane_init( SANE_Int *version_code, SANE_Auth_Callback authorize )
 			decodeVal( str, "red_lampoff",   _INT, &config.adj.rlampoff,&ival);
 			decodeVal( str, "green_lampoff", _INT, &config.adj.glampoff,&ival);
 			decodeVal( str, "blue_lampoff",  _INT, &config.adj.blampoff,&ival);
-			
+
 			ival = 0;
 			decodeVal( str, "enableTPA", _INT, &config.adj.enableTpa, &ival);
 			decodeVal( str, "cacheCalData",
@@ -1390,9 +1458,9 @@ sane_init( SANE_Int *version_code, SANE_Auth_Callback authorize )
 
 		/* check for sections: */
 		} else if( 0 == strncmp( str, _SECTION, strlen(_SECTION))) {
-		
+
 		    char *tmp;
-		
+
 		    /* new section, try and attach previous device */
 		    if( config.devName[0] != '\0' ) {
 				attach( config.devName, &config, 0 );
@@ -1402,7 +1470,7 @@ sane_init( SANE_Int *version_code, SANE_Auth_Callback authorize )
 					                   " ignored!\n" );
 				 }
 			}
-		
+
 			/* re-initialize the configuration structure */
 			init_config_struct( &config );
 		
@@ -1410,12 +1478,12 @@ sane_init( SANE_Int *version_code, SANE_Auth_Callback authorize )
 			decodeUsbIDs( str, &tmp );
 		
 			DBG( _DBG_SANE_INIT, "... next device\n" );
-			continue;				
+			continue;
 
 		} else if( SANE_TRUE == decodeDevName( str, config.devName )) {
 			continue;
 		}
-		
+
 		/* ignore other stuff... */
 		DBG( _DBG_SANE_INIT, "ignoring >%s<\n", str );
 	}
@@ -1433,6 +1501,7 @@ sane_init( SANE_Int *version_code, SANE_Auth_Callback authorize )
 void
 sane_exit( void )
 {
+	DevList        *tmp;
 	Plustek_Device *dev, *next;
 
 	DBG( _DBG_SANE_INIT, "sane_exit\n" );
@@ -1463,6 +1532,13 @@ sane_exit( void )
 	if( devlist )
 		free( devlist );
 
+	while( usbDevs ) {
+		tmp = usbDevs->next;
+		free( usbDevs );
+		usbDevs = tmp;
+	}
+
+	usbDevs      = NULL;
 	devlist      = NULL;
 	auth         = NULL;
 	first_dev    = NULL;
@@ -1482,15 +1558,15 @@ sane_get_devices(const SANE_Device ***device_list, SANE_Bool local_only )
 
 	/* already called, so cleanup */
 	if( devlist )
-    	free( devlist );
+		free( devlist );
 
 	devlist = malloc((num_devices + 1) * sizeof (devlist[0]));
 	if ( NULL == devlist )
-    	return SANE_STATUS_NO_MEM;
+		return SANE_STATUS_NO_MEM;
 
 	i = 0;
 	for (dev = first_dev; i < num_devices; dev = dev->next)
-    	devlist[i++] = &dev->sane;
+		devlist[i++] = &dev->sane;
 	devlist[i++] = 0;
 
 	*device_list = devlist;
@@ -1505,20 +1581,20 @@ sane_open( SANE_String_Const devicename, SANE_Handle* handle )
 	SANE_Status      status;
 	Plustek_Device  *dev;
 	Plustek_Scanner *s;
-    CnfDef           config;
+	CnfDef           config;
 
 	DBG( _DBG_SANE_INIT, "sane_open - %s\n", devicename );
 
 	if( devicename[0] ) {
-    	for( dev = first_dev; dev; dev = dev->next ) {
+		for( dev = first_dev; dev; dev = dev->next ) {
 			if( strcmp( dev->sane.name, devicename ) == 0 )
 				break;
-        }
+		}
 
 		if( !dev ) {
 
 			memset( &config, 0, sizeof(CnfDef));
-			
+
 			status = attach( devicename, &config, &dev );
 			if( SANE_STATUS_GOOD != status )
 				return status;
@@ -1529,11 +1605,11 @@ sane_open( SANE_String_Const devicename, SANE_Handle* handle )
 	}
 
 	if( !dev )
-    	return SANE_STATUS_INVAL;
+		return SANE_STATUS_INVAL;
 
 	s = malloc (sizeof (*s));
 	if( NULL == s )
-    	return SANE_STATUS_NO_MEM;
+		return SANE_STATUS_NO_MEM;
 
 	memset(s, 0, sizeof (*s));
 	s->r_pipe   = -1;
@@ -1546,9 +1622,8 @@ sane_open( SANE_String_Const devicename, SANE_Handle* handle )
 	/* insert newly opened handle into list of open handles: */
 	s->next      = first_handle;
 	first_handle = s;
+	*handle      = s;
 
-	*handle = s;
-	
 	return SANE_STATUS_GOOD;
 }
 
@@ -1643,6 +1718,7 @@ sane_control_option( SANE_Handle handle, SANE_Int option,
 			case OPT_CUSTOM_GAMMA:
 			case OPT_LAMPOFF_ONEND:
 			case OPT_CACHECAL:
+			case OPT_SPEEDUP:
 			case OPT_OVR_REDGAIN:
 			case OPT_OVR_GREENGAIN:
 			case OPT_OVR_BLUEGAIN:
@@ -1657,6 +1733,18 @@ sane_control_option( SANE_Handle handle, SANE_Int option,
 				*(SANE_Word *)value = s->val[option].w;
 				break;
 
+			case OPT_BUTTON_0:
+				usb_UpdateButtonStatus( s );
+			case OPT_BUTTON_1:
+			case OPT_BUTTON_2:
+			case OPT_BUTTON_3:
+			case OPT_BUTTON_4:
+				/* copy the button state */
+				*(SANE_Word*)value = s->val[option].w;
+				/* clear the button state */
+				s->val[option].w = SANE_FALSE;
+				break;
+
 			case OPT_CONTRAST:
 			case OPT_BRIGHTNESS:
 				*(SANE_Word *)value =
@@ -1755,6 +1843,11 @@ sane_control_option( SANE_Handle handle, SANE_Int option,
 					dev->adj.cacheCalData = s->val[option].w;
 					break;
 
+				case OPT_SPEEDUP:
+					s->val[option].w = *(SANE_Word *)value;
+					dev->adj.disableSpeedup = !(s->val[option].w);
+					break;
+
 				case OPT_LAMPSWITCH:
 					s->val[option].w = *(SANE_Word *)value;
 					usb_LampSwitch( dev, s->val[option].w );
@@ -2114,9 +2207,9 @@ sane_start( SANE_Handle handle )
 
 	/* open the driver and get some information about the scanner
 	 */
-	dev->fd = usbDev_open( dev );
+	dev->fd = usbDev_open( dev, NULL, SANE_TRUE );
 	if( dev->fd < 0 ) {
-		DBG( _DBG_ERROR,"sane_start: open failed: %d\n", errno );
+		DBG( _DBG_ERROR, "sane_start: open failed: %d\n", errno);
 
 		if( errno == EBUSY )
 			return SANE_STATUS_DEVICE_BUSY;
@@ -2127,6 +2220,7 @@ sane_start( SANE_Handle handle )
 	result = usbDev_getCaps( dev );
 	if( result < 0 ) {
 		DBG( _DBG_ERROR, "usbDev_getCaps() failed(%d)\n", result);
+		sanei_access_unlock( dev->sane.name );
 		usbDev_close( dev );
 		return SANE_STATUS_IO_ERROR;
 	}
@@ -2192,6 +2286,7 @@ sane_start( SANE_Handle handle )
 	if( result < 0 ) {
 		DBG( _DBG_ERROR, "usbDev_getCropInfo() failed(%d)\n", result );
 		usbDev_close( dev );
+		sanei_access_unlock( dev->sane.name );
 		return SANE_STATUS_IO_ERROR;
 	}
 
@@ -2220,6 +2315,7 @@ sane_start( SANE_Handle handle )
 	if( result < 0 ) {
 		DBG( _DBG_ERROR, "usbDev_setScanEnv() failed(%d)\n", result );
 		usbDev_close( dev );
+		sanei_access_unlock( dev->sane.name );
 		return SANE_STATUS_IO_ERROR;
 	}
 
@@ -2238,6 +2334,7 @@ sane_start( SANE_Handle handle )
 	if( result < 0 ) {
 		DBG( _DBG_ERROR, "usbDev_startScan() failed(%d)\n", result );
 		usbDev_close( dev );
+		sanei_access_unlock( dev->sane.name );
 		return SANE_STATUS_IO_ERROR;
 	}
 
@@ -2250,6 +2347,7 @@ sane_start( SANE_Handle handle )
 	if( NULL == s->buf ) {
 		DBG( _DBG_ERROR, "realloc failed\n" );
 		usbDev_close( dev );
+		sanei_access_unlock( dev->sane.name );
 		return SANE_STATUS_NO_MEM;
 	}
 
@@ -2270,10 +2368,11 @@ sane_start( SANE_Handle handle )
 	}
 
 	/* create reader routine as new process */
-	s->bytes_read = 0;
-	s->r_pipe     = fds[0];
-	s->w_pipe     = fds[1];
-	s->reader_pid = sanei_thread_begin( reader_process, s );
+	s->bytes_read    = 0;
+	s->r_pipe        = fds[0];
+	s->w_pipe        = fds[1];
+	s->ipc_read_done = SANE_FALSE;
+	s->reader_pid    = sanei_thread_begin( reader_process, s );
 
 	cancelRead = SANE_FALSE;
 	
@@ -2303,9 +2402,43 @@ sane_read( SANE_Handle handle, SANE_Byte *data,
 {
 	Plustek_Scanner *s = (Plustek_Scanner*)handle;
 	ssize_t          nread;
+#ifdef USE_IPC
+	static	 IPCDef       ipc;
+	unsigned char        *buf;
+	static unsigned long  c = 0;
+#endif
 
 	*length = 0;
 
+#ifdef USE_IPC
+	/* first try and read IPC... */
+	if( !s->ipc_read_done ) {
+
+		buf = (unsigned char*)&ipc;
+		for( c = 0; c < sizeof(ipc); ) {
+			nread = read( s->r_pipe, buf, sizeof(ipc));
+			if( nread < 0 ) {
+				if( EAGAIN != errno ) {
+					do_cancel( s, SANE_TRUE );
+					return SANE_STATUS_IO_ERROR;
+				} else {
+					return SANE_STATUS_GOOD;
+				}
+			} else {
+				c   += nread;
+				buf += nread;
+				if( c == sizeof(ipc)) {
+					s->ipc_read_done = SANE_TRUE;
+					break;
+				}
+			}
+		}
+		s->hw->transferRate = ipc.transferRate;
+		DBG( _DBG_INFO, "IPC: Transferrate = %lu Bytes/s\n", 
+		     ipc.transferRate );
+	}
+#endif
+
 	/* here we read all data from the driver... */
 	nread = read( s->r_pipe, data, max_length );
 	DBG( _DBG_READ, "sane_read - read %ld bytes\n", (long)nread );
@@ -2388,7 +2521,7 @@ sane_set_io_mode( SANE_Handle handle, SANE_Bool non_blocking )
 	}
 	
 	if( fcntl (s->r_pipe, F_SETFL, non_blocking ? O_NONBLOCK : 0) < 0) {
-		DBG( _DBG_ERROR, "ERROR: cant set to non-blocking mode !\n" );
+		DBG( _DBG_ERROR, "ERROR: could not set to non-blocking mode !\n" );
 		return SANE_STATUS_IO_ERROR;
 	}
 
