@@ -1,6 +1,6 @@
 /* sane - Scanner Access Now Easy.
 
-   Copyright (C) 2002 Frank Zago (fzago at austin dot rr dot com)
+   Copyright (C) 2002 Frank Zago (sane at zago dot net)
    Copyright (C) 2002 Other SANE contributors
    
    This file is part of the SANE package.
@@ -43,7 +43,7 @@
 */
 
 /* 
-   $Id: sceptre.c,v 1.10 2002/11/19 05:01:21 fzago Exp $
+   $Id: sceptre.c,v 1.13 2005/09/19 19:57:48 fzago-guest Exp $
    Sceptre S1200 SCSI scanner (sometimes also called S120)
 */
 
@@ -521,13 +521,10 @@ sceptre_adjust_raster (Sceptre_Scanner * dev, size_t size_in)
  * Since this scanner does not have REQUEST SENSE, it is always an
  * error if this function is called.*/
 static SANE_Status
-sceptre_sense_handler (int scsi_fd, unsigned char *result, void *arg)
+sceptre_sense_handler (int scsi_fd, unsigned char __sane_unused__ *result, void __sane_unused__ *arg)
 {
   DBG (DBG_proc, "sceptre_sense_handler (scsi_fd = %d)\n", scsi_fd);
 
-  result = result;		/* silence gcc */
-  arg = arg;			/* silence gcc */
-
   return SANE_STATUS_IO_ERROR;
 }
 
@@ -1310,7 +1307,7 @@ sceptre_send_gamma (Sceptre_Scanner * dev)
 /* Entry points */
 
 SANE_Status
-sane_init (SANE_Int * version_code, SANE_Auth_Callback authorize)
+sane_init (SANE_Int * version_code, SANE_Auth_Callback __sane_unused__ authorize)
 {
   FILE *fp;
   char dev_name[PATH_MAX];
@@ -1320,8 +1317,6 @@ sane_init (SANE_Int * version_code, SANE_Auth_Callback authorize)
 
   DBG (DBG_proc, "sane_init: enter\n");
 
-  authorize = authorize;	/* silence gcc */
-
   DBG (DBG_error, "This is sane-sceptre version %d.%d-%d\n", V_MAJOR,
        V_MINOR, BUILD);
   DBG (DBG_error, "(C) 2002 by Frank Zago\n");
@@ -1359,15 +1354,13 @@ sane_init (SANE_Int * version_code, SANE_Auth_Callback authorize)
 }
 
 SANE_Status
-sane_get_devices (const SANE_Device *** device_list, SANE_Bool local_only)
+sane_get_devices (const SANE_Device *** device_list, SANE_Bool __sane_unused__ local_only)
 {
   Sceptre_Scanner *dev;
   int i;
 
   DBG (DBG_proc, "sane_get_devices: enter\n");
 
-  local_only = local_only;	/* silence gcc */
-
   if (devlist)
     free (devlist);
 
@@ -1998,16 +1991,13 @@ sane_read (SANE_Handle handle, SANE_Byte * buf, SANE_Int max_len,
 }
 
 SANE_Status
-sane_set_io_mode (SANE_Handle handle, SANE_Bool non_blocking)
+sane_set_io_mode (SANE_Handle __sane_unused__ handle, SANE_Bool __sane_unused__ non_blocking)
 {
   SANE_Status status;
   Sceptre_Scanner *dev = handle;
 
   DBG (DBG_proc, "sane_set_io_mode: enter\n");
 
-  handle = handle;		/* silence gcc */
-  non_blocking = non_blocking;	/* silence gcc */
-
   if (dev->scanning == SANE_FALSE)
     {
       return (SANE_STATUS_INVAL);
@@ -2028,13 +2018,10 @@ sane_set_io_mode (SANE_Handle handle, SANE_Bool non_blocking)
 }
 
 SANE_Status
-sane_get_select_fd (SANE_Handle handle, SANE_Int * fd)
+sane_get_select_fd (SANE_Handle __sane_unused__ handle, SANE_Int __sane_unused__ * fd)
 {
   DBG (DBG_proc, "sane_get_select_fd: enter\n");
 
-  handle = handle;		/* silence gcc */
-  fd = fd;			/* silence gcc */
-
   DBG (DBG_proc, "sane_get_select_fd: exit\n");
 
   return SANE_STATUS_UNSUPPORTED;
