@@ -46,7 +46,7 @@
    This file is a component of the implementation of a backend for many
    of the the AGFA SnapScan and Acer Vuego/Prisa flatbed scanners. */
 
-/* $Id: snapscan-options.c,v 1.12 2003/11/09 21:43:45 oliver-guest Exp $
+/* $Id: snapscan-options.c,v 1.33 2005/12/04 15:03:00 oliver-guest Exp $
    SANE SnapScan backend */
 
 /* default option values */
@@ -66,6 +66,7 @@
 
 static SANE_Int def_rgb_lpr = 4;
 static SANE_Int def_gs_lpr = 12;
+static SANE_Int def_bpp = 8;
 
 
 /* predefined preview mode name */
@@ -154,6 +155,35 @@ static const SANE_Range y_range_tpo_1670 =
     SANE_FIX (0.0), SANE_FIX (228.0), 0
 };        /* mm */
 
+/* TPO range for the Epson 2480 */
+static const SANE_Range x_range_tpo_2480 =
+{
+    SANE_FIX (0.0), SANE_FIX (55.0), 0
+};        /* mm */
+static const SANE_Range y_range_tpo_2480 =
+{
+    SANE_FIX (0.0), SANE_FIX (125.0), 0
+};        /* mm */
+/* TPO range for the Epson 2580 */
+static const SANE_Range x_range_tpo_2580 =
+{
+    SANE_FIX (0.0), SANE_FIX (55.0), 0
+};        /* mm */
+static const SANE_Range y_range_tpo_2580 =
+{
+    SANE_FIX (0.0), SANE_FIX (80.0), 0
+};        /* mm */
+
+/* TPO range for the Epson 3490 */
+static const SANE_Range x_range_tpo_3490 =
+{
+    SANE_FIX (0.0), SANE_FIX (33.0), 0
+};        /* mm */
+static const SANE_Range y_range_tpo_3490 =
+{
+    SANE_FIX (0.0), SANE_FIX (162.0), 0
+};        /* mm */
+
 static SANE_Range x_range_tpo;
 static SANE_Range y_range_tpo;
 static const SANE_Range gamma_range =
@@ -162,7 +192,7 @@ static const SANE_Range gamma_range =
 };
 static const SANE_Range gamma_vrange =
 {
-    0, 255, 1
+    0, 65535, 1
 };
 static const SANE_Range lpr_range =
 {
@@ -190,6 +220,8 @@ static const SANE_Range positive_percent_range =
     1 << SANE_FIXED_SCALE_SHIFT
 };
 
+static void control_options(SnapScan_Scanner *pss);
+
 /* init_options -- initialize the option set for a scanner; expects the
    scanner structure's hardware configuration byte (hconfig) to be valid.
 
@@ -205,8 +237,14 @@ static void init_options (SnapScan_Scanner * ps)
         {8, 50, 75, 100, 150, 200, 300, 450, 600};
     static SANE_Word resolutions_1200[] =
         {10, 50, 75, 100, 150, 200, 300, 450, 600, 900, 1200};
+    static SANE_Word resolutions_1200_5000e[] =
+        {9, 50, 75, 100, 150, 200, 300, 450, 600, 1200};
     static SANE_Word resolutions_1600[] =
-        {10, 50, 75, 100, 150, 200, 300, 450, 600, 900, 1200, 1600};
+        {10, 50, 75, 100, 150, 200, 300, 400, 600, 800, 1600};
+    static SANE_Word resolutions_2400[] =
+        {10, 50, 75, 100, 150, 200, 300, 400, 600, 1200, 2400};
+    static SANE_Word resolutions_3200[] =
+        {15, 50, 150, 200, 240, 266, 300, 350, 360, 400, 600, 720, 800, 1200, 1600, 3200};
     static SANE_String_Const names_all[] =
         {md_colour, md_bilevelcolour, md_greyscale, md_lineart, NULL};
     static SANE_String_Const names_basic[] =
@@ -215,6 +253,8 @@ static void init_options (SnapScan_Scanner * ps)
         {md_auto, md_colour, md_bilevelcolour, md_greyscale, md_lineart, NULL};
     static SANE_String_Const preview_names_basic[] =
         {md_auto, md_colour, md_greyscale, md_lineart, NULL};
+    static SANE_Int bit_depth_list[4];
+    int bit_depths;
     SANE_Option_Descriptor *po = ps->options;
 
     /* Initialize TPO range */
@@ -230,10 +270,27 @@ static void init_options (SnapScan_Scanner * ps)
         x_range_tpo = x_range_tpo_e50;
         y_range_tpo = y_range_tpo_e50;
         break;
+    case PERFECTION1270:
     case PERFECTION1670:
         x_range_tpo = x_range_tpo_1670;
         y_range_tpo = y_range_tpo_1670;
         break;
+    case PERFECTION2480:
+        if (ps->hconfig_epson & 0x20)
+        {
+           x_range_tpo = x_range_tpo_2580;
+           y_range_tpo = y_range_tpo_2580;
+        }
+        else
+        {
+           x_range_tpo = x_range_tpo_2480;
+           y_range_tpo = y_range_tpo_2480;
+        }
+        break;
+    case PERFECTION3490:
+        x_range_tpo = x_range_tpo_3490;
+        y_range_tpo = y_range_tpo_3490;
+        break;
     default:
         x_range_tpo = x_range_tpo_default;
         y_range_tpo = y_range_tpo_default;
@@ -280,13 +337,23 @@ static void init_options (SnapScan_Scanner * ps)
     case SNAPSCANE52:
     case PRISA5300:
     case PRISA1240:
-    case PRISA5000:
     case ARCUS1200:
         po[OPT_SCANRES].constraint.word_list = resolutions_1200;
         break;
+    case PRISA5000E:
+    case PRISA5000:
+    case PRISA5150:
+        po[OPT_SCANRES].constraint.word_list = resolutions_1200_5000e;
+        break;
     case PERFECTION1670:
         po[OPT_SCANRES].constraint.word_list = resolutions_1600;
         break;
+    case PERFECTION2480:
+        po[OPT_SCANRES].constraint.word_list = resolutions_2400;
+        break;
+    case PERFECTION3490:
+        po[OPT_SCANRES].constraint.word_list = resolutions_3200;
+        break;
     default:
         po[OPT_SCANRES].constraint.word_list = resolutions_600;
         break;
@@ -305,8 +372,8 @@ static void init_options (SnapScan_Scanner * ps)
     ps->preview = DEFAULT_PREVIEW;
 
     po[OPT_HIGHQUALITY].name = "high-quality";
-    po[OPT_HIGHQUALITY].title = SANE_I18N("High quality");
-    po[OPT_HIGHQUALITY].desc = SANE_I18N("Scan in (slower) high quality mode");
+    po[OPT_HIGHQUALITY].title = SANE_I18N("Quality scan");
+    po[OPT_HIGHQUALITY].desc = SANE_I18N("Highest quality but lower speed");
     po[OPT_HIGHQUALITY].type = SANE_TYPE_BOOL;
     po[OPT_HIGHQUALITY].unit = SANE_UNIT_NONE;
     po[OPT_HIGHQUALITY].size = sizeof (SANE_Word);
@@ -314,7 +381,13 @@ static void init_options (SnapScan_Scanner * ps)
         SANE_CAP_SOFT_SELECT | SANE_CAP_SOFT_DETECT | SANE_CAP_AUTOMATIC;
     po[OPT_HIGHQUALITY].constraint_type = SANE_CONSTRAINT_NONE;
     ps->highquality = DEFAULT_HIGHQUALITY;
-
+    if (ps->pdev->model == PERFECTION1270)
+    {
+        po[OPT_HIGHQUALITY].cap |= SANE_CAP_INACTIVE;
+        ps->val[OPT_QUALITY_CAL].b = SANE_TRUE;
+        ps->highquality=SANE_TRUE;
+    }
+    
     po[OPT_BRIGHTNESS].name = SANE_NAME_BRIGHTNESS;
     po[OPT_BRIGHTNESS].title = SANE_TITLE_BRIGHTNESS;
     po[OPT_BRIGHTNESS].desc = SANE_DESC_BRIGHTNESS;
@@ -349,6 +422,7 @@ static void init_options (SnapScan_Scanner * ps)
     {
     case SNAPSCAN310:
     case PRISA310:
+    case PERFECTION3490:
         po[OPT_MODE].constraint.string_list = names_basic;
         break;
     default:
@@ -375,6 +449,7 @@ static void init_options (SnapScan_Scanner * ps)
     {
     case SNAPSCAN310:
     case PRISA310:
+    case PERFECTION3490:
         po[OPT_PREVIEW_MODE].constraint.string_list = preview_names_basic;
         break;
     default:
@@ -489,6 +564,30 @@ static void init_options (SnapScan_Scanner * ps)
     po[OPT_ENHANCEMENT_GROUP].cap = 0;
     po[OPT_ENHANCEMENT_GROUP].constraint_type = SANE_CONSTRAINT_NONE;
 
+    /* bit depth */     
+    po[OPT_BIT_DEPTH].name  = SANE_NAME_BIT_DEPTH;
+    po[OPT_BIT_DEPTH].title = SANE_TITLE_BIT_DEPTH;
+    po[OPT_BIT_DEPTH].desc  = SANE_DESC_BIT_DEPTH;
+    po[OPT_BIT_DEPTH].type  = SANE_TYPE_INT;
+    po[OPT_BIT_DEPTH].unit  = SANE_UNIT_BIT;
+    po[OPT_BIT_DEPTH].size = sizeof (SANE_Word);    
+    po[OPT_BIT_DEPTH].constraint_type = SANE_CONSTRAINT_WORD_LIST;
+    po[OPT_BIT_DEPTH].cap = SANE_CAP_SOFT_SELECT | SANE_CAP_SOFT_DETECT;
+    bit_depths = 0;
+    bit_depth_list[++bit_depths] = def_bpp;
+    switch (ps->pdev->model)
+    {
+    case PERFECTION2480:
+    case PERFECTION3490:
+        bit_depth_list[++bit_depths] = 16;
+        break;
+    default:
+        break;
+    }    
+    bit_depth_list[0] = bit_depths;
+    po[OPT_BIT_DEPTH].constraint.word_list = bit_depth_list;
+    ps->val[OPT_BIT_DEPTH].w = def_bpp;
+    
     po[OPT_QUALITY_CAL].name = SANE_NAME_QUALITY_CAL;
     po[OPT_QUALITY_CAL].title = SANE_TITLE_QUALITY_CAL;
     po[OPT_QUALITY_CAL].desc = SANE_DESC_QUALITY_CAL;
@@ -504,6 +603,7 @@ static void init_options (SnapScan_Scanner * ps)
     if ((!(ps->hconfig & HCFG_CAL_ALLOWED))
         || (ps->pdev->model == SNAPSCANE52)
         || (ps->pdev->model == PERFECTION1670)
+        || (ps->pdev->model == PRISA5150)
         || (ps->pdev->model == PRISA5300)) {
         po[OPT_QUALITY_CAL].cap |= SANE_CAP_INACTIVE;
         ps->val[OPT_QUALITY_CAL].b = SANE_FALSE;
@@ -738,12 +838,13 @@ static void init_options (SnapScan_Scanner * ps)
     po[OPT_GS_LPR].constraint_type = SANE_CONSTRAINT_RANGE;
     po[OPT_GS_LPR].constraint.range = &lpr_range;
     ps->gs_lpr = def_gs_lpr;
+    control_options(ps);
 }
 
 const SANE_Option_Descriptor *sane_get_option_descriptor (SANE_Handle h,
                                                           SANE_Int n)
 {
-    DBG (DL_CALL_TRACE,
+    DBG (DL_OPTION_TRACE,
          "sane_snapscan_get_option_descriptor (%p, %ld)\n",
          (void *) h,
          (long) n);
@@ -769,7 +870,8 @@ static void control_options(SnapScan_Scanner *pss)
     pss->options[OPT_GAMMA_VECTOR_R].cap |= SANE_CAP_INACTIVE;
     pss->options[OPT_GAMMA_VECTOR_G].cap |= SANE_CAP_INACTIVE;
     pss->options[OPT_GAMMA_VECTOR_B].cap |= SANE_CAP_INACTIVE;
-
+    pss->options[OPT_BIT_DEPTH].cap  |= SANE_CAP_INACTIVE;
+    
     if ((pss->mode == MD_COLOUR) ||
         ((pss->mode == MD_BILEVELCOLOUR) && (pss->hconfig & HCFG_HT) &&
          pss->halftone))
@@ -822,6 +924,18 @@ static void control_options(SnapScan_Scanner *pss)
             pss->options[OPT_GAMMA_GS].cap &= ~SANE_CAP_INACTIVE;
         }
     }
+    if ((pss->mode == MD_GREYSCALE) || (pss->mode == MD_COLOUR))
+    {
+        switch(pss->pdev->model)
+        {
+        case PERFECTION2480:
+        case PERFECTION3490:
+            pss->options[OPT_BIT_DEPTH].cap &= ~SANE_CAP_INACTIVE;
+            break;
+        default:
+            break;
+        }
+    }        
 }
 
 SANE_Status sane_control_option (SANE_Handle h,
@@ -835,7 +949,7 @@ SANE_Status sane_control_option (SANE_Handle h,
     SnapScan_Device *pdev = pss->pdev;
     static SANE_Status status;
 
-    DBG (DL_CALL_TRACE,
+    DBG (DL_OPTION_TRACE,
         "%s (%p, %ld, %ld, %p, %p)\n",
         me,
         (void *) h,
@@ -957,6 +1071,9 @@ SANE_Status sane_control_option (SANE_Handle h,
         case OPT_GS_LPR:
             *(SANE_Int *) v = pss->gs_lpr;
             break;
+        case OPT_BIT_DEPTH:
+            *(SANE_Int *) v = pss->val[OPT_BIT_DEPTH].w;
+            break;        
         default:
             DBG (DL_MAJOR_ERROR,
                  "%s: invalid option number %ld\n",
@@ -1001,7 +1118,7 @@ SANE_Status sane_control_option (SANE_Handle h,
                 *i |= SANE_INFO_RELOAD_PARAMS;
             break;
         case OPT_HIGHQUALITY:
-            pss->preview = *(SANE_Bool *) v;
+            pss->highquality = *(SANE_Bool *) v;
             if (i)
                 *i |= SANE_INFO_RELOAD_PARAMS;
             break;
@@ -1366,6 +1483,11 @@ SANE_Status sane_control_option (SANE_Handle h,
         case OPT_GS_LPR:
             pss->gs_lpr = *(SANE_Int *) v;
             break;
+        case OPT_BIT_DEPTH:
+            pss->val[OPT_BIT_DEPTH].w = *(SANE_Int *) v;
+            if (i)
+                *i |= SANE_INFO_RELOAD_PARAMS;
+            break;
         default:
             DBG (DL_MAJOR_ERROR,
                  "%s: invalid option number %ld\n",
@@ -1373,21 +1495,21 @@ SANE_Status sane_control_option (SANE_Handle h,
                  (long) n);
             return SANE_STATUS_UNSUPPORTED;
         }
-        DBG (DL_DATA_TRACE, "%s: option %s set to value ",
+        DBG (DL_OPTION_TRACE, "%s: option %s set to value ",
              me, pss->options[n].name);
         switch (pss->options[n].type)
         {
         case SANE_TYPE_INT:
-            DBG (DL_VERBOSE, "%ld\n", (long) (*(SANE_Int *) v));
+            DBG (DL_OPTION_TRACE, "%ld\n", (long) (*(SANE_Int *) v));
             break;
         case SANE_TYPE_BOOL:
             {
                 char *valstr = (*(SANE_Bool *) v == SANE_TRUE)  ?  "TRUE"  :  "FALSE";
-                DBG (DL_DATA_TRACE, "%s\n", valstr);
+                DBG (DL_OPTION_TRACE, "%s\n", valstr);
             }
             break;
         default:
-            DBG (DL_DATA_TRACE, "other than an integer or boolean.\n");
+            DBG (DL_OPTION_TRACE, "other than an integer or boolean.\n");
             break;
         }
         break;
@@ -1468,6 +1590,9 @@ SANE_Status sane_control_option (SANE_Handle h,
         case OPT_GS_LPR:
             pss->gs_lpr = def_gs_lpr;
             break;
+        case OPT_BIT_DEPTH:
+            pss->val[OPT_BIT_DEPTH].w = def_bpp;
+            break;
         default:
             DBG (DL_MAJOR_ERROR,
                  "%s: invalid option number %ld\n",
@@ -1485,6 +1610,69 @@ SANE_Status sane_control_option (SANE_Handle h,
 
 /*
  * $Log: snapscan-options.c,v $
+ * Revision 1.33  2005/12/04 15:03:00  oliver-guest
+ * Some fixes for Benq 5150
+ *
+ * Revision 1.32  2005/11/23 20:57:01  oliver-guest
+ * Disable bilevel colour / halftoning for Epson 3490
+ *
+ * Revision 1.31  2005/11/17 23:47:10  oliver-guest
+ * Revert previous 'fix', disable 2400 dpi for Epson 3490, use 1600 dpi instead
+ *
+ * Revision 1.30  2005/11/15 20:11:18  oliver-guest
+ * Enabled quality calibration for the Epson 3490
+ *
+ * Revision 1.29  2005/10/31 21:08:47  oliver-guest
+ * Distinguish between Benq 5000/5000E/5000U
+ *
+ * Revision 1.28  2005/10/24 19:46:40  oliver-guest
+ * Preview and range fix for Epson 2480/2580
+ *
+ * Revision 1.27  2005/10/13 22:43:30  oliver-guest
+ * Fixes for 16 bit scan mode from Simon Munton
+ *
+ * Revision 1.26  2005/10/11 18:47:07  oliver-guest
+ * Fixes for Epson 3490 and 16 bit scan mode
+ *
+ * Revision 1.25  2005/09/28 22:09:26  oliver-guest
+ * Reenabled enhanced inquiry command for Epson scanners (duh\!)
+ *
+ * Revision 1.24  2005/09/28 21:33:10  oliver-guest
+ * Added 16 bit option for Epson scanners (untested)
+ *
+ * Revision 1.23  2005/08/16 20:15:10  oliver-guest
+ * Removed C++-style comment
+ *
+ * Revision 1.22  2005/08/15 18:06:37  oliver-guest
+ * Added support for Epson 3490/3590 (thanks to Matt Judge)
+ *
+ * Revision 1.21  2005/07/20 21:37:29  oliver-guest
+ * Changed TPO scanning area for 2480/2580, reenabled 2400 DPI for 2480/2580
+ *
+ * Revision 1.20  2005/05/22 11:50:24  oliver-guest
+ * Disabled 2400 DPI for Epson 2480
+ *
+ * Revision 1.19  2004/12/09 23:21:47  oliver-guest
+ * Added quality calibration for Epson 2480 (by Simon Munton)
+ *
+ * Revision 1.18  2004/12/01 22:12:02  oliver-guest
+ * Added support for Epson 1270
+ *
+ * Revision 1.17  2004/09/02 20:59:11  oliver-guest
+ * Added support for Epson 2480
+ *
+ * Revision 1.16  2004/04/08 21:53:10  oliver-guest
+ * Use sanei_thread in snapscan backend
+ *
+ * Revision 1.15  2004/04/02 20:19:23  oliver-guest
+ * Various bugfixes for gamma corretion (thanks to Robert Tsien)
+ *
+ * Revision 1.14  2004/02/01 13:32:26  oliver-guest
+ * Fixed resolutions for Epson 1670
+ *
+ * Revision 1.13  2003/11/28 23:23:18  oliver-guest
+ * Correct length of wordlist for resolutions_1600
+ *
  * Revision 1.12  2003/11/09 21:43:45  oliver-guest
  * Disabled quality calibration for Epson Perfection 1670
  *
