@@ -142,7 +142,8 @@ UMAX_Cmd;
 typedef enum
 {
   ASTRA_1220U = 0x0010,
-  ASTRA_2000U = 0x0030
+  ASTRA_2000U = 0x0030,
+  ASTRA_2100U = 0x0130
 }
 UMAX_Model;
 
@@ -369,7 +370,7 @@ cwrite (UMAX_Handle * scan, UMAX_Cmd cmd, size_t len, const unsigned char *data,
   static unsigned char *escaped = NULL;
   static size_t escaped_size = 0;
 
-  DBG (80, "cwrite: cmd = %d, len = %d\n", cmd, len);
+  DBG (80, "cwrite: cmd = %d, len = %lu\n", cmd, (u_long) len);
 
   CHK (usync (scan, cmd | CMD_WRITE, len));
 
@@ -413,7 +414,7 @@ cread (UMAX_Handle * scan, UMAX_Cmd cmd, size_t len, unsigned char *data,
   SANE_Status res;
   UMAX_Status_Byte s0, s4;
 
-  DBG (80, "cread: cmd = %d, len = %d\n", cmd, len);
+  DBG (80, "cread: cmd = %d, len = %lu\n", cmd, (u_long) len);
 
   CHK (usync (scan, cmd | CMD_READ, len));
 
@@ -432,7 +433,7 @@ cread (UMAX_Handle * scan, UMAX_Cmd cmd, size_t len, unsigned char *data,
           CHK (sanei_pv8630_bulkread (scan->fd, data, &n));
           if (n < req)
             {
-              DBG (1, "qread: Expecting to read %d, only got %d\n", req, n);
+              DBG (1, "qread: Expecting to read %lu, only got %lu\n", (u_long) req, (u_long) n);
               return SANE_STATUS_IO_ERROR;
             }
           data += n;
@@ -1654,6 +1655,14 @@ UMAX_open_device (UMAX_Handle * scan, const char *dev)
            "UMAX_open_device: Scanner is a 2000U. Expect color problems :)\n");
       scan->model = ASTRA_2000U;
       break;
+     case ASTRA_2100U:
+      /* The UMAX Astra 2100U is only partially supported by
+         this driver. Expect severe color problems! :) [???]
+       */
+      DBG (1,
+           "UMAX_open_device: Scanner is a 2100U. Expect color problems :)\n");
+      scan->model = ASTRA_2100U;
+      break;
     case ASTRA_1220U:
       scan->model = ASTRA_1220U;
       break;
@@ -1687,6 +1696,8 @@ UMAX_get_device_name (UMAX_Handle * scan)
       return "Astra 1220U";
     case ASTRA_2000U:
       return "Astra 2000U";
+    case ASTRA_2100U:
+      return "Astra 2100U";
     }
   return "Unknown";
 }
