@@ -203,7 +203,7 @@ This option lets the user select a gray scale scan
 static SANE_Word optionGrayscaleValue = SANE_FALSE;
 
 static SANE_Option_Descriptor optionGrayscaleDescriptor = {
-  SANE_I18N ("gray"),
+  "gray",
   SANE_I18N ("Grayscale scan"),
   SANE_I18N ("Do a grayscale rather than color scan"),
   SANE_TYPE_BOOL,
@@ -243,7 +243,7 @@ lamp in the UMAX scanner
 */
 
 static SANE_Option_Descriptor optionLampOffDescriptor = {
-  SANE_I18N ("lamp-off"),
+  "lamp-off",
   SANE_I18N ("Lamp off"),
   SANE_I18N ("Turn off scanner lamp"),
   SANE_TYPE_BUTTON,
@@ -592,12 +592,13 @@ sane_init (SANE_Int * version_code, SANE_Auth_Callback authorize)
   DBG (2, "sane_init: version_code %s 0, authorize %s 0\n",
        version_code == 0 ? "=" : "!=", authorize == 0 ? "=" : "!=");
   DBG (1, "sane_init: SANE umax1220u backend version %d.%d.%d from %s\n",
-       V_MAJOR, V_MINOR, BUILD, PACKAGE_VERSION);
+       V_MAJOR, V_MINOR, BUILD, PACKAGE_STRING);
 
   if (version_code)
     *version_code = SANE_VERSION_CODE (V_MAJOR, V_MINOR, BUILD);
 
   sanei_usb_init ();
+  sanei_pv8630_init ();
 
   fp = sanei_config_open (UMAX_CONFIG_FILE);
   if (!fp)
