@@ -1,7 +1,8 @@
 /* sane - Scanner Access Now Easy.
    Copyright (C) 2003 Rene Rebe (sanei_read_int,sanei_set_timeout)
-   Copyright (C) 2001 - 2003 Henning Meier-Geinitz
+   Copyright (C) 2001 - 2005 Henning Meier-Geinitz
    Copyright (C) 2001 Frank Zago (sanei_usb_control_msg)
+   Copyright (C) 2005 Paul Smedley <paul@smedley.info> (OS/2 usbcalls)
    This file is part of the SANE package.
 
    This program is free software; you can redistribute it and/or
@@ -62,6 +63,34 @@
 #include <usb.h>
 #endif /* HAVE_LIBUSB */
 
+#ifdef HAVE_USBCALLS
+#include <usb.h>
+#include <os2.h>
+#include <usbcalls.h>
+#define MAX_RW 64000
+static int usbcalls_timeout = 30 * 1000;	/* 30 seconds */
+USBHANDLE dh;
+PHEV pUsbIrqStartHev=NULL;
+
+static
+struct usb_descriptor_header *GetNextDescriptor( struct usb_descriptor_header *currHead, UCHAR  *lastBytePtr)
+{
+  UCHAR    *currBytePtr, *nextBytePtr;
+
+  if (!currHead->bLength)
+     return (NULL);
+  currBytePtr=(UCHAR *)currHead;
+  nextBytePtr=currBytePtr+currHead->bLength;
+  if (nextBytePtr>=lastBytePtr)
+     return (NULL);
+  return ((struct usb_descriptor_header*)nextBytePtr);
+}
+#endif /* HAVE_USBCALLS */
+
+#if defined (__FreeBSD__)
+#include <dev/usb/usb.h>
+#endif /* __FreeBSD__ */
+
 #define BACKEND_NAME	sanei_usb
 #include "../include/sane/sane.h"
 #include "../include/sane/sanei_debug.h"
@@ -72,7 +101,9 @@ typedef enum
 {
   sanei_usb_method_scanner_driver = 0,	/* kernel scanner driver 
 					   (Linux, BSD) */
-  sanei_usb_method_libusb
+  sanei_usb_method_libusb,
+
+  sanei_usb_method_usbcalls
 }
 sanei_usb_access_method_type;
 
@@ -129,7 +160,7 @@ typedef struct
   unsigned short index;
   unsigned short length;
 }
-devrequest __attribute__ ((packed));
+devrequest;
 
 /* From /usr/src/linux/driver/usb/scanner.h */
 struct ctrlmsg_ioctl
@@ -189,7 +220,7 @@ print_buffer (const SANE_Byte * buffer, SANE_Int size)
 }
 
 static void
-kernel_get_vendor_product (int fd, int *vendorID, int *productID)
+kernel_get_vendor_product (int fd, const char *name, int *vendorID, int *productID)
 {
 #if defined (__linux__)
   /* read the vendor and product IDs via the IOCTLs */
@@ -197,13 +228,13 @@ kernel_get_vendor_product (int fd, int *vendorID, int *productID)
     {
       if (ioctl (fd, SCANNER_IOCTL_VENDOR_OLD, vendorID) == -1)
 	DBG (3, "kernel_get_vendor_product: ioctl (vendor) "
-	     "of device %d failed: %s\n", fd, strerror (errno));
+	     "of device %s failed: %s\n", name, strerror (errno));
     }
   if (ioctl (fd, SCANNER_IOCTL_PRODUCT, productID) == -1)
     {
       if (ioctl (fd, SCANNER_IOCTL_PRODUCT_OLD, productID) == -1)
 	DBG (3, "sanei_usb_get_vendor_product: ioctl (product) "
-	     "of device %d failed: %s\n", fd, strerror (errno));
+	     "of device %s failed: %s\n", name, strerror (errno));
     }
 #elif defined(__BEOS__)
   {
@@ -218,7 +249,47 @@ kernel_get_vendor_product (int fd, int *vendorID, int *productID)
     *vendorID = vendor;
     *productID = product;
   }
-#endif /* defined (__linux__), defined(__BEOS__) */
+#elif defined (__FreeBSD__)
+  {
+    int controller;
+    int ctrl_fd;
+    char buf[40];
+    int dev;
+    
+    for (controller = 0; ; controller++ )
+      {
+	snprintf (buf, sizeof (buf) - 1, "/dev/usb%d", controller);
+	ctrl_fd = open (buf, O_RDWR);
+	
+	/* If we can not open the usb controller device, treat it
+	   as the end of controller devices */
+	if (ctrl_fd < 0)
+	  break;
+	
+	/* Search for the scanner device on this bus */
+	for (dev = 1; dev < USB_MAX_DEVICES; dev++)
+	  {  
+	    struct usb_device_info devInfo;
+	    devInfo.udi_addr = dev;
+	    
+	    if (ioctl (ctrl_fd, USB_DEVICEINFO, &devInfo) == -1)
+	      break; /* Treat this as the end of devices for this controller */
+	    
+	    snprintf (buf, sizeof (buf), "/dev/%s", devInfo.udi_devnames[0]);
+	    if (strncmp (buf, name, sizeof (buf)) == 0)
+	      {
+		*vendorID = (int) devInfo.udi_vendorNo;
+		*productID = (int) devInfo.udi_productNo;
+		close (ctrl_fd);
+		return;
+	      }
+	  }
+	close (ctrl_fd);
+      }
+    DBG (3, "kernel_get_vendor_product: Could not retrieve "
+	 "vendor/product ID from device %s\n", name);
+  }
+#endif /* defined (__linux__), defined(__BEOS__), ... */
   /* put more os-dependant stuff ... */
 }
 
@@ -314,7 +385,7 @@ sanei_usb_init (void)
 		}
 	      vendor = -1;
 	      product = -1;
-	      kernel_get_vendor_product (fd, &vendor, &product);
+	      kernel_get_vendor_product (fd, devname, &vendor, &product);
 	      close (fd);
 	      devices[dn].devname = strdup (devname);
 	      if (!devices[dn].devname)
@@ -436,6 +507,81 @@ sanei_usb_init (void)
 	}
     }
 #endif /* HAVE_LIBUSB */
+#ifdef HAVE_USBCALLS
+  /* Check for devices using OS/2 USBCALLS Interface */
+
+   CHAR ucData[2048];
+   struct usb_device_descriptor *pDevDesc;
+   struct usb_config_descriptor   *pCfgDesc;
+   struct usb_interface_descriptor *intf;
+   struct usb_endpoint_descriptor  *ep;
+   struct usb_descriptor_header    *pDescHead;
+
+   APIRET rc;
+   ULONG ulNumDev, ulDev, ulBufLen;
+
+   ulBufLen = sizeof(ucData);
+   memset(&ucData,0,sizeof(ucData));
+   rc = UsbQueryNumberDevices( &ulNumDev);
+
+   if(rc==0 && ulNumDev)
+   {
+       for (ulDev=1; ulDev<=ulNumDev; ulDev++)
+       {
+         rc = UsbQueryDeviceReport( ulDev,
+                                  &ulBufLen,
+                                  ucData);
+
+         pDevDesc = (struct usb_device_descriptor*)ucData;
+         pCfgDesc = (struct usb_config_descriptor*) (ucData+sizeof(struct usb_device_descriptor));
+	  int interface=0;
+	  SANE_Bool found;
+	  if (!pCfgDesc->bConfigurationValue)
+	    {
+	      DBG (1, "sanei_usb_init: device 0x%04x/0x%04x is not configured\n",
+		   pDevDesc->idVendor, pDevDesc->idProduct);
+	      continue;
+	    }
+	  if (pDevDesc->idVendor == 0 || pDevDesc->idProduct == 0)
+	    {
+	      DBG (5, "sanei_usb_init: device 0x%04x/0x%04x looks like a root hub\n",
+		   pDevDesc->idVendor, pDevDesc->idProduct);
+	      continue;
+	    }
+	  found = SANE_FALSE;
+          
+          if (pDevDesc->bDeviceClass = USB_CLASS_VENDOR_SPEC)
+           {
+             found = SANE_TRUE;
+           }
+
+	  if (!found)
+	    {
+	      DBG (5, "sanei_usb_init: device 0x%04x/0x%04x: no suitable interfaces\n",
+		   pDevDesc->idVendor, pDevDesc->idProduct);
+	      continue;
+	    }
+
+	  snprintf (devname, sizeof (devname), "usbcalls:%d",
+		    ulDev);
+	  devices[dn].devname = strdup (devname);
+          devices[dn].fd = ulDev; /* store usbcalls device number */
+	  devices[dn].vendor = pDevDesc->idVendor;
+	  devices[dn].product = pDevDesc->idProduct;
+	  devices[dn].method = sanei_usb_method_usbcalls;
+	  devices[dn].open = SANE_FALSE;
+	  devices[dn].interface_nr = interface;
+	  DBG (4, "sanei_usb_init: found usbcalls device (0x%04x/0x%04x) as device number %s\n",
+	       pDevDesc->idVendor, pDevDesc->idProduct,devices[dn].devname);
+          dn++;
+          if (dn >= MAX_DEVICES)
+	    return;
+
+       }
+   }
+
+#endif /* HAVE_USBCALLS */
+
   DBG (5, "sanei_usb_init: found %d devices\n", dn);
 }
 
@@ -496,7 +642,7 @@ sanei_usb_get_vendor_product (SANE_Int dn, SANE_Word * vendor,
     }
 
   if (devices[dn].method == sanei_usb_method_scanner_driver)
-    kernel_get_vendor_product (devices[dn].fd, &vendorID, &productID);
+    kernel_get_vendor_product (devices[dn].fd, devices[dn].devname, &vendorID, &productID);
   else if (devices[dn].method == sanei_usb_method_libusb)
     {
 #ifdef HAVE_LIBUSB
@@ -508,13 +654,22 @@ sanei_usb_get_vendor_product (SANE_Int dn, SANE_Word * vendor,
       return SANE_STATUS_UNSUPPORTED;
 #endif /* HAVE_LIBUSB */
     }
+  else if (devices[dn].method == sanei_usb_method_usbcalls)
+   {
+#ifdef HAVE_USBCALLS
+     vendorID = devices[dn].vendor;
+     productID = devices[dn].product;
+#else
+      DBG (1, "sanei_usb_get_vendor_product: usbcalls support missing\n");
+      return SANE_STATUS_UNSUPPORTED;
+#endif /* HAVE_USBCALLS */
+   }
   else
     {
       DBG (1, "sanei_usb_get_vendor_product: access method %d not "
 	   "implemented\n", devices[dn].method);
       return SANE_STATUS_INVAL;
     }
-
   if (vendor)
     *vendor = vendorID;
   if (product)
@@ -649,12 +804,15 @@ sanei_usb_open (SANE_String_Const devname, SANE_Int * dn)
 	    }
 	  else if (errno == EBUSY)
 	    {
-	      DBG (1, "Maybe the kernel scanner driver claims the "
-		   "scanner's interface?\n");
-	      status = SANE_STATUS_DEVICE_BUSY;
+	      DBG (3, "Maybe the kernel scanner driver or usblp claims the "
+		   "interface? Ignoring this error...\n");
+	      status = SANE_STATUS_GOOD;
+	    }
+	  if (status != SANE_STATUS_GOOD)
+	    {
+	      usb_close (devices[devcount].libusb_handle);
+	      return status;
 	    }
-	  usb_close (devices[devcount].libusb_handle);
-	  return status;
 	}
 
       /* Claim the interface */
@@ -704,8 +862,7 @@ sanei_usb_open (SANE_String_Const devname, SANE_Int * dn)
 		      int address, direction, transfer_type;
 
 		      endpoint = &interface->endpoint[num];
-		      DBG (5, "sanei_usb_open:num: %d endpoint: 0x%x\n", num,
-			   endpoint);
+		      DBG (5, "sanei_usb_open: endpoint nr: %d\n", num);
 		      transfer_type =
 			endpoint->bmAttributes & USB_ENDPOINT_TYPE_MASK;
 		      address =
@@ -714,12 +871,12 @@ sanei_usb_open (SANE_String_Const devname, SANE_Int * dn)
 		      direction =
 			endpoint->bEndpointAddress & USB_ENDPOINT_DIR_MASK;
 
-		      DBG (5, "sanei_usb_open: direction: %d)\n", direction);
+		      DBG (5, "sanei_usb_open: direction: %d\n", direction);
 		      transfer_type =
 			endpoint->bmAttributes & USB_ENDPOINT_TYPE_MASK;
 
 		      DBG (5,
-			   "sanei_usb_open:  address: %d transfertype: %d\n",
+			   "sanei_usb_open: address: %d transfertype: %d\n",
 			   address, transfer_type);
 
 
@@ -880,6 +1037,153 @@ sanei_usb_open (SANE_String_Const devname, SANE_Int * dn)
 		 devname, strerror (errno));
 	}
     }
+  else if (devices[devcount].method == sanei_usb_method_usbcalls)
+    {
+#ifdef HAVE_USBCALLS
+      CHAR ucData[2048];
+      struct usb_device_descriptor *pDevDesc;
+      struct usb_config_descriptor   *pCfgDesc;
+      struct usb_interface_descriptor *interface;
+      struct usb_endpoint_descriptor  *endpoint;
+      struct usb_descriptor_header    *pDescHead;
+
+      ULONG  ulBufLen;
+      ulBufLen = sizeof(ucData);
+      memset(&ucData,0,sizeof(ucData));
+
+      int result, num,rc;
+      int address, direction, transfer_type;
+
+      DBG (5, "devname = %s, devcount = %d\n",devices[devcount].devname,devcount);
+      DBG (5, "USBCalls device number to open = %d\n",devices[devcount].fd);
+      DBG (5, "USBCalls Vendor/Product to open = 0x%04x/0x%04x\n",
+               devices[devcount].vendor,devices[devcount].product);
+      
+      rc = UsbOpen (&dh, 
+			devices[devcount].vendor,
+			devices[devcount].product,
+			USB_ANY_PRODUCTVERSION,
+			USB_OPEN_FIRST_UNUSED);
+      DBG (1, "sanei_usb_open: UsbOpen rc = %d\n",rc);
+      if (rc!=0)
+	{
+	  SANE_Status status = SANE_STATUS_INVAL;
+	  DBG (1, "sanei_usb_open: can't open device `%s': %s\n",
+	       devname, strerror (rc));
+	  return status;
+	}
+      rc = UsbQueryDeviceReport( devices[devcount].fd,
+                                  &ulBufLen,
+                                  ucData);
+      DBG (1, "sanei_usb_open: UsbQueryDeviceReport rc = %d\n",rc);
+      pDevDesc = (struct usb_device_descriptor*)ucData;
+      pCfgDesc = (struct usb_config_descriptor*) (ucData+sizeof(struct usb_device_descriptor));
+      UCHAR *pCurPtr = (UCHAR*) pCfgDesc;
+      UCHAR *pEndPtr = pCurPtr+ pCfgDesc->wTotalLength;
+      pDescHead = (struct usb_descriptor_header *) (pCurPtr+pCfgDesc->bLength);
+      /* Set the configuration */
+      if (pDevDesc->bNumConfigurations > 1)
+	{
+	  DBG (3, "sanei_usb_open: more than one "
+	       "configuration (%d), choosing first config (%d)\n",
+	       pDevDesc->bNumConfigurations, 
+	       pCfgDesc->bConfigurationValue);
+	}
+      DBG (5, "UsbDeviceSetConfiguration parameters: dh = %p, bConfigurationValue = %d\n",
+               dh,pCfgDesc->bConfigurationValue);
+      result = UsbDeviceSetConfiguration (dh,
+				      pCfgDesc->bConfigurationValue);
+      DBG (1, "sanei_usb_open: UsbDeviceSetConfiguration rc = %d\n",result);
+      if (result)
+	{
+	  SANE_Status status = SANE_STATUS_INVAL;
+	  DBG (1, "sanei_usb_open: usbcalls complained on UsbDeviceSetConfiguration, rc= %d\n", result);
+	  status = SANE_STATUS_ACCESS_DENIED;
+	  UsbClose (dh);
+	  return status;
+	}
+      
+      /* Now we look for usable endpoints */
+      
+      for (pDescHead = (struct usb_descriptor_header *) (pCurPtr+pCfgDesc->bLength);
+            pDescHead;pDescHead = GetNextDescriptor(pDescHead,pEndPtr) )
+	{
+          switch(pDescHead->bDescriptorType)
+          {
+            case USB_DT_INTERFACE:
+              interface = (struct usb_interface_descriptor *) pDescHead;
+              DBG (5, "Found %d endpoints\n",interface->bNumEndpoints);
+              DBG (5, "bAlternateSetting = %d\n",interface->bAlternateSetting);
+              break;
+            case USB_DT_ENDPOINT:
+	      endpoint = (struct usb_endpoint_descriptor*)pDescHead;
+              address = endpoint->bEndpointAddress;
+	      direction = endpoint->bEndpointAddress & USB_ENDPOINT_DIR_MASK;
+	      transfer_type = endpoint->bmAttributes & USB_ENDPOINT_TYPE_MASK;
+	      /* save the endpoints we need later */
+	      if (transfer_type == USB_ENDPOINT_TYPE_INTERRUPT)
+	      {
+   	       DBG (5, "sanei_usb_open: found interupt-%s endpoint (address %2x)\n",
+  	            direction ? "in" : "out", address);
+	       if (direction)	/* in */
+	       {
+	         if (devices[devcount].int_in_ep)
+		   DBG (3, "sanei_usb_open: we already have a int-in endpoint "
+		        "(address: %d), ignoring the new one\n",
+		        devices[devcount].int_in_ep);
+	         else
+		   devices[devcount].int_in_ep = endpoint->bEndpointAddress;
+	       }
+	       else
+	         if (devices[devcount].int_out_ep)
+		   DBG (3, "sanei_usb_open: we already have a int-out endpoint "
+		        "(address: %d), ignoring the new one\n",
+		        devices[devcount].int_out_ep);
+	         else
+		   devices[devcount].int_out_ep = endpoint->bEndpointAddress;
+	     }
+	     else if (transfer_type == USB_ENDPOINT_TYPE_BULK)
+	     {
+	       DBG (5, "sanei_usb_open: found bulk-%s endpoint (address %2x)\n",
+	            direction ? "in" : "out", address);
+	       if (direction)	/* in */
+	         {
+		   if (devices[devcount].bulk_in_ep)
+		     DBG (3, "sanei_usb_open: we already have a bulk-in endpoint "
+		          "(address: %d), ignoring the new one\n",
+		          devices[devcount].bulk_in_ep);
+		   else
+		     devices[devcount].bulk_in_ep = endpoint->bEndpointAddress;
+	         }
+	       else
+	         {
+	           if (devices[devcount].bulk_out_ep)
+		     DBG (3, "sanei_usb_open: we already have a bulk-out endpoint "
+		          "(address: %d), ignoring the new one\n",
+		          devices[devcount].bulk_out_ep);
+	           else
+		     devices[devcount].bulk_out_ep = endpoint->bEndpointAddress;
+	         }
+	       }
+	     /* ignore currently unsupported endpoints */
+	     else {
+	         DBG (5, "sanei_usb_open: ignoring %s-%s endpoint "
+		      "(address: %d)\n",
+		      transfer_type == USB_ENDPOINT_TYPE_CONTROL ? "control" :
+		      transfer_type == USB_ENDPOINT_TYPE_ISOCHRONOUS
+		      ? "isochronous" : "interrupt",
+		      direction ? "in" : "out", address);
+	         continue;
+	          }
+          break;
+          }
+        }
+#else
+      DBG (1, "sanei_usb_open: can't open device `%s': "
+	   "usbcalls support missing\n", devname);
+      return SANE_STATUS_UNSUPPORTED;
+#endif /* HAVE_USBCALLS */
+    }
   else
     {
       DBG (1, "sanei_usb_open: access method %d not implemented\n",
@@ -911,6 +1215,16 @@ sanei_usb_close (SANE_Int dn)
     }
   if (devices[dn].method == sanei_usb_method_scanner_driver)
     close (devices[dn].fd);
+  else if (devices[dn].method == sanei_usb_method_usbcalls)
+    {
+#ifdef HAVE_USBCALLS
+      int rc;
+      rc=UsbClose (dh);
+      DBG (5,"rc of UsbClose = %d\n",rc);
+#else
+    DBG (1, "sanei_usb_close: usbcalls support missing\n");
+#endif
+    }
   else
 #ifdef HAVE_LIBUSB
     {
@@ -941,7 +1255,7 @@ sanei_usb_set_timeout (SANE_Int timeout)
 #ifdef HAVE_LIBUSB
   libusb_timeout = timeout;
 #else
-  DBG (1, "sanei_usb_close: libusb support missing\n");
+  DBG (1, "sanei_usb_set_timeout: libusb support missing\n");
 #endif /* HAVE_LIBUSB */
 }
 
@@ -986,6 +1300,42 @@ sanei_usb_read_bulk (SANE_Int dn, SANE_Byte * buffer, size_t * size)
       return SANE_STATUS_UNSUPPORTED;
     }
 #endif /* not HAVE_LIBUSB */
+  else if (devices[dn].method == sanei_usb_method_usbcalls)
+  {
+#ifdef HAVE_USBCALLS
+    int rc;
+    char* buffer_ptr = (char*) buffer;
+    while (*size)
+    {
+      ULONG ulToRead = (*size>MAX_RW)?MAX_RW:*size;
+      ULONG ulNum = ulToRead;
+      DBG (5, "Entered usbcalls UsbBulkRead with dn = %d\n",dn);
+      DBG (5, "Entered usbcalls UsbBulkRead with dh = %p\n",dh);
+      DBG (5, "Entered usbcalls UsbBulkRead with bulk_in_ep = 0x%02x\n",devices[dn].bulk_in_ep);
+      DBG (5, "Entered usbcalls UsbBulkRead with interface_nr = %d\n",devices[dn].interface_nr);
+      DBG (5, "Entered usbcalls UsbBulkRead with usbcalls_timeout = %d\n",usbcalls_timeout);
+
+      if (devices[dn].bulk_in_ep){
+        rc = UsbBulkRead (dh, devices[dn].bulk_in_ep, devices[dn].interface_nr,
+                               &ulToRead, buffer_ptr, usbcalls_timeout);
+        DBG (1, "sanei_usb_read_bulk: rc = %d\n",rc);}
+      else
+      {
+          DBG (1, "sanei_usb_read_bulk: can't read without a bulk-in endpoint\n");
+          return SANE_STATUS_INVAL;
+      }
+      if (rc || (ulNum!=ulToRead)) return SANE_STATUS_INVAL;
+      *size -=ulToRead;
+      buffer_ptr += ulToRead;
+      read_size += ulToRead;
+    }
+#else /* not HAVE_USBCALLS */
+    {
+      DBG (1, "sanei_usb_read_bulk: usbcalls support missing\n");
+      return SANE_STATUS_UNSUPPORTED;
+    }
+#endif /* not HAVE_USBCALLS */
+  }
   else
     {
       DBG (1, "sanei_usb_read_bulk: access method %d not implemented\n",
@@ -1062,6 +1412,42 @@ sanei_usb_write_bulk (SANE_Int dn, const SANE_Byte * buffer, size_t * size)
       return SANE_STATUS_UNSUPPORTED;
     }
 #endif /* not HAVE_LIBUSB */
+  else if (devices[dn].method == sanei_usb_method_usbcalls)
+  {
+#ifdef HAVE_USBCALLS
+    int rc;
+    DBG (5, "Entered usbcalls UsbBulkWrite with dn = %d\n",dn);
+    DBG (5, "Entered usbcalls UsbBulkWrite with dh = %p\n",dh);
+    DBG (5, "Entered usbcalls UsbBulkWrite with bulk_out_ep = 0x%02x\n",devices[dn].bulk_out_ep);
+    DBG (5, "Entered usbcalls UsbBulkWrite with interface_nr = %d\n",devices[dn].interface_nr);
+    DBG (5, "Entered usbcalls UsbBulkWrite with usbcalls_timeout = %d\n",usbcalls_timeout);
+    while (*size)
+    {
+      ULONG ulToWrite = (*size>MAX_RW)?MAX_RW:*size;
+
+      DBG (5, "size requested to write = %lu, ulToWrite = %lu\n",(unsigned long) *size,ulToWrite);
+      if (devices[dn].bulk_out_ep){
+        rc = UsbBulkWrite (dh, devices[dn].bulk_out_ep, devices[dn].interface_nr,
+                               ulToWrite, (char*) buffer, usbcalls_timeout);
+        DBG (1, "sanei_usb_write_bulk: rc = %d\n",rc);}
+      else
+      {
+          DBG (1, "sanei_usb_write_bulk: can't read without a bulk-out endpoint\n");
+          return SANE_STATUS_INVAL;
+      }
+      if (rc) return SANE_STATUS_INVAL;
+      *size -=ulToWrite;
+      buffer += ulToWrite;
+      write_size += ulToWrite;
+      DBG (5, "size = %d, write_size = %d\n",*size, write_size);
+    }
+#else /* not HAVE_USBCALLS */
+    {
+      DBG (1, "sanei_usb_write_bulk: usbcalls support missing\n");
+      return SANE_STATUS_UNSUPPORTED;
+    }
+#endif /* not HAVE_USBCALLS */
+  }
   else
     {
       DBG (1, "sanei_usb_write_bulk: access method %d not implemented\n",
@@ -1172,6 +1558,30 @@ sanei_usb_control_msg (SANE_Int dn, SANE_Int rtype, SANE_Int req,
       return SANE_STATUS_UNSUPPORTED;
     }
 #endif /* not HAVE_LIBUSB */
+  else if (devices[dn].method == sanei_usb_method_usbcalls)
+     {
+#ifdef HAVE_USBCALLS
+      int result;
+
+      result = UsbCtrlMessage (dh, rtype, req,
+				value, index, len, (char *) data, 
+				usbcalls_timeout);
+      DBG (5, "rc of usb_control_msg = %d\n",result);
+      if (result < 0)
+	{
+	  DBG (1, "sanei_usb_control_msg: usbcalls complained: %d\n",result);
+	  return SANE_STATUS_INVAL;
+	}
+      if ((rtype & 0x80) && debug_level > 10)
+	print_buffer (data, len);
+      return SANE_STATUS_GOOD;
+#else /* not HAVE_USBCALLS */
+    {
+      DBG (1, "sanei_usb_control_msg: usbcalls support missing\n");
+      return SANE_STATUS_UNSUPPORTED;
+    }
+#endif /* not HAVE_USBCALLS */
+     }
   else
     {
       DBG (1, "sanei_usb_control_msg: access method %d not implemented\n",
@@ -1226,6 +1636,35 @@ sanei_usb_read_int (SANE_Int dn, SANE_Byte * buffer, size_t * size)
       return SANE_STATUS_UNSUPPORTED;
     }
 #endif /* not HAVE_LIBUSB */
+  else if (devices[dn].method == sanei_usb_method_usbcalls)
+    {
+#ifdef HAVE_USBCALLS
+      int rc;
+      USHORT usNumBytes=*size; 
+      DBG (5, "Entered usbcalls UsbIrqStart with dn = %d\n",dn);
+      DBG (5, "Entered usbcalls UsbIrqStart with dh = %p\n",dh);
+      DBG (5, "Entered usbcalls UsbIrqStart with int_in_ep = 0x%02x\n",devices[dn].int_in_ep);
+      DBG (5, "Entered usbcalls UsbIrqStart with interface_nr = %d\n",devices[dn].interface_nr);
+      DBG (5, "Entered usbcalls UsbIrqStart with bytes to read = %u\n",usNumBytes);
+
+      if (devices[dn].int_in_ep){
+         rc = UsbIrqStart (dh,devices[dn].int_in_ep,devices[dn].interface_nr,
+			usNumBytes, (char *) buffer, pUsbIrqStartHev);
+         DBG (5, "rc of UsbIrqStart = %d\n",rc);
+        }
+      else
+	{
+	  DBG (1, "sanei_usb_read_int: can't read without an int "
+	       "endpoint\n");
+	  return SANE_STATUS_INVAL;
+	}
+      if (rc) return SANE_STATUS_INVAL;
+      read_size += usNumBytes;
+#else
+      DBG (1, "sanei_usb_read_int: usbcalls support missing\n");
+      return SANE_STATUS_UNSUPPORTED;
+#endif /* HAVE_USBCALLS */
+    }
   else
     {
       DBG (1, "sanei_usb_read_int: access method %d not implemented\n",
