@@ -1,7 +1,7 @@
 /*
    check-usb-chip.c -- Find out what USB scanner chipset is used
    
-   Copyright (C) 2003, 2004 Henning Meier-Geinitz <henning@meier-geinitz.de>
+   Copyright (C) 2003-2005 Henning Meier-Geinitz <henning@meier-geinitz.de>
    Copyright (C) 2003 Gerhard Jger <gerhard@gjaeger.de>
                       for LM983x tests
    Copyright (C) 2003 Gerard Klaver <gerard at gkall dot hobby dot nl>
@@ -28,7 +28,6 @@
 
 #include "../include/sane/config.h"
 #include "../include/sane/sane.h"
-
 #include <stdio.h>
 #include <string.h>
 #include <errno.h>
@@ -841,7 +840,7 @@ static char *
 check_ma1509 (struct usb_device *dev)
 {
   char req[8];
-  unsigned char inquiry[0x60];
+  char inquiry[0x60];
   usb_dev_handle *handle;
   int result;
 
@@ -2062,9 +2061,841 @@ check_icm532b (struct usb_device *dev)
   finish_interface (handle);
   return "ICM532B";
 }
-
 /* ====================================== end of icm532b ==================*/
 
+
+/* Check for the combination of a PowerVision PV8630 (USB->parport bridge)
+   and National Semiconductor LM9830 */
+static char *
+check_pv8630_lm9830 (struct usb_device *dev)
+{
+  usb_dev_handle *handle;
+  int result;
+  char data;
+
+  if (verbose > 2)
+    printf ("    checking for PV8630/LM9830 ...\n");
+
+  /* Check device descriptor */
+  if (dev->descriptor.bDeviceClass != USB_CLASS_PER_INTERFACE)
+    {
+      if (verbose > 2)
+	printf ("    this is not a PV8630/LM9830 (bDeviceClass = %d)\n",
+		dev->descriptor.bDeviceClass);
+      return 0;
+    }
+  if (dev->descriptor.bcdUSB != 0x100)
+    {
+      if (verbose > 2)
+	printf ("    this is not a PV8630/LM9830 (bcdUSB = 0x%x)\n",
+		dev->descriptor.bcdUSB);
+      return 0;
+    }
+  if (dev->descriptor.bDeviceSubClass != 0x00)
+    {
+      if (verbose > 2)
+	printf ("    this is not a PV8630/LM9830 (bDeviceSubClass = 0x%x)\n",
+		dev->descriptor.bDeviceSubClass);
+      return 0;
+    }
+  if (dev->descriptor.bDeviceProtocol != 0x00)
+    {
+      if (verbose > 2)
+	printf ("    this is not a PV8630/LM9830 (bDeviceProtocol = 0x%x)\n",
+		dev->descriptor.bDeviceProtocol);
+      return 0;
+    }
+
+  /* Check endpoints */
+  if (dev->config[0].interface[0].altsetting[0].bNumEndpoints != 3)
+    {
+      if (verbose > 2)
+	printf ("    this is not a PV8630/LM9830 (bNumEndpoints = %d)\n",
+		dev->config[0].interface[0].altsetting[0].bNumEndpoints);
+      return 0;
+    }
+  /* Endpoint 0 */
+  if ((dev->config[0].interface[0].altsetting[0].endpoint[0].
+       bEndpointAddress != 0x01)
+      || (dev->config[0].interface[0].altsetting[0].endpoint[0].
+	  bmAttributes != 0x02)
+      || (dev->config[0].interface[0].altsetting[0].endpoint[0].
+	  wMaxPacketSize != 0x40)
+      || (dev->config[0].interface[0].altsetting[0].endpoint[0].bInterval !=
+	  0x00))
+    {
+      if (verbose > 2)
+	printf
+	  ("    this is not a PV8630/LM9830 (bEndpointAddress = 0x%x, bmAttributes = 0x%x, "
+	   "wMaxPacketSize = 0x%x, bInterval = 0x%x)\n",
+	   dev->config[0].interface[0].altsetting[0].endpoint[0].
+	   bEndpointAddress,
+	   dev->config[0].interface[0].altsetting[0].endpoint[0].bmAttributes,
+	   dev->config[0].interface[0].altsetting[0].endpoint[0].
+	   wMaxPacketSize,
+	   dev->config[0].interface[0].altsetting[0].endpoint[0].bInterval);
+      return 0;
+    }
+  /* Endpoint 1 */
+  if ((dev->config[0].interface[0].altsetting[0].endpoint[1].
+       bEndpointAddress != 0x82)
+      || (dev->config[0].interface[0].altsetting[0].endpoint[1].
+	  bmAttributes != 0x02)
+      || (dev->config[0].interface[0].altsetting[0].endpoint[1].
+	  wMaxPacketSize != 0x40)
+      || (dev->config[0].interface[0].altsetting[0].endpoint[1].bInterval !=
+	  0x00))
+    {
+      if (verbose > 2)
+	printf
+	  ("    this is not a PV8630/LM9830 (bEndpointAddress = 0x%x, bmAttributes = 0x%x, "
+	   "wMaxPacketSize = 0x%x, bInterval = 0x%x)\n",
+	   dev->config[0].interface[0].altsetting[0].endpoint[1].
+	   bEndpointAddress,
+	   dev->config[0].interface[0].altsetting[0].endpoint[1].bmAttributes,
+	   dev->config[0].interface[0].altsetting[0].endpoint[1].
+	   wMaxPacketSize,
+	   dev->config[0].interface[0].altsetting[0].endpoint[1].bInterval);
+      return 0;
+    }
+  /* Endpoint 2 */
+  if ((dev->config[0].interface[0].altsetting[0].endpoint[2].
+       bEndpointAddress != 0x83)
+      || (dev->config[0].interface[0].altsetting[0].endpoint[2].
+	  bmAttributes != 0x03)
+      || (dev->config[0].interface[0].altsetting[0].endpoint[2].
+	  wMaxPacketSize != 0x01)
+      || (dev->config[0].interface[0].altsetting[0].endpoint[2].bInterval !=
+	  0x01))
+    {
+      if (verbose > 2)
+	printf
+	  ("    this is not a PV8630/LM9830 (bEndpointAddress = 0x%x, bmAttributes = 0x%x, "
+	   "wMaxPacketSize = 0x%x, bInterval = 0x%x)\n",
+	   dev->config[0].interface[0].altsetting[0].endpoint[2].
+	   bEndpointAddress,
+	   dev->config[0].interface[0].altsetting[0].endpoint[2].bmAttributes,
+	   dev->config[0].interface[0].altsetting[0].endpoint[2].
+	   wMaxPacketSize,
+	   dev->config[0].interface[0].altsetting[0].endpoint[2].bInterval);
+      return 0;
+    }
+
+  /* Now we write/read a register (red offset) */
+  result = prepare_interface (dev, &handle);
+  if (!result)
+    return "PV8630/LM9830?";
+
+  result = 
+    usb_control_msg (handle, 0x40, 0x01, 0x38, 0x01, NULL, 0, TIMEOUT);
+  if (result < 0)
+    {
+      if (verbose > 2)
+	printf ("    Couldn't send write register number (%s)\n",
+		usb_strerror ());
+      finish_interface (handle);
+      return 0;
+    }
+
+  result =
+    usb_control_msg (handle, 0x40, 0x01, 0x0f, 0x00,  NULL, 0, TIMEOUT);
+  if (result < 0)
+    {
+      if (verbose > 2)
+	printf ("    Couldn't send register data (%s)\n",
+		usb_strerror ());
+      finish_interface (handle);
+      return 0;
+    }
+
+  result = 
+    usb_control_msg (handle, 0x40, 0x01, 0x38, 0x01, NULL, 0, TIMEOUT);
+  if (result < 0)
+    {
+      if (verbose > 2)
+	printf ("    Couldn't send read register number (%s)\n",
+		usb_strerror ());
+      finish_interface (handle);
+      return 0;
+    }
+
+  result =
+    usb_control_msg (handle, 0xc0, 0x00, 0, 0x00, &data, 1, TIMEOUT);
+  if (result <= 0)
+    {
+      if (verbose > 2)
+	printf ("    Couldn't read register data (%s)\n",
+		usb_strerror ());
+      finish_interface (handle);
+      return 0;
+    }
+
+  if (data != 0x0f)
+    {
+      if (verbose > 2)
+	printf ("    Data read != data written (%d/%d)\n", data, 0x0f);
+      finish_interface (handle);
+      return 0;
+    }
+
+  finish_interface (handle);
+  return "PV8630/LM9830";
+}
+
+
+/* Check for Toshiba M011 */
+static char *
+check_m011 (struct usb_device *dev)
+{
+  usb_dev_handle *handle;
+  int result;
+  char data;
+
+  if (verbose > 2)
+    printf ("    checking for M011 ...\n");
+
+  /* Check device descriptor */
+  if (dev->descriptor.bDeviceClass != USB_CLASS_VENDOR_SPEC)
+    {
+      if (verbose > 2)
+	printf ("    this is not a M011 (bDeviceClass = %d)\n",
+		dev->descriptor.bDeviceClass);
+      return 0;
+    }
+  if (dev->descriptor.bcdUSB != 0x100)
+    {
+      if (verbose > 2)
+	printf ("    this is not a M011 (bcdUSB = 0x%x)\n",
+		dev->descriptor.bcdUSB);
+      return 0;
+    }
+  if (dev->descriptor.bDeviceSubClass != USB_CLASS_VENDOR_SPEC)
+    {
+      if (verbose > 2)
+	printf ("    this is not a M011 (bDeviceSubClass = 0x%x)\n",
+		dev->descriptor.bDeviceSubClass);
+      return 0;
+    }
+  if (dev->descriptor.bDeviceProtocol != USB_CLASS_VENDOR_SPEC)
+    {
+      if (verbose > 2)
+	printf ("    this is not a M011 (bDeviceProtocol = 0x%x)\n",
+		dev->descriptor.bDeviceProtocol);
+      return 0;
+    }
+
+  /* Check endpoints */
+  if (dev->config[0].interface[0].altsetting[0].bNumEndpoints != 1)
+    {
+      if (verbose > 2)
+	printf ("    this is not a M011 (bNumEndpoints = %d)\n",
+		dev->config[0].interface[0].altsetting[0].bNumEndpoints);
+      return 0;
+    }
+  /* Endpoint 0 */
+  if ((dev->config[0].interface[0].altsetting[0].endpoint[0].
+       bEndpointAddress != 0x82)
+      || (dev->config[0].interface[0].altsetting[0].endpoint[0].
+	  bmAttributes != 0x02)
+      || (dev->config[0].interface[0].altsetting[0].endpoint[0].
+	  wMaxPacketSize != 0x40)
+      || (dev->config[0].interface[0].altsetting[0].endpoint[0].bInterval !=
+	  0x00))
+    {
+      if (verbose > 2)
+	printf
+	  ("    this is not a M011 (bEndpointAddress = 0x%x, bmAttributes = 0x%x, "
+	   "wMaxPacketSize = 0x%x, bInterval = 0x%x)\n",
+	   dev->config[0].interface[0].altsetting[0].endpoint[0].
+	   bEndpointAddress,
+	   dev->config[0].interface[0].altsetting[0].endpoint[0].bmAttributes,
+	   dev->config[0].interface[0].altsetting[0].endpoint[0].
+	   wMaxPacketSize,
+	   dev->config[0].interface[0].altsetting[0].endpoint[0].bInterval);
+      return 0;
+    }
+
+  /* Now we write/read a register (red offset) */
+  result = prepare_interface (dev, &handle);
+  if (!result)
+    return "M011?";
+
+  data = 0x63;
+
+  result = 
+    usb_control_msg (handle, 0x40, 0x08, 0x34, 0x00, &data, 1, TIMEOUT);
+  if (result < 0)
+    {
+      if (verbose > 2)
+	printf ("    Couldn't write register (%s)\n",
+		usb_strerror ());
+      finish_interface (handle);
+      return 0;
+    }
+  data = 0;
+
+  result =
+    usb_control_msg (handle, 0xc0, 0x00, 0x34, 0x00, &data, 1, TIMEOUT);
+  if (result <= 0)
+    {
+      if (verbose > 2)
+	printf ("    Couldn't read register (%s)\n",
+		usb_strerror ());
+      finish_interface (handle);
+      return 0;
+    }
+
+  if (data != 0x63)
+    {
+      if (verbose > 2)
+	printf ("    Data read != data written (%d/%d)\n", data, 0x63);
+      finish_interface (handle);
+      return 0;
+    }
+
+  finish_interface (handle);
+  return "M011";
+}
+
+
+/* Check for Realtek rts8858c */
+static int
+rts88xx_read_reg (usb_dev_handle * handle, unsigned char *req, unsigned char *res, int size)
+{
+  int result;
+
+  result =
+    usb_bulk_write (handle, 0x02, (char *)req, 0x04, TIMEOUT);
+  if (result < 0)
+    return 0;
+
+  result =
+    usb_bulk_read (handle,  0x81, (char *)res, size, TIMEOUT);
+  if (result < 0)
+    return 0;
+
+  return 1;
+}
+
+static char *
+check_rts8858c (struct usb_device *dev)
+{
+  unsigned char req[4];
+  unsigned char res[10];
+  usb_dev_handle *handle;
+  int result;
+
+  if (verbose > 2)
+    printf ("    checking for rts8858c ...\n");
+
+  /* Check device descriptor */
+  if (dev->descriptor.bDeviceClass != 0)
+    {
+      if (verbose > 2)
+	printf ("    this is not a rts8858c (bDeviceClass = %d)\n",
+		dev->descriptor.bDeviceClass);
+      return 0;
+    }
+  if (dev->descriptor.bcdUSB != 0x110)
+    {
+      if (verbose > 2)
+	printf ("    this is not a rts8858c (bcdUSB = 0x%x)\n",
+		dev->descriptor.bcdUSB);
+      return 0;
+    }
+  if (dev->descriptor.bDeviceSubClass != 0)
+    {
+      if (verbose > 2)
+	printf ("    this is not a rts8858c (bDeviceSubClass = 0x%x)\n",
+		dev->descriptor.bDeviceSubClass);
+      return 0;
+    }
+  if (dev->descriptor.bDeviceProtocol != 0)
+    {
+      if (verbose > 2)
+	printf ("    this is not a rts8858c (bDeviceProtocol = 0x%x)\n",
+		dev->descriptor.bDeviceProtocol);
+      return 0;
+    }
+
+  /* Check endpoints */
+  if (dev->config[0].interface[0].altsetting[0].bNumEndpoints != 3)
+    {
+      if (verbose > 2)
+	printf ("    this is not a rts8858c (bNumEndpoints = %d)\n",
+		dev->config[0].interface[0].altsetting[0].bNumEndpoints);
+      return 0;
+    }
+  if ((dev->config[0].interface[0].altsetting[0].endpoint[0].
+       bEndpointAddress != 0x81)
+      || (dev->config[0].interface[0].altsetting[0].endpoint[0].
+	  bmAttributes != 0x02)
+      || (dev->config[0].interface[0].altsetting[0].endpoint[0].
+	  wMaxPacketSize != 0x40)
+      || (dev->config[0].interface[0].altsetting[0].endpoint[0].bInterval !=
+	  0x00))
+    {
+      if (verbose > 2)
+	printf
+	  ("    this is not a rts8858c (bEndpointAddress = 0x%x, bmAttributes = 0x%x, "
+	   "wMaxPacketSize = 0x%x, bInterval = 0x%x)\n",
+	   dev->config[0].interface[0].altsetting[0].endpoint[0].
+	   bEndpointAddress,
+	   dev->config[0].interface[0].altsetting[0].endpoint[0].bmAttributes,
+	   dev->config[0].interface[0].altsetting[0].endpoint[0].
+	   wMaxPacketSize,
+	   dev->config[0].interface[0].altsetting[0].endpoint[0].bInterval);
+      return 0;
+    }
+
+  if ((dev->config[0].interface[0].altsetting[0].endpoint[1].
+       bEndpointAddress != 0x02)
+      || (dev->config[0].interface[0].altsetting[0].endpoint[1].
+	  bmAttributes != 0x02)
+      || (dev->config[0].interface[0].altsetting[0].endpoint[1].
+	  wMaxPacketSize != 0x08)
+      || (dev->config[0].interface[0].altsetting[0].endpoint[1].bInterval !=
+	  0x00))
+    {
+      if (verbose > 2)
+	printf
+	  ("    this is not a rts8858c (bEndpointAddress = 0x%x, bmAttributes = 0x%x, "
+	   "wMaxPacketSize = 0x%x, bInterval = 0x%x)\n",
+	   dev->config[0].interface[0].altsetting[0].endpoint[1].
+	   bEndpointAddress,
+	   dev->config[0].interface[0].altsetting[0].endpoint[1].bmAttributes,
+	   dev->config[0].interface[0].altsetting[0].endpoint[1].
+	   wMaxPacketSize,
+	   dev->config[0].interface[0].altsetting[0].endpoint[1].bInterval);
+      return 0;
+    }
+
+  if ((dev->config[0].interface[0].altsetting[0].endpoint[2].
+       bEndpointAddress != 0x83)
+      || (dev->config[0].interface[0].altsetting[0].endpoint[2].
+	  bmAttributes != 0x03)
+      || (dev->config[0].interface[0].altsetting[0].endpoint[2].
+	  wMaxPacketSize != 0x01)
+      || (dev->config[0].interface[0].altsetting[0].endpoint[2].bInterval !=
+	  0xFA))
+    {
+      if (verbose > 2)
+	printf
+	  ("    this is not a rts8858c (bEndpointAddress = 0x%x, bmAttributes = 0x%x, "
+	   "wMaxPacketSize = 0x%x, bInterval = 0x%x)\n",
+	   dev->config[0].interface[0].altsetting[0].endpoint[2].
+	   bEndpointAddress,
+	   dev->config[0].interface[0].altsetting[0].endpoint[2].bmAttributes,
+	   dev->config[0].interface[0].altsetting[0].endpoint[2].
+	   wMaxPacketSize,
+	   dev->config[0].interface[0].altsetting[0].endpoint[2].bInterval);
+      return 0;
+    }
+
+  /* Now we read 10 registers */
+  result = prepare_interface (dev, &handle);
+  if (!result)
+    return "rts8858c?";
+
+  memset (req, 0, 4);
+  req[0] = 0x80;		/* get registers 0x12-0x1c */
+  req[1] = 0x12;
+  req[2] = 0x00;
+  req[3] = 0x0a;
+
+  result = rts88xx_read_reg(handle,req,res,req[3]);
+  if (result <= 0)
+    {
+      if (verbose > 2)
+	printf ("    Couldn't read registers\n");
+      finish_interface (handle);
+      return 0;
+    }
+
+  if (res[1] != 0x03 || res[2] != 0x00)
+    {
+      if (verbose > 2)
+	printf ("    Unexpected result from register reading (0x%0x/0x%0x)\n",
+		res[1], res[2]);
+      finish_interface (handle);
+      return 0;
+    }
+  finish_interface (handle);
+  return "rts8858c";
+}	/* end of rts8858 detection */
+
+
+/* Check for Service & Quality SQ113 */
+static char *
+check_sq113 (struct usb_device *dev)
+{
+  usb_dev_handle *handle;
+  int result;
+  unsigned char data;
+  unsigned char buffer[4];
+
+  if (verbose > 2)
+    printf ("    checking for SQ113 ...\n");
+
+  /* Check device descriptor */
+  if (dev->descriptor.bDeviceClass != 0)
+    {
+      if (verbose > 2)
+	printf ("    this is not a SQ113 (bDeviceClass = %d)\n",
+		dev->descriptor.bDeviceClass);
+      return 0;
+    }
+  if (dev->descriptor.bcdUSB != 0x200)
+    {
+      if (verbose > 2)
+	printf ("    this is not a SQ113 (bcdUSB = 0x%x)\n",
+		dev->descriptor.bcdUSB);
+      return 0;
+    }
+  if (dev->descriptor.bDeviceSubClass != 0)
+    {
+      if (verbose > 2)
+	printf ("    this is not a SQ113 (bDeviceSubClass = 0x%x)\n",
+		dev->descriptor.bDeviceSubClass);
+      return 0;
+    }
+  if (dev->descriptor.bDeviceProtocol != 0)
+    {
+      if (verbose > 2)
+	printf ("    this is not a SQ113 (bDeviceProtocol = 0x%x)\n",
+		dev->descriptor.bDeviceProtocol);
+      return 0;
+    }
+
+  /* Check interface */
+  if (dev->config[0].interface[0].altsetting[0].bInterfaceClass != 255)
+    {
+      if (verbose > 2)
+	printf ("    this is not a SQ113 (bInterfaceClass = %d)\n",
+		dev->config[0].interface[0].altsetting[0].bInterfaceClass);
+      return 0;
+    }
+
+  if (dev->config[0].interface[0].altsetting[0].bInterfaceSubClass != 255)
+    {
+      if (verbose > 2)
+	printf ("    this is not a SQ113 (bInterfaceSubClass = %d)\n",
+		dev->config[0].interface[0].altsetting[0].bInterfaceSubClass);
+      return 0;
+    }
+  if (dev->config[0].interface[0].altsetting[0].bInterfaceProtocol != 255)
+    {
+      if (verbose > 2)
+	printf ("    this is not a SQ113 (bInterfaceProtocol = %d)\n",
+		dev->config[0].interface[0].altsetting[0].bInterfaceProtocol);
+      return 0;
+    }
+
+  /* Check endpoints */
+  if (dev->config[0].interface[0].altsetting[0].bNumEndpoints != 3)
+    {
+      if (verbose > 2)
+	printf ("    this is not a SQ113 (bNumEndpoints = %d)\n",
+		dev->config[0].interface[0].altsetting[0].bNumEndpoints);
+      return 0;
+    }
+  /* Endpoint 0 */
+  if ((dev->config[0].interface[0].altsetting[0].endpoint[0].
+       bEndpointAddress != 0x01)
+      || (dev->config[0].interface[0].altsetting[0].endpoint[0].
+	  bmAttributes != 0x02)
+      || ((dev->config[0].interface[0].altsetting[0].endpoint[0].
+	  wMaxPacketSize != 0x40)
+	  && (dev->config[0].interface[0].altsetting[0].endpoint[0].
+	      wMaxPacketSize != 0x200))
+      || (dev->config[0].interface[0].altsetting[0].endpoint[0].bInterval !=
+	  0x00))
+    {
+      if (verbose > 2)
+	printf
+	  ("    this is not a SQ113 (bEndpointAddress = 0x%x, bmAttributes = 0x%x, "
+	   "wMaxPacketSize = 0x%x, bInterval = 0x%x)\n",
+	   dev->config[0].interface[0].altsetting[0].endpoint[0].
+	   bEndpointAddress,
+	   dev->config[0].interface[0].altsetting[0].endpoint[0].bmAttributes,
+	   dev->config[0].interface[0].altsetting[0].endpoint[0].
+	   wMaxPacketSize,
+	   dev->config[0].interface[0].altsetting[0].endpoint[0].bInterval);
+      return 0;
+    }
+
+  /* Endpoint 1 */
+  if ((dev->config[0].interface[0].altsetting[0].endpoint[1].
+       bEndpointAddress != 0x82)
+      || (dev->config[0].interface[0].altsetting[0].endpoint[1].
+	  bmAttributes != 0x02)
+      || ((dev->config[0].interface[0].altsetting[0].endpoint[1].
+	  wMaxPacketSize != 0x40)
+	  && (dev->config[0].interface[0].altsetting[0].endpoint[1].
+	      wMaxPacketSize != 0x200))
+      || (dev->config[0].interface[0].altsetting[0].endpoint[1].bInterval !=
+	  0x00))
+    {
+      if (verbose > 2)
+	printf
+	  ("    this is not a SQ113 (bEndpointAddress = 0x%x, bmAttributes = 0x%x, "
+	   "wMaxPacketSize = 0x%x, bInterval = 0x%x)\n",
+	   dev->config[0].interface[0].altsetting[0].endpoint[1].
+	   bEndpointAddress,
+	   dev->config[0].interface[0].altsetting[0].endpoint[1].bmAttributes,
+	   dev->config[0].interface[0].altsetting[0].endpoint[1].
+	   wMaxPacketSize,
+	   dev->config[0].interface[0].altsetting[0].endpoint[1].bInterval);
+      return 0;
+    }
+  /* Endpoint 2 */
+  if ((dev->config[0].interface[0].altsetting[0].endpoint[2].
+       bEndpointAddress != 0x83)
+      || (dev->config[0].interface[0].altsetting[0].endpoint[2].
+	  bmAttributes != 0x03)
+      || (dev->config[0].interface[0].altsetting[0].endpoint[2].
+	  wMaxPacketSize != 0x1)
+      || (dev->config[0].interface[0].altsetting[0].endpoint[2].bInterval !=
+	  0x03))
+    {
+      if (verbose > 2)
+	printf
+	  ("    this is not a SQ113 (bEndpointAddress = 0x%x, bmAttributes = 0x%x, "
+	   "wMaxPacketSize = 0x%x, bInterval = 0x%x)\n",
+	   dev->config[0].interface[0].altsetting[0].endpoint[2].
+	   bEndpointAddress,
+	   dev->config[0].interface[0].altsetting[0].endpoint[2].bmAttributes,
+	   dev->config[0].interface[0].altsetting[0].endpoint[2].
+	   wMaxPacketSize,
+	   dev->config[0].interface[0].altsetting[0].endpoint[2].bInterval);
+      return 0;
+    }
+
+  /* Now we read the status register */
+  result = prepare_interface (dev, &handle);
+  if (!result)
+    return "SQ113?";
+  
+  buffer [0] = 0x5f;
+  buffer [1] = 0x00;
+  buffer [2] = 0x5f;
+  buffer [3] = 0x00;
+
+  result = 
+    usb_control_msg (handle, 0x40, 0x01, 0xb0, 0, (char *) buffer, 4, TIMEOUT);
+  if (result < 0)
+    {
+      if (verbose > 2)
+	printf ("    Couldn't set bank (%s)\n",
+		usb_strerror ());
+      finish_interface (handle);
+      return 0;
+    }
+
+  data = 0x00;
+
+  buffer [0] = 0x8b;
+  buffer [1] = data;
+  buffer [2] = 0x8b;
+  buffer [3] = data;
+
+  result = 
+    usb_control_msg (handle, 0x40, 0x01, 0xb0, 0, (char *) buffer, 4, TIMEOUT);
+  if (result < 0)
+    {
+      if (verbose > 2)
+	printf ("    Couldn't write register (%s)\n",
+		usb_strerror ());
+      finish_interface (handle);
+      return 0;
+    }
+
+  buffer [0] = 0x8b;
+  buffer [1] = 0x8b;
+  buffer [2] = 0x8b;
+  buffer [3] = 0x8b;
+  result = 
+    usb_control_msg (handle, 0x40, 0x01, 0x04, 0x8b, (char *) buffer, 4, TIMEOUT);
+  if (result < 0)
+    {
+      if (verbose > 2)
+	printf ("    Couldn't set read register address (%s)\n",
+		usb_strerror ());
+      finish_interface (handle);
+      return 0;
+    }
+
+  result = 
+    usb_control_msg (handle, 0xc0, 0x01, 0x07, 0, (char *) buffer, 4, TIMEOUT);
+  if (result < 0)
+    {
+      if (verbose > 2)
+	printf ("    Couldn't read register (%s)\n",
+		usb_strerror ());
+      finish_interface (handle);
+      return 0;
+    }
+
+  if ((buffer[0] & 0x10) != 0x10)
+    {
+      if (verbose > 2)
+	printf ("    Sensor not home? (0x%02x)\n", buffer[0]);
+      finish_interface (handle);
+      return 0;
+    }
+
+  finish_interface (handle);
+  return "SQ113";
+}
+
+/* Check for Realtek RTS8822L-01H chipset*/
+static char *
+check_rts8822l01h (struct usb_device *dev)
+{
+  char data[2];
+  usb_dev_handle *handle;
+  int result;
+
+  if (verbose > 2)
+    printf ("    checking for RTS8822L-01H ...\n");
+
+  /* Check device descriptor */
+  if (dev->descriptor.bDeviceClass != 0)
+    {
+      if (verbose > 2)
+	printf ("    this is not a RTS8822L-01H (bDeviceClass = %d)\n",
+		dev->descriptor.bDeviceClass);
+      return 0;
+    }
+  if (dev->descriptor.bcdUSB != 0x200)
+    {
+      if (verbose > 2)
+	printf ("    this is not a RTS8822L-01H (bcdUSB = 0x%x)\n",
+		dev->descriptor.bcdUSB);
+      return 0;
+    }
+  if (dev->descriptor.bDeviceSubClass != 0)
+    {
+      if (verbose > 2)
+	printf ("    this is not a RTS8822L-01H (bDeviceSubClass = 0x%x)\n",
+		dev->descriptor.bDeviceSubClass);
+      return 0;
+    }
+  if (dev->descriptor.bDeviceProtocol != 0)
+    {
+      if (verbose > 2)
+	printf ("    this is not a RTS8822L-01H (bDeviceProtocol = 0x%x)\n",
+		dev->descriptor.bDeviceProtocol);
+      return 0;
+    }
+
+  /* Check endpoints */
+  if (dev->config[0].interface[0].altsetting[0].bNumEndpoints != 3)
+    {
+      if (verbose > 2)
+	printf ("    this is not a RTS8822L-01H (bNumEndpoints = %d)\n",
+		dev->config[0].interface[0].altsetting[0].bNumEndpoints);
+      return 0;
+    }
+  if ((dev->config[0].interface[0].altsetting[0].endpoint[0].
+       bEndpointAddress != 0x81)
+      || (dev->config[0].interface[0].altsetting[0].endpoint[0].
+	  bmAttributes != 0x02)
+      || (dev->config[0].interface[0].altsetting[0].endpoint[0].
+	  wMaxPacketSize != 0x200)
+      || (dev->config[0].interface[0].altsetting[0].endpoint[0].bInterval !=
+	  0x00))
+    {
+      if (verbose > 2)
+	printf
+	  ("    this is not a RTS8822L-01H (bEndpointAddress = 0x%x, bmAttributes = 0x%x, "
+	   "wMaxPacketSize = 0x%x, bInterval = 0x%x)\n",
+	   dev->config[0].interface[0].altsetting[0].endpoint[0].
+	   bEndpointAddress,
+	   dev->config[0].interface[0].altsetting[0].endpoint[0].bmAttributes,
+	   dev->config[0].interface[0].altsetting[0].endpoint[0].
+	   wMaxPacketSize,
+	   dev->config[0].interface[0].altsetting[0].endpoint[0].bInterval);
+      return 0;
+    }
+
+  if ((dev->config[0].interface[0].altsetting[0].endpoint[1].
+       bEndpointAddress != 0x02)
+      || (dev->config[0].interface[0].altsetting[0].endpoint[1].
+	  bmAttributes != 0x02)
+      || (dev->config[0].interface[0].altsetting[0].endpoint[1].
+	  wMaxPacketSize != 0x200)
+      || (dev->config[0].interface[0].altsetting[0].endpoint[1].bInterval !=
+	  0x00))
+    {
+      if (verbose > 2)
+	printf
+	  ("    this is not a RTS8822L-01H (bEndpointAddress = 0x%x, bmAttributes = 0x%x, "
+	   "wMaxPacketSize = 0x%x, bInterval = 0x%x)\n",
+	   dev->config[0].interface[0].altsetting[0].endpoint[1].
+	   bEndpointAddress,
+	   dev->config[0].interface[0].altsetting[0].endpoint[1].bmAttributes,
+	   dev->config[0].interface[0].altsetting[0].endpoint[1].
+	   wMaxPacketSize,
+	   dev->config[0].interface[0].altsetting[0].endpoint[1].bInterval);
+      return 0;
+    }
+
+  if ((dev->config[0].interface[0].altsetting[0].endpoint[2].
+       bEndpointAddress != 0x83)
+      || (dev->config[0].interface[0].altsetting[0].endpoint[2].
+	  bmAttributes != 0x03)
+      || (dev->config[0].interface[0].altsetting[0].endpoint[2].
+	  wMaxPacketSize != 0x01)
+      || (dev->config[0].interface[0].altsetting[0].endpoint[2].bInterval !=
+	  0x0c))
+    {
+      if (verbose > 2)
+	printf
+	  ("    this is not a RTS8822L-01H (bEndpointAddress = 0x%x, bmAttributes = 0x%x, "
+	   "wMaxPacketSize = 0x%x, bInterval = 0x%x)\n",
+	   dev->config[0].interface[0].altsetting[0].endpoint[2].
+	   bEndpointAddress,
+	   dev->config[0].interface[0].altsetting[0].endpoint[2].bmAttributes,
+	   dev->config[0].interface[0].altsetting[0].endpoint[2].
+	   wMaxPacketSize,
+	   dev->config[0].interface[0].altsetting[0].endpoint[2].bInterval);
+      return 0;
+    }
+
+  /* Now we read 1 register */
+  result = prepare_interface (dev, &handle);
+  if (!result)
+    return "rts8822L-01H?";
+
+  memset (data, 0, 2);
+
+  result =
+    usb_control_msg(handle, 0xc0, 0x04, 0xfe11, 0x100, data, 0x02, TIMEOUT);
+
+  if (result <= 0)
+    {
+      if (verbose > 2)
+	printf ("    Couldn't send read control message (%s)\n",
+		strerror (errno));
+      finish_interface (handle);
+      return 0;
+    }
+
+  if ((data[0] & 0x01) != 0x01)
+    {
+      if (verbose > 2)
+	printf ("    Unexpected result from register 0xfe11 : 0x%0x%0x\n",
+		data[1], data[0]);
+      finish_interface (handle);
+      return 0;
+    }
+  finish_interface (handle);
+  return "RTS8822L-01H";
+}	/* end of RTS8822L-01H detection */
+
+
+
 char *
 check_usb_chip (struct usb_device *dev, int verbosity, SANE_Bool from_file)
 {
@@ -2111,6 +2942,21 @@ check_usb_chip (struct usb_device *dev, int verbosity, SANE_Bool from_file)
   if (!chip_name)
     chip_name = check_icm532b (dev);
 
+  if (!chip_name)
+    chip_name = check_pv8630_lm9830 (dev);
+
+  if (!chip_name)
+    chip_name = check_m011 (dev);
+
+  if (!chip_name)
+    chip_name = check_rts8822l01h (dev);
+
+  if (!chip_name)
+    chip_name = check_rts8858c (dev);
+
+  if (!chip_name)
+    chip_name = check_sq113 (dev);
+
   if (verbose > 2)
     {
       if (chip_name)
