@@ -1,6 +1,6 @@
 /* sane-find-scanner.c
 
-   Copyright (C) 1997-2003 Oliver Rauch, Henning Meier-Geinitz, and others.
+   Copyright (C) 1997-2005 Oliver Rauch, Henning Meier-Geinitz, and others.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
@@ -30,11 +30,6 @@
 #include <dirent.h>
 #include <errno.h>
 
-#ifdef HAVE_LIBUSB
-#include "usb.h"
-extern char * check_usb_chip (struct usb_device *dev, int verbosity);
-#endif
-
 #if defined (HAVE_WINDOWS_H)
 #include <windows.h>
 #include <ddk/scsi.h>
@@ -45,6 +40,12 @@ extern char * check_usb_chip (struct usb_device *dev, int verbosity);
 #include "../include/sane/sanei_scsi.h"
 #include "../include/sane/sanei_usb.h"
 #include "../include/sane/sanei_pa4s2.h"
+#include "../include/sane/sanei_config.h"
+
+#ifdef HAVE_LIBUSB
+#include "usb.h"
+extern char * check_usb_chip (struct usb_device *dev, int verbosity, SANE_Bool from_file);
+#endif
 
 #ifndef PATH_MAX
 # define PATH_MAX 1024
@@ -92,6 +93,8 @@ usage (char *msg)
   fprintf (stderr, "\t-f: force opening devname as SCSI even if it looks "
 	   "like USB\n");
   fprintf (stderr, "\t-p: enable scannig for parallel port devices\n");
+  fprintf (stderr, "\t-F file: try to detect chipset from given "
+	   "/proc/bus/usb/devices file\n");
   if (msg)
     fprintf (stderr, "\t%s\n", msg);
 }
@@ -382,20 +385,45 @@ static char *
 get_libusb_string_descriptor (struct usb_device *dev, int index)
 {
   usb_dev_handle *handle;
-  char *buffer;
+  char *buffer, short_buffer[2];
   struct usb_string_descriptor *sd;
-  int size = 100;
+  int size = 2;
   int i;
 
   if (!index)
     return 0;
+
+
+  handle = usb_open (dev);
+  if (!handle)
+    return 0;
+
+  sd = (struct usb_string_descriptor *) short_buffer;
+
+  if (usb_control_msg (handle,
+		       USB_ENDPOINT_IN + USB_TYPE_STANDARD + USB_RECIP_DEVICE,
+		       USB_REQ_GET_DESCRIPTOR,
+		       (USB_DT_STRING << 8) + index, 0, short_buffer,
+		       size, 1000) < 0)
+    {
+      usb_close (handle);
+      return 0;
+    }
+
+  if (sd->bLength < 2 
+      || sd->bDescriptorType != USB_DT_STRING)
+    {
+      usb_close (handle);
+      return 0;
+    }
+  
+  size = sd->bLength;
+
   buffer = calloc (1, size + 1);
   if (!buffer)
     return 0;
   sd = (struct usb_string_descriptor *) buffer;
-  handle = usb_open (dev);
-  if (!handle)
-    return 0;
+
   if (usb_control_msg (handle,
 		       USB_ENDPOINT_IN + USB_TYPE_STANDARD + USB_RECIP_DEVICE,
 		       USB_REQ_GET_DESCRIPTOR,
@@ -405,6 +433,7 @@ get_libusb_string_descriptor (struct usb_device *dev, int index)
       usb_close (handle);
       return 0;
     }
+
   if (sd->bLength < 2 || sd->bLength > size
       || sd->bDescriptorType != USB_DT_STRING)
     {
@@ -433,7 +462,7 @@ get_libusb_product (struct usb_device *dev)
 }
 
 static void
-check_libusb_device (struct usb_device *dev)
+check_libusb_device (struct usb_device *dev, SANE_Bool from_file)
 {
   int is_scanner = 0;
   char *vendor = get_libusb_vendor (dev);
@@ -610,7 +639,7 @@ check_libusb_device (struct usb_device *dev)
 
   if (is_scanner > 0)
     {
-      char * chipset = check_usb_chip (dev, verbose);
+      char * chipset = check_usb_chip (dev, verbose, from_file);
 
       printf ("found USB scanner (vendor=0x%04x", dev->descriptor.idVendor);
       if (vendor)
@@ -620,7 +649,10 @@ check_libusb_device (struct usb_device *dev)
 	printf (" [%s]", product);
       if (chipset)
 	printf (", chip=%s", chipset);
-      printf (") at libusb:%s:%s\n", dev->bus->dirname, dev->filename);
+      if (from_file)
+	printf (")\n");
+      else
+	printf (") at libusb:%s:%s\n", dev->bus->dirname, dev->filename);
 
       libusb_device_found = SANE_TRUE;
       device_found = SANE_TRUE;
@@ -783,6 +815,32 @@ static char **build_scsi_dev_list()
 }
 #endif
 
+#if defined (HAVE_IOKIT_CDB_IOSCSILIB_H) || \
+    defined (HAVE_IOKIT_SCSI_COMMANDS_SCSICOMMANDOPERATIONCODES_H)
+char **scsi_dev_list;
+int scsi_dev_list_index;
+
+static SANE_Status AddToSCSIDeviceList (const char *dev) {
+  if (scsi_dev_list_index < 99) {
+    scsi_dev_list [scsi_dev_list_index] = strdup (dev);
+    scsi_dev_list_index++;
+    return SANE_STATUS_GOOD;
+  }
+  else
+    return SANE_STATUS_NO_MEM;
+}
+
+static char **build_scsi_dev_list()
+{
+  scsi_dev_list_index = 0;
+  scsi_dev_list = malloc (100 * sizeof(char *));
+  sanei_scsi_find_devices (NULL, NULL, NULL, -1, -1, -1, -1,
+			   AddToSCSIDeviceList);
+  scsi_dev_list [scsi_dev_list_index] = NULL;
+  return scsi_dev_list;
+}
+#endif
+
 static int
 check_mustek_pp_device (void)
 {
@@ -856,9 +914,209 @@ check_mustek_pp_device (void)
 	   "  # man-page for setup instructions.\n");
 
   return (found > 0 || scsi > 0);
-
 }
 
+#ifdef HAVE_LIBUSB
+static SANE_Bool
+parse_num (char* search, const char* line, int base, long int * number)
+{
+  char* start_number;
+
+  start_number = strstr (line, search);
+  if (start_number == NULL)
+    return SANE_FALSE;
+  start_number += strlen (search);
+ 
+  *number = strtol (start_number, NULL, base);
+  if (verbose > 2)
+    printf ("Found %s%ld\n", search, *number);
+  return SANE_TRUE;
+}
+
+static SANE_Bool
+parse_bcd (char* search, const char* line, long int * number)
+{
+  char* start_number;
+  char* end_number;
+  int first_part;
+  int second_part;
+
+  start_number = strstr (line, search);
+  if (start_number == NULL)
+    return SANE_FALSE;
+  start_number += strlen (search);
+ 
+  first_part = strtol (start_number, &end_number, 10);
+  start_number = end_number + 1; /* skip colon */
+  second_part = strtol (start_number, NULL, 10);
+  *number = ((first_part / 10) << 12) + ((first_part % 10) << 8) 
+    + ((second_part / 10) << 4) + (second_part % 10);
+  if (verbose > 2)
+    printf ("Found %s%ld\n", search, *number);
+  return SANE_TRUE;
+}
+
+static void
+parse_file (char *filename)
+{
+  FILE * parsefile;
+  char line [PATH_MAX], *token;
+  const char * p;
+  struct usb_device *dev = 0;
+  long int number = 0;
+  int current_config = 1;
+  int current_if = -1;
+  int current_as = -1;
+  int current_ep = -1;
+
+  if (verbose > 1)
+    printf ("trying to open %s\n", filename);
+  parsefile = fopen (filename, "r");
+
+  if (parsefile == NULL)
+    {
+      if (verbose > 0)
+	printf ("opening %s failed: %s\n", filename, strerror (errno));
+      return;
+    }
+
+  while (sanei_config_read (line, PATH_MAX, parsefile))
+    {
+      if (verbose > 2)
+	printf ("parsing line: `%s'\n", line);
+      p = sanei_config_get_string (line, &token);
+      if (!token || !p || token[0] == '\0')
+	continue;
+      if (token[1] != ':')
+	{
+	  if (verbose > 2)
+	    printf ("missing `:'?\n");
+	  continue;
+	}
+      switch (token[0])
+	{
+	case 'T':
+	  if (dev)
+	    check_libusb_device (dev, SANE_TRUE);
+	  dev = calloc (1, sizeof (struct usb_device));
+	  dev->bus = calloc (1, sizeof (struct usb_bus));
+	  current_config = 1;
+	  current_if = -1;
+	  current_as = -1;
+	  current_ep = -1;
+	  break;
+	case 'D':
+	  if (parse_bcd ("Ver=", line, &number))
+	    dev->descriptor.bcdUSB = number;
+	  if (parse_num ("Cls=", line, 16, &number))
+	    dev->descriptor.bDeviceClass = number;
+	  if (parse_num ("Sub=", line, 16, &number))
+	    dev->descriptor.bDeviceSubClass = number;
+	  if (parse_num ("Prot=", line, 16, &number))
+	    dev->descriptor.bDeviceProtocol = number;
+	  if (parse_num ("MxPS=", line, 10, &number))
+	    dev->descriptor.bMaxPacketSize0 = number;
+	  if (parse_num ("#Cfgs=", line, 10, &number))
+	    dev->descriptor.bNumConfigurations = number;
+	  dev->config = calloc (number, sizeof (struct usb_config_descriptor));
+	  break;
+	case 'P':
+	  if (parse_num ("Vendor=", line, 16, &number))
+	    dev->descriptor.idVendor = number;
+	  if (parse_num ("ProdID=", line, 16, &number))
+	    dev->descriptor.idProduct = number;
+	  if (parse_bcd ("Rev=", line, &number))
+	    dev->descriptor.bcdDevice = number;
+	  break;
+	case 'C':
+	  current_if = -1;
+	  current_as = -1;
+	  current_ep = -1;
+	  if (parse_num ("Cfg#=", line, 10, &number))
+	    {
+	      current_config = number - 1;
+	      dev->config[current_config].bConfigurationValue = number;
+	    }
+	  if (parse_num ("Ifs=", line, 10, &number))
+	    dev->config[current_config].bNumInterfaces = number;
+	  dev->config[current_config].interface 
+	    = calloc (number, sizeof (struct usb_interface));
+	  if (parse_num ("Atr=", line, 16, &number))
+	    dev->config[current_config].bmAttributes = number;
+	  if (parse_num ("MxPwr=", line, 10, &number))
+	    dev->config[current_config].MaxPower = number / 2;
+	  break;
+	case 'I':
+	  current_ep = -1;
+	  if (parse_num ("If#=", line, 10, &number))
+	    {
+	      if (current_if != number)
+		{
+		  current_if = number;
+		  current_as = -1;
+		  dev->config[current_config].interface[current_if].altsetting
+		    = calloc (20, sizeof (struct usb_interface_descriptor));
+		  /* Can't read number of altsettings */
+		  dev->config[current_config].interface[current_if].num_altsetting = 1;
+		}
+	      else
+		dev->config[current_config].interface[current_if].num_altsetting++;
+	    }
+	  if (parse_num ("Alt=", line, 10, &number))
+	    {
+	      current_as = number;
+	      dev->config[current_config].interface[current_if].altsetting[current_as].bInterfaceNumber 
+		= current_if;
+	      dev->config[current_config].interface[current_if].altsetting[current_as].bAlternateSetting 
+		= current_as;
+	    }
+	  if (parse_num ("#EPs=", line, 10, &number))
+	    dev->config[current_config].interface[current_if].altsetting[current_as].bNumEndpoints 
+	      = number;
+	  dev->config[current_config].interface[current_if].altsetting[current_as].endpoint 
+	    = calloc (number, sizeof (struct usb_endpoint_descriptor));
+	  if (parse_num ("Cls=", line, 16, &number))
+	    dev->config[current_config].interface[current_if].altsetting[current_as].bInterfaceClass 
+	      = number;
+	  if (parse_num ("Sub=", line, 16, &number))
+	    dev->config[current_config].interface[current_if].altsetting[current_as].bInterfaceSubClass 
+	      = number;
+	  if (parse_num ("Prot=", line, 16, &number))
+	    dev->config[current_config].interface[current_if].altsetting[current_as].bInterfaceProtocol 
+	      = number;
+	  break;
+	case 'E':
+	  current_ep++;
+	  if (parse_num ("Ad=", line, 16, &number))
+	    dev->config[current_config].interface[current_if].altsetting[current_as]
+	      .endpoint[current_ep].bEndpointAddress = number;
+	  if (parse_num ("Atr=", line, 16, &number))
+	    dev->config[current_config].interface[current_if].altsetting[current_as]
+	      .endpoint[current_ep].bmAttributes = number;
+	  if (parse_num ("MxPS=", line, 10, &number))
+	    dev->config[current_config].interface[current_if].altsetting[current_as]
+	      .endpoint[current_ep].wMaxPacketSize = number;
+	  if (parse_num ("Ivl=", line, 10, &number))
+	    dev->config[current_config].interface[current_if].altsetting[current_as]
+	      .endpoint[current_ep].bInterval = number;
+	  break;
+	case 'S':
+	case 'B':
+	  continue;
+	default:
+	  if (verbose > 1)
+	    printf ("ignoring unknown line identifier: %c\n", token[0]);
+	  continue;
+	}
+      free (token);
+    }
+  if (dev)
+    check_libusb_device (dev, SANE_TRUE);
+  fclose (parsefile);
+  return;
+}
+#endif
+
 int
 main (int argc, char **argv)
 {
@@ -898,6 +1156,14 @@ main (int argc, char **argv)
 	  enable_pp_checks = SANE_TRUE;
 	  break;
 
+	case 'F':
+#ifdef HAVE_LIBUSB
+	  parse_file ((char *) (*(++ap)));
+#else
+	  printf ("libusb not available: option -F can't be used\n");
+#endif
+	  exit (0);
+
 	default:
 	  printf ("unknown option: -%c, try -h for help\n", (*ap)[1]);
 	  exit (0);
@@ -1135,6 +1401,16 @@ main (int argc, char **argv)
 	"/dev/uscanner10", "/dev/uscanner11",
 	"/dev/uscanner12", "/dev/uscanner13",
 	"/dev/uscanner14", "/dev/uscanner15",
+#elif defined(__NetBSD__)
+	"/dev/uscanner",
+	"/dev/uscanner0", "/dev/uscanner1",
+	"/dev/uscanner2", "/dev/uscanner3",
+	"/dev/uscanner4", "/dev/uscanner5",
+	"/dev/uscanner6", "/dev/uscanner7",
+	"/dev/uscanner8", "/dev/uscanner9",
+	"/dev/uscanner10", "/dev/uscanner11",
+	"/dev/uscanner12", "/dev/uscanner13",
+	"/dev/uscanner14", "/dev/uscanner15",
 #elif defined(__OpenBSD__)
 	"/dev/uscanner",
 	"/dev/uscanner0", "/dev/uscanner1",
@@ -1149,18 +1425,25 @@ main (int argc, char **argv)
 	0
       };
 
-#if defined (HAVE_WINDOWS_H)
+#if defined (HAVE_WINDOWS_H) || \
+    defined (HAVE_IOKIT_CDB_IOSCSILIB_H) || \
+    defined (HAVE_IOKIT_SCSI_COMMANDS_SCSICOMMANDOPERATIONCODES_H)
    /* Build a list of valid of possible scanners found */
-   dev_list = build_scsi_dev_list();
+      dev_list = build_scsi_dev_list();
 #else
       dev_list = default_dev_list;
 #endif
 
       usb_dev_list = usb_default_dev_list;
     }
+  if (verbose > 1)
+    printf ("This is sane-find-scanner from %s\n", PACKAGE_STRING);
 
   if (verbose > 0)
-    printf ("\n");
+    printf ("\n  # sane-find-scanner will now attempt to detect your scanner. If the"
+	    "\n  # result is different from what you expected, first make sure your"
+	    "\n  # scanner is powered up and properly connected to your computer.\n\n");
+
   if (verbose > 1)
     printf ("searching for SCSI scanners:\n");
 
@@ -1201,7 +1484,7 @@ main (int argc, char **argv)
       if (verbose > 0)
 	printf
 	  ("  # No SCSI scanners found. If you expected something different, "
-	   "make sure that\n  # you have loaded a SCSI driver for your SCSI "
+	   "make sure that\n  # you have loaded a kernel SCSI driver for your SCSI "
 	   "adapter.\n");
       if (!check_sg ())
 	{
@@ -1263,11 +1546,14 @@ main (int argc, char **argv)
 	  {
 	    for (dev = bus->devices; dev; dev = dev->next)
 	      {
-		check_libusb_device (dev);
+		check_libusb_device (dev, SANE_FALSE);
 	      }			/* for (dev) */
 	  }			/* for (bus) */
       }				/* if (usb_dev_list == ap) */
   }
+#else
+  if (verbose > 1)
+    printf ("libusb not available\n");
 #endif /* HAVE_LIBUSB */
 
   if (device_found)
@@ -1298,8 +1584,15 @@ main (int argc, char **argv)
       if (verbose > 0)
 	printf
 	  ("  # No USB scanners found. If you expected something different, "
-	   "make sure that\n  # you have loaded a driver for your USB host "
-	   "controller and have installed a\n  # kernel scanner module.\n");
+	   "make sure that\n  # you have loaded a kernel driver for your USB host "
+	   "controller and have setup\n  # the USB system correctly. "
+	   "See man sane-usb for details.\n");
+#ifndef HAVE_LIBUSB
+      if (verbose > 0)
+	printf ("  # SANE has been built without libusb support. This may be a "
+		"reason\n  # for not detecting USB scanners. Read README for "
+		"more details.\n");
+#endif
     }
   if (enable_pp_checks == SANE_TRUE) 
     {
